package refactoringtestplugin.changereturntype;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.structure.ChangeSignatureRefactoring;
import org.eclipse.ltk.core.refactoring.Refactoring;

import refactoringtestplugin.RefactoringTest;

public abstract class ChangeReturnTypeTest extends RefactoringTest {

	private static final String TARGET_METHOD_NAME = "m";
	
	@Override
	protected String getLogDirPath() {
		return getSystemTempDir() + "/changereturntype";
	}
	
	@Override
	protected Refactoring getRefactoring(ICompilationUnit icu) throws JavaModelException {
		ChangeSignatureRefactoring refactoring = new ChangeSignatureRefactoring(findTargetMethod(icu));
		String returnType = refactoring.getReturnTypeString();
		String newReturnType;
		if (returnType.equals("void")) {
			// add a return type if none exists
			newReturnType = "Object";
		}
		else if (returnType.equals("Object")){
			// remove return type 
			newReturnType = "void";
		}
		else if (returnType.equals("Integer")){
			// narrow return type
			newReturnType = "Byte";
		}
		else {
			// generalize return type
			newReturnType = "Object";
		}
		refactoring.setNewReturnTypeName(newReturnType);
		return refactoring;
	}

	private IMethod findTargetMethod(ICompilationUnit icu) throws JavaModelException {
		// this only works because the generator will always put the target
		// method first in the class, even if there is another method
		// with the same name that references it.
		for (IType type : icu.getTypes()) {
			for (IMethod method : type.getMethods()) {
				if (method.getElementName().equals(TARGET_METHOD_NAME)) {
					return method;
				}
			}
		}
		throw new RuntimeException("Method " + TARGET_METHOD_NAME + " was not found"); // TODO: better exception

	}

	
}
