package refactoringtestplugin.commongenerators;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.basic.CompilationUnitGenerator;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.GetterSetterGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.MultipleFieldDeclarationGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationFragmentGenerator;
import edu.uiuc.astgen.fieldreferences.DualClass2FieldsChildDeclarationGenerator;
import edu.uiuc.astgen.fieldreferences.DualClass2FieldsParentDeclarationGenerator;
import edu.uiuc.astgen.fieldreferences.DualClassFieldGetterSetter;
import edu.uiuc.astgen.fieldreferences.DualClassFieldReferenceGenerator;
import edu.uiuc.astgen.fieldreferences.FieldReferenceType;
import edu.uiuc.astgen.fieldreferences.SingleClass2FieldsOptInitializedGenerator;
import edu.uiuc.astgen.fieldreferences.SingleClassFieldReferenceGenerator;
import edu.uiuc.astgen.fieldreferences.TripleClass2FieldsChildDeclarationGenerator;
import org.testorrery.Chain;
import org.testorrery.IGenerator;

public class FieldReferences {

	private static AST ast = ASTNodeGeneratorBase.ast();

	public static IGenerator<CompilationUnit> singleClassFieldReference(String fieldName) {
		final FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(fieldName);
		fieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newSimpleType(ast.newSimpleName("Byte")),
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean")),
				ast.newSimpleType(ast.newSimpleName("Object"))));
		
		SingleClassFieldReferenceGenerator typeDeclGen = new SingleClassFieldReferenceGenerator("A", fieldDeclGen);
		
		CompilationUnitGenerator cuGen = new CompilationUnitGenerator();
		cuGen.addDeclarationGenerator(typeDeclGen);
		return cuGen;
	}
	
	public static IGenerator<CompilationUnit> dualClassFieldReference(String fieldName) {
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(fieldName);
		fieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean")),
				ast.newSimpleType(ast.newSimpleName("Object"))));
		return new DualClassFieldReferenceGenerator(fieldDeclGen);
	}
	
	// Kely added these methods 
	public static IGenerator<CompilationUnit> singleClass2FieldsOptInitialized(String className, String fieldName1, String fieldName2) {
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName(className));
		IGenerator<FieldReferenceType> fieldRefTypeGen = new Chain<FieldReferenceType>(
				FieldReferenceType.NEW_INSTANCE, 
				FieldReferenceType.QUALIFIED_THIS, 
				FieldReferenceType.THIS,
				FieldReferenceType.UNQUALIFIED,
				FieldReferenceType.SUPER);		
		
		MultipleFieldDeclarationGenerator multipleFieldDeclGen = new MultipleFieldDeclarationGenerator(fieldName1, fieldName2, typeDecl, true);
		multipleFieldDeclGen.setFieldRefTypeGen(fieldRefTypeGen);
		multipleFieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newPrimitiveType(PrimitiveType.INT),
				ast.newPrimitiveType(PrimitiveType.BOOLEAN),
				ast.newSimpleType(ast.newSimpleName("String")),
				ast.newSimpleType(ast.newSimpleName("Byte")),
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean"))
								));	
		SingleClass2FieldsOptInitializedGenerator typeDeclGen = new SingleClass2FieldsOptInitializedGenerator(className, multipleFieldDeclGen);
		
		CompilationUnitGenerator cuGen = new CompilationUnitGenerator();
		cuGen.addDeclarationGenerator(typeDeclGen);
		return cuGen;		
	}
	
	public static IGenerator<CompilationUnit> dualClass2FieldsParentDeclaration(String classNameA, String classNameB, String fieldName1, String fieldName2) {
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName(classNameA));
		IGenerator<FieldReferenceType> fieldRefTypeGen = new Chain<FieldReferenceType>(
				FieldReferenceType.NEW_INSTANCE, 
				FieldReferenceType.QUALIFIED_THIS, 
				FieldReferenceType.THIS,
				FieldReferenceType.UNQUALIFIED,
				FieldReferenceType.SUPER);		
		
		MultipleFieldDeclarationGenerator multipleFieldDeclGen = new MultipleFieldDeclarationGenerator(fieldName1, fieldName2, typeDecl, false);
		multipleFieldDeclGen.setFieldRefTypeGen(fieldRefTypeGen);
		multipleFieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newPrimitiveType(PrimitiveType.INT),
				ast.newPrimitiveType(PrimitiveType.BOOLEAN),
				ast.newSimpleType(ast.newSimpleName("String")),
				ast.newSimpleType(ast.newSimpleName("Byte")),
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean"))
				));				
		
		DualClass2FieldsParentDeclarationGenerator cuGen = new DualClass2FieldsParentDeclarationGenerator(classNameA, classNameB, multipleFieldDeclGen);
		return cuGen;		
	}
	
	public static IGenerator<CompilationUnit> dualClass2FieldsChildDeclaration(String classNameA, String classNameB, String fieldName1, String fieldName2) {
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName(classNameA));
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.INT,
				PrimitiveType.LONG);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator(fieldName2));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);
	
		fieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				//ast.newPrimitiveType(PrimitiveType.INT),
				//ast.newPrimitiveType(PrimitiveType.BOOLEAN),
				//ast.newSimpleType(ast.newSimpleName("String")),
				//ast.newSimpleType(ast.newSimpleName("Byte")),
				//ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean"))));		

		DualClass2FieldsChildDeclarationGenerator cuGen = new DualClass2FieldsChildDeclarationGenerator(fieldDeclGen, classNameA, classNameB, fieldName1);
		
		return cuGen;		
	}		
	
	public static IGenerator<CompilationUnit> dualClassGetterSetter(String classNameA, String classNameB, String fieldName) {
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName(classNameA));		
		
		ModifierGenerator visibilityModifierGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, Modifier.ModifierKeyword.PRIVATE_KEYWORD,
				Modifier.ModifierKeyword.PROTECTED_KEYWORD, (Modifier.ModifierKeyword)null);		
		
		GetterSetterGenerator getSetGen = new GetterSetterGenerator(fieldName);
		getSetGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newPrimitiveType(PrimitiveType.INT),
				ast.newPrimitiveType(PrimitiveType.BOOLEAN),
				ast.newSimpleType(ast.newName("String"))));
		getSetGen.setVisibilityModifierGen(visibilityModifierGen);

		ModifierGenerator fieldVisibilityModifierGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, Modifier.ModifierKeyword.PRIVATE_KEYWORD,
				Modifier.ModifierKeyword.PROTECTED_KEYWORD, (Modifier.ModifierKeyword)null);		
		
		DualClassFieldGetterSetter cuGen = new DualClassFieldGetterSetter(classNameA, classNameB, fieldName);
		cuGen.setGetterSetterGen(getSetGen);
		cuGen.setFieldVisibilityModifierGen(fieldVisibilityModifierGen);
		return cuGen;		
	}
	
	public static IGenerator<CompilationUnit> tripleClass2FieldsChildDeclaration(String classNameA, String classNameB, String classNameC, String fieldName1, String fieldName2) {
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName(classNameA));
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN);
				
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator(fieldName2));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);

		TripleClass2FieldsChildDeclarationGenerator cuGen = new TripleClass2FieldsChildDeclarationGenerator(fieldDeclGen, classNameA, classNameB, classNameC, fieldName1);
		
		return cuGen;		
	}		
	
}
