package refactoringtestplugin.encapsulatefield;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.CompilationUnit;

import org.testorrery.IGenerator;
import org.testorrery.Chain;

public class Demo extends EncapsulateFieldTest {

	@Override
	public String getDescription() {
		return "Four samples in which a single class declares field and references it in some expression.";
	}
	
	@Override
	protected String getLogDirPath() {
		return super.getLogDirPath() + "/demo";
	}
	
	@Override
	protected IGenerator<CompilationUnit> getCUGen() {
		String[] sources = new String[] {
			// refactoring completes normally
			String.format("package p; public class A {int %s;}}", FIELD_TO_REFACTOR),		
			// refactoring completes but results in compilation error
			String.format("package p; public class A {int %s; void m() {(%1$s)++;}}", FIELD_TO_REFACTOR), 
			// parsable but not compilable
			String.format("package p; public class A {intt %s;}", FIELD_TO_REFACTOR),  
			// refactoring not valid (already has getfield method)			
			String.format("package p; public class A {int %1$s; int get%2$s() {return %1$s;}}", FIELD_TO_REFACTOR, Character.toUpperCase(FIELD_TO_REFACTOR.charAt(0)) + FIELD_TO_REFACTOR.substring(1)),
			// refactoring does not encapsulate all occurrences
			String.format("package p; public class A {int %1$s; void m() {%1$s = %1$s;}}", FIELD_TO_REFACTOR),
			// refactoring error on type of field
			String.format("package p; public class A {byte %1$s; void m() {%1$s++;}}", FIELD_TO_REFACTOR)
		};
			
		ASTParser parser = ASTParser.newParser(AST.JLS3);
		Chain<CompilationUnit> cuGen = new Chain<CompilationUnit>();
		for (String source : sources) {
			parser.setSource(source.toCharArray());
			parser.setKind(ASTParser.K_COMPILATION_UNIT);
			cuGen.add((CompilationUnit)parser.createAST(new NullProgressMonitor()));
		}
		return cuGen;	
	}
}
