package refactoringtestplugin.encapsulatefield;

import java.util.Arrays;
import java.util.Collection;
import java.util.regex.Pattern;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.internal.corext.refactoring.sef.SelfEncapsulateFieldRefactoring;
import org.eclipse.ltk.core.refactoring.Refactoring;

import refactoringtestplugin.ProblemReport;
import refactoringtestplugin.RefactoringTest;


public abstract class EncapsulateFieldTest extends RefactoringTest {
	protected static final String FIELD_TO_REFACTOR = "theField";
	protected static final String CLASS_NAME = "A";
	
	@Override
	protected String getLogDirPath() {
		return getSystemTempDir() + "/encapsulatefield";
	}
	
	@Override
	protected Refactoring getRefactoring(ICompilationUnit icu) throws JavaModelException {
		IField field = findIField(icu, FIELD_TO_REFACTOR);			
		SelfEncapsulateFieldRefactoring refactoring = new SelfEncapsulateFieldRefactoring(field);
		String name = field.getElementName();
		name = Character.toUpperCase(name.charAt(0)) + name.substring(1);
		refactoring.setGetterName("get" + name);
		refactoring.setSetterName("set" + name);
		refactoring.setVisibility(Flags.AccPublic);
		
		return refactoring;
	}
	
	private IField findIField(ICompilationUnit icu, String fieldName) throws JavaModelException{
		for (IType type : icu.getTypes()) {
			for (IField field : type.getFields()) {
				if (field.getElementName().equals(fieldName)) {
					return field;
				}
			}
		}
		throw new RuntimeException("Field " + fieldName + " was not found"); // TODO: better exception
	}
	
	@Override
	protected ProblemReport checkCustomPostconditions(ICompilationUnit icu) throws Exception {
		String source = icu.getSource();
		int count = 0;
		int position = 0;
		while(true) { 
			position = source.indexOf(FIELD_TO_REFACTOR, position);	
			if (position >= 0) {
				count++;
				position++;
			}
			else {
				break;
			}
		}
		if (count > 5) {
			return new ProblemReport("A field was not encapsulated");
		}		
		return super.checkCustomPostconditions(icu);
	}
}
