package refactoringtestplugin.innertotop;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.basic.ClassGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.classrelationships.InheritanceType;
import edu.uiuc.astgen.classrelationships.LocationType;
import edu.uiuc.astgen.classrelationships.ReferenceType;
import edu.uiuc.astgen.classrelationships.RelatedPair;
import edu.uiuc.astgen.classrelationships.RelatedPairGenerator;
import edu.uiuc.astgen.classrelationships.RelationshipSpecificationGenerator;
import org.testorrery.ConvertingGenerator;
import org.testorrery.IGenerator;

public class ClassRelationships extends InnerToTopTest {

	@Override
	public String getDescription() {
		return "Two classes, A and B. B is an inner class of A. A and B can both reference each other.";
	}
	
	@Override
	protected String getLogDirPath() {
		return super.getLogDirPath() + "/classrelationships";
	}
	
	@Override
	protected IGenerator<CompilationUnit> getCUGen() {
		RelationshipSpecificationGenerator crbg = new RelationshipSpecificationGenerator();		
		crbg.setInheritance(InheritanceType.NO_INHERITANCE, InheritanceType.SUBCLASS);
		crbg.setLocation(LocationType.INNER);
		crbg.setReference(ReferenceType.values()); // all reference types
		
		crbg.setEnableReverseInheritance(false); // B can inherit from A
		crbg.setEnableReverseLocation(false); // B always inside A
		crbg.setEnableReverseReference(true); // both can reference each other 
		
		ClassGenerator classAGen = new ClassGenerator("A");
		classAGen.setClassModifierGen(new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD));
		ClassGenerator classBGen = new ClassGenerator("B");
				
		RelatedPairGenerator crg = new RelatedPairGenerator();
		crg.setClassAGen(classAGen);
		crg.setClassBGen(classBGen);
		crg.setBuilderGen(crbg);
		
		return new ConvertingGenerator<RelatedPair, CompilationUnit>(crg) {
			@Override
			protected CompilationUnit convert(RelatedPair relatedPair) {
				AST ast = ASTNodeGeneratorBase.ast();
				TypeDeclaration classA = relatedPair.getClassA(); 
				TypeDeclaration classB = relatedPair.getClassB();
				
				CompilationUnit cu = ast.newCompilationUnit();
				cu.types().add(classA);
				if (classB.getParent() == null) {
					cu.types().add(classB);
				}
				return cu;
			}
		};
	}

}
