package refactoringtestplugin.pullupfield;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.internal.corext.refactoring.structure.PullUpRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.structure.PullUpRefactoringProcessor;
import org.eclipse.jdt.internal.corext.refactoring.structure.PushDownRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.structure.PushDownRefactoringProcessor;
import org.eclipse.jdt.internal.ui.preferences.JavaPreferencesSettings;
import org.eclipse.ltk.core.refactoring.Refactoring;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;

import org.testorrery.IGenerator;
import refactoringtestplugin.RefactoringTest;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

public abstract class PullUpFieldTest extends RefactoringTest {

	protected static final String FIELD_TO_REFACTOR = "g";
	protected static final String CLASS_NAME = "B";
	
	@Override
	protected String getLogDirPath() {
		return super.getSystemTempDir() + "/pullupfield";
	}

	@Override
	protected Refactoring getRefactoring(ICompilationUnit icu) throws JavaModelException {
		// Under Development 4/16/07
		//throw new NotImplementedException();
		final IMember[] members = findMembers(icu);
		PullUpRefactoringProcessor processor = 
			new PullUpRefactoringProcessor(
					members, JavaPreferencesSettings.getCodeGenerationSettings(members[0].getJavaProject()));
				//	JavaPreferencesSettings.getCodeGenerationSettings(getProject()));
		PullUpRefactoring refactoring  = new PullUpRefactoring(processor);
        setSuperclassAsTargetClass(refactoring);
		return refactoring;
	}

	/* Copied from org.eclipse.jdt.ui.tests.refactoring.PullUpTests. */
	private IType[] getPossibleTargetClasses(PullUpRefactoring ref) throws JavaModelException{
		return ref.getPullUpProcessor().getCandidateTypes(new RefactoringStatus(), new NullProgressMonitor());
	}
	
	private void setSuperclassAsTargetClass(PullUpRefactoring ref) throws JavaModelException {
		IType[] possibleClasses= getPossibleTargetClasses(ref);
		ref.getPullUpProcessor().setDestinationType(possibleClasses[possibleClasses.length - 1]);
	}
    /* End copy. */

	private IMember[] findMembers(ICompilationUnit icu) throws JavaModelException {
		for (IType type : icu.getTypes()) {
			for (IField field : type.getFields()) {
				if (field.getElementName().equals(FIELD_TO_REFACTOR)) {
					return new IMember[] {field};
				}
			}
		}
		// Kely add this part
		for (IType type : icu.getTypes()) {			
			IType[] listOfTypes = type.getTypes();
			for (IType subType : listOfTypes) {
				for (IField field : subType.getFields()) {
					if (field.getElementName().equals(FIELD_TO_REFACTOR)) {
						return new IMember[] {field};
					}
				}				
			}
		}	
		
		for (IType type : icu.getTypes()) {
			IMethod[] listOfMethods = type.getMethods();
			for (IMethod method : listOfMethods){
				IType localType = method.getType("B", 1);
				for (IField field : localType.getFields()) {
					if (field.getElementName().equals(FIELD_TO_REFACTOR)) {
						return new IMember[] {field};
					}
				}								
			}
		}
		throw new RuntimeException("Field " + FIELD_TO_REFACTOR + " was not found"); // TODO: better exception
	}

}
