package refactoringtestplugin.pullupmethod;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.internal.corext.refactoring.structure.PullUpRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.structure.PullUpRefactoringProcessor;
import org.eclipse.jdt.internal.corext.refactoring.structure.PushDownRefactoring;
import org.eclipse.jdt.internal.corext.refactoring.structure.PushDownRefactoringProcessor;
import org.eclipse.jdt.internal.corext.util.JavaModelUtil;
import org.eclipse.jdt.internal.ui.preferences.JavaPreferencesSettings;
import org.eclipse.ltk.core.refactoring.Refactoring;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;

import org.testorrery.IGenerator;
import refactoringtestplugin.RefactoringTest;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

public abstract class PullUpMethodTest extends RefactoringTest {

	protected static final String METHOD_TO_REFACTOR = "m";
	protected static final String CLASS_NAME = "B";
	
	@Override
	protected String getLogDirPath() {
		return super.getSystemTempDir() + "/pullupmethod";
	}

	@Override
	protected Refactoring getRefactoring(ICompilationUnit icu) throws JavaModelException {
		final IMember[] members = findMembers(icu);
		PullUpRefactoringProcessor processor = 
			new PullUpRefactoringProcessor(
					members, JavaPreferencesSettings.getCodeGenerationSettings(members[0].getJavaProject()));
				//	JavaPreferencesSettings.getCodeGenerationSettings(getProject()));
		IType type= getType(icu, CLASS_NAME);
		IMethod[] methods = getMethods(type, new String[] {METHOD_TO_REFACTOR} , new String[][]{new String[0]});
		processor.setDeletedMethods(methods);
		PullUpRefactoring refactoring  = new PullUpRefactoring(processor);
        setSuperclassAsTargetClass(refactoring);
		return refactoring;
	}

	/* Copied from org.eclipse.jdt.ui.tests.refactoring.PullUpTests. */
	private IType[] getPossibleTargetClasses(PullUpRefactoring ref) throws JavaModelException{
		return ref.getPullUpProcessor().getCandidateTypes(new RefactoringStatus(), new NullProgressMonitor());
	}
	
	private void setSuperclassAsTargetClass(PullUpRefactoring ref) throws JavaModelException {
		IType[] possibleClasses= getPossibleTargetClasses(ref);
		ref.getPullUpProcessor().setDestinationType(possibleClasses[possibleClasses.length - 1]);
	}
    /* End copy. */

	private IMember[] findMembers(ICompilationUnit icu) throws JavaModelException {
		for (IType type : icu.getTypes()) {
			for (IMethod method : type.getMethods()) {
				if (method.getElementName().equals(METHOD_TO_REFACTOR)) {
					return new IMember[] {method};
				}
			}
		}
		// Kely add this part
		for (IType type : icu.getTypes()) {			
			IType[] listOfTypes = type.getTypes();
			for (IType subType : listOfTypes) {
				for (IMethod method : subType.getMethods()) {
					if (method.getElementName().equals(METHOD_TO_REFACTOR)) {
						return new IMember[] {method};
					}
				}				
			}
		}	
		
		for (IType type : icu.getTypes()) {
			IMethod[] listOfMethods = type.getMethods();
			for (IMethod method : listOfMethods){
				IType localType = method.getType(CLASS_NAME, 1);
				for (IMethod subMethod : localType.getMethods()) {
					if (subMethod.getElementName().equals(METHOD_TO_REFACTOR)) {
						return new IMember[] {subMethod};
					}
				}								
			}
		}
		throw new RuntimeException("Method " + METHOD_TO_REFACTOR + " was not found"); // TODO: better exception
	}
	
	public static IMethod[] getMethods(IType type, String[] names, String[][] signatures) throws JavaModelException{
		if (names == null || signatures == null)
			return new IMethod[0];
		List methods= new ArrayList(names.length);
		for (int i = 0; i < names.length; i++) {
			IMethod method= type.getMethod(names[i], signatures[i]);
			//assertTrue("method " + method.getElementName() + " does not exist", method.exists());
			if (!methods.contains(method))
				methods.add(method);
		}
		return (IMethod[]) methods.toArray(new IMethod[methods.size()]);	
	}
	
	protected IType getType(ICompilationUnit cu, String name) throws JavaModelException {
		IType[] types= cu.getAllTypes();
		for (int i= 0; i < types.length; i++)
			if (JavaModelUtil.getTypeQualifiedName(types[i]).equals(name) ||
			    types[i].getElementName().equals(name))
				return types[i];
		
		for (int i= 0; i < types.length; i++){
			IMethod method = types[i].getMethod("dummyMethod_A", new String[] {});
			IType localType = method.getType(name, 1);
			if (localType != null) return localType;
//			IType subType = types[i].getType(name);
//			if (subType != null) return subType;			
		}		
			
		return null;
	}	

}
