package refactoringtestplugin.removeparameter;

import java.util.List;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.corext.refactoring.ParameterInfo;
import org.eclipse.jdt.internal.corext.refactoring.structure.ChangeSignatureRefactoring;
import org.eclipse.ltk.core.refactoring.Refactoring;

import refactoringtestplugin.RefactoringTest;

public abstract class RemoveParameterTest extends RefactoringTest {

	protected static final String TARGET_METHOD_NAME = "m";
	protected static final String TARGET_PARAMETER_NAME = "a";
	
	@Override
	protected String getLogDirPath() {
		return getSystemTempDir() + "/removeparameter";
	}

	@Override
	protected Refactoring getRefactoring(ICompilationUnit icu) throws JavaModelException {
		ChangeSignatureRefactoring refactoring = new ChangeSignatureRefactoring(findTargetMethod(icu));
		List parameterInfos = refactoring.getParameterInfos();
		for (Object object : parameterInfos) {
			ParameterInfo param = (ParameterInfo) object;
			param.markAsDeleted();
		}
		return refactoring;
	}

	private IMethod findTargetMethod(ICompilationUnit icu) throws JavaModelException {
		// this only works because the generator will always put the target
		// method first in the class, even if there is another method
		// with the same name that references it.
		for (IType type : icu.getTypes()) {
			for (IMethod method : type.getMethods()) {
				if (method.getElementName().equals(TARGET_METHOD_NAME)) {
					return method;
				}
			}
		}
		throw new RuntimeException("Method " + TARGET_METHOD_NAME + " was not found"); // TODO: better exception

	}


}
