package refactoringtestplugin.renamefield;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.CompilationUnit;

import refactoringtestplugin.ProblemReport;

import org.testorrery.IGenerator;
import org.testorrery.Chain;

public class Demo extends RenameFieldTest {

	@Override
	public String getDescription() {
		return "Simple inputs that demonstrate the behavior of the rename field refactoring";
	}
	
	@Override
	protected String getLogDirPath() {
		return super.getLogDirPath() + "/demo";
	}
	
	@Override
	protected IGenerator<CompilationUnit> getCUGen() {
		String[] sources = new String[] {
			// refactoring completes normally
			String.format("package p; public class A {int %s;}}", FIELD_TO_REFACTOR),	
			// refactoring cannot complete because field already exists
			String.format("package p; public class A {int %s; int %s;}}", FIELD_TO_REFACTOR, NEW_FIELD_NAME),
			// compilation unit does not compile
			String.format("package p; public class A {intt %s;}}", FIELD_TO_REFACTOR),
			// fails on a custom postcondition check
			String.format("package p; public class FAILURE {int %s;}}", FIELD_TO_REFACTOR)	
		};
			
		ASTParser parser = ASTParser.newParser(AST.JLS3);
		Chain<CompilationUnit> cuGen = new Chain<CompilationUnit>();
		for (String source : sources) {
			parser.setSource(source.toCharArray());
			parser.setKind(ASTParser.K_COMPILATION_UNIT);
			cuGen.add((CompilationUnit)parser.createAST(new NullProgressMonitor()));
		}
		return cuGen;
	}
	
	@Override
	protected ProblemReport checkCustomPostconditions(ICompilationUnit icu) throws Exception {
		if (icu.getElementName().equals("FAILURE.java")) {
			return new ProblemReport("DUMMY FAILURE!");
		}
		return super.checkCustomPostconditions(icu);
	}
}
