package refactoringtestplugin.renamefield;

import java.util.Arrays;
import java.util.Collection;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.internal.corext.refactoring.rename.RenameFieldProcessor;
import org.eclipse.ltk.core.refactoring.Refactoring;
import org.eclipse.ltk.core.refactoring.participants.RenameRefactoring;

import refactoringtestplugin.ProblemReport;
import refactoringtestplugin.RefactoringTest;

public abstract class RenameFieldTest extends RefactoringTest {
	protected static final String FIELD_TO_REFACTOR = "theField";
	protected static final String NEW_FIELD_NAME = "theNewField";
	
	/**
	 * Pre-refactoring compilation unit used to check reverse refactoring
	 */
	private String originalCUSource;
	
	@Override
	protected String getLogDirPath() {
		return getSystemTempDir() + "/renamefield";
	}
	
	@Override
	protected Refactoring getRefactoring(ICompilationUnit icu) throws JavaModelException {
		IField field = findIField(icu, FIELD_TO_REFACTOR);		
		RenameFieldProcessor processor = new RenameFieldProcessor(field);
		RenameRefactoring refactoring = new RenameRefactoring(processor);
		processor.setNewElementName(NEW_FIELD_NAME);
		return refactoring;
	}
	
	private IField findIField(ICompilationUnit icu, String fieldName) throws JavaModelException{
		for (IType type : icu.getTypes()) {
			for (IField field : type.getFields()) {
				if (field.getElementName().equals(fieldName)) {
					return field;
				}
			}
		}
		throw new RuntimeException("Field " + fieldName + " was not found"); // TODO: better exception
	}
	
	@Override
	protected ICompilationUnit createICompilationUnit(IPackageFragmentRoot packFragRoot, CompilationUnit cu, String cuName) throws JavaModelException {
		ICompilationUnit originalCU = super.createICompilationUnit(packFragRoot, cu, cuName);
		originalCUSource = originalCU.getSource();
		return originalCU;
	}
	
	@Override
	protected ProblemReport checkCustomPostconditions(ICompilationUnit icu) throws Exception {
		performReverseRefactoring(icu);
		if (!icu.getSource().equals(originalCUSource)) {
			return new ProblemReport("Reverse rename refactoring does not yield original compilation unit");
		}
		return new ProblemReport();
	}

	private void performReverseRefactoring(ICompilationUnit icu) throws Exception {
		IField field = findIField(icu, NEW_FIELD_NAME);		
		RenameFieldProcessor processor = new RenameFieldProcessor(field);
		RenameRefactoring refactoring = new RenameRefactoring(processor);
		processor.setNewElementName(FIELD_TO_REFACTOR);
		performRefactoring(refactoring);			
	}
}
