package refactoringtestplugin.renamemethod;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.internal.corext.refactoring.rename.RenameMethodProcessor;
import org.eclipse.jdt.internal.corext.refactoring.rename.RenameNonVirtualMethodProcessor;
import org.eclipse.ltk.core.refactoring.Refactoring;
import org.eclipse.ltk.core.refactoring.participants.RenameRefactoring;

import org.testorrery.IGenerator;
import org.testorrery.Literal;
import refactoringtestplugin.ProblemReport;
import refactoringtestplugin.RefactoringTest;

public abstract class RenameMethodTest extends RefactoringTest {

	protected static final String METHOD_TO_RENAME = "m";
	protected static final String NEW_METHOD_NAME = "n";
	
	@Override
	protected String getLogDirPath() {
		return super.getSystemTempDir() + "/renamemethod";
	}

	@Override
	protected Refactoring getRefactoring(ICompilationUnit icu) throws JavaModelException {
		RenameMethodProcessor processor = new RenameNonVirtualMethodProcessor(findMethod(icu));
		processor.setNewElementName(NEW_METHOD_NAME);
		RenameRefactoring refactoring = new RenameRefactoring(processor);
		return refactoring;
	}

	private IMethod findMethod(ICompilationUnit icu) {
		try {
			for (IType type : icu.getTypes()) {
				for (IMethod method : type.getMethods()) {
					if (method.getElementName().equals(METHOD_TO_RENAME)
					 	/* && method.getNumberOfParameters() == 0 */) {
						return method;
					}
				}
			}
		} 
		catch (JavaModelException e) {
			throw new RuntimeException(e);
		}
		throw new RuntimeException("Method " + METHOD_TO_RENAME + " was not found"); 

	}
	
	@Override
	protected ProblemReport checkCustomPostconditions(ICompilationUnit icu) throws Exception {
		// TODO: Check that all method names have been replaced (and not the referencing method)
		return new ProblemReport();
	}

}
