package org.testorrery;

/**
 * Abstract decorator that allows one to duplicate a generator modulo the iteration state.
 * Used in generators that produce recursive data.
 * 
 * @author <a href="mailto:bdaniel3@uiuc.edu">Brett Daniel</a>
 */
public abstract class CopyableGenerator<T> extends Generator<T> implements ICopyableGenerator<T> {

	private IGenerator<T> currentCopy;
	
	public IGenerator<T> getCurrentCopy() {
		if (currentCopy == null) {
			currentCopy = getNewCopy();
		}
		return currentCopy;
	}
	
	@Override
	protected T generateNext() {
		return getCurrentCopy().next();
	}

	@Override
	public boolean hasNext() {
		return getCurrentCopy().hasNext();
	}
	
	@Override
	public void reset() {
		super.reset();
		getCurrentCopy().reset();
	}
	
	public CopyableGenerator<T> copy() {
		return new CopyableGenerator<T>() {
			protected IGenerator<T> getNewCopy() {
				return CopyableGenerator.this.getNewCopy();
			}
		};
	}
	
	protected abstract IGenerator<T> getNewCopy();
	
}
