package org.testorrery;

/**
 * Abstract composite generator that allows one to "skip" invalid combinations 
 * of child generators
 */
public abstract class FilteredCompositeGenerator<ValueType> extends
		CompositeGenerator<ValueType> {
	
	private boolean hasGenerated = false;
	
	protected abstract boolean iterationIsValid();
	
	@Override
	public boolean hasNext() {
		if (hasGenerated) return true;
		
		while (super.hasNext()) {			
			getIterator().next();
			if (iterationIsValid()) {
				hasGenerated = true;
				return true;
			}
		}
		return false;
	}
	
	@Override
	protected ValueType generateNext() {
		hasGenerated = false;
		return generateCurrent();
	}
}
