package org.testorrery;

import java.util.Iterator;

/**
 * Base class for generators. 
 * Handles reset bookkeeping and delegation to derived classes.
 *
 * @param <ValueType> The type of object the derived generator creates 
 * 
 */
public abstract class Generator<ValueType> implements IGenerator<ValueType> {
	private boolean isReset;
	private ValueType current;

	protected Generator() {
		isReset = true;
	}
		
	/**
	 * {@inheritDoc}
	 */
	public void reset() {
		isReset = true;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public ValueType next() {
		current = generateNext();
		isReset = false;
		return current;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public boolean isReset() {
		return isReset;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public ValueType current() {
		return current;
	}	
	
	/**
	 * {@inheritDoc}
	 */
	public abstract boolean hasNext();
	
	/**
	 * Creates and returns the next object
	 */
	protected abstract ValueType generateNext();	

	/**
	 * {@inheritDoc}
	 */
	public final Iterator<ValueType> iterator() {
		return new ForLoopIterator<ValueType>(this);
	}
}
