package org.testorrery;

/**
 * Interface used in {@link GeneratorTester} for callbacks needed 
 * to verify that a generator is behaving correctly.
 * 
 * @author Brett Daniel
 *
 * @param <GeneratedObjectType> The type of object that the target generator produces
 */
public interface IGeneratedObjectTester<GeneratedObjectType> {
	/**
	 * Returns the implementing object to its initial state. 
	 */
	void reset();
	
	/**
	 * Returns true if the two objects are considered equal
	 */
	boolean areEqual(GeneratedObjectType expected, GeneratedObjectType actual);
	
	/**
	 * Returns true if the generator was expected to create the given object
	 */
	boolean wasExpected(GeneratedObjectType generatedObject);
	
	/**
	 * Returns true if all the generated objects have been found (that is,
	 *  they have all been passed to <code>wasExpected</code>).
	 */
	boolean foundAllExpected();
}