package org.testorrery;

/**
 * A generator that always returns a single value
 *
 * @param <T> The type of the generated object
 */
public class Literal<T> extends Generator<T> {
	private T value;
		
	public Literal(T value) {
		this.value = value;		
	}

	@Override
	public boolean hasNext() {
		return isReset();
	}
	
	@Override
	protected T generateNext() {
		if (!isReset()) {
			throw new IterationException();
		}
		return value;
	}
	
	/**
	 * Reset the Literal to return a new value the next time next() is called.
	 * Overloads reset()
	 * @param newValue The new value to return
	 */
	protected void reset(T newValue) {
		super.reset();
		value = newValue;
	}

	/**
	 * Returns the value of this literal regardless of the reset state
	 */
	public T getInnerValue() {
		return value;
	}	
	/**
	 * Sets the inner value of this literal to a new value
	 */
	public void setInnerValue(T value) {
		this.value = value;
	}
}
