package org.testorrery;

/**
 * Callback for a generator that will be copied on demand.
 * Used for recursive or circular data structures.
 */
public class Recurse<ValueType> extends Generator<ValueType> {
	private IRecursive<ValueType> standin;
	private IGenerator<ValueType> actual;
	
	public Recurse(IRecursive<ValueType> standin) {
		this.standin = standin;
	}
	
	protected IGenerator<ValueType> getActual() {
		if (actual == null) {
			actual = standin.recurse();
		}
		return actual;
	}

	@Override
	public boolean hasNext() {
		return getActual().hasNext();
	}
	
	@Override
	protected ValueType generateNext() {
		return getActual().next();
	}
	
	public void reset() {
		super.reset();
		if (actual != null) {
			actual.reset();
		}
	}	
}
