package org.testorrery.examples;

public class Graph<ValueType> {

	private java.util.Map<ValueType, GraphNode> nodes;
	
	public Graph() {
		nodes = new java.util.HashMap<ValueType, GraphNode>();
	}
	
	public void addNode(ValueType newNodeValue) {
		if (nodes.containsKey(newNodeValue)) {
			throw new IllegalArgumentException(String.format("Cannot add duplicate key \"%1$d\"", newNodeValue));
		}
		GraphNode newNode = new GraphNode(newNodeValue);		
		nodes.put(newNodeValue, newNode);
	}
	
	public void linkNodes(ValueType fromName, ValueType toName) {
		if (!nodes.containsKey(fromName)) {
			throw new IllegalArgumentException(String.format("Cannot find node \"%1$d\"", fromName));
		}
		if (!nodes.containsKey(toName)) {
			throw new IllegalArgumentException(String.format("Cannot find node \"%1$d\"", toName));
		}

		GraphNode fromNode = nodes.get(fromName);
		GraphNode toNode = nodes.get(toName);
		
		fromNode.addReference(toNode);
	}
	
	public int nodeCount() {
		return nodes.size();
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (GraphNode node : nodes.values()) {
			sb.append(node.toString());
		}
		return sb.toString();
	}
	
	public class GraphNode {
		java.util.Map<ValueType, GraphNode> references;
		private ValueType value;
		
		public GraphNode(ValueType value) {
			references = new java.util.HashMap<ValueType, GraphNode>();
			this.value = value;
		}
		
		public ValueType getValue() {
			return value;
		}
		
		public void addReference(GraphNode otherNode) {
			references.put(otherNode.getValue(), otherNode);
		}	
		
		public void clearReferences() {
			references.clear();
		}
		
		public String toString() {
			StringBuilder sb = new StringBuilder();
			sb.append("\"").append(value.toString()).append("\" references ");
			if (references.size() == 0) {
				sb.append("nothing");
			}
			else {
				boolean isFirst = true;
				for (GraphNode reference : references.values()) {
					if (!isFirst) {
						sb.append(", ");
					}
					sb.append("\"").append(reference.getValue().toString()).append("\"");
					isFirst = false;			
				}				
			}
			sb.append("\n");
			return sb.toString();
		}
			
	}
}
