package org.testorrery.examples;

import java.util.ArrayList;

import org.testorrery.Chain;
import org.testorrery.CompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

/**
 * Produces all graphs for the given values
 */
public class GraphGenerator<ValueType> extends CompositeGenerator<Graph<ValueType>> {

	private IGenerator<ValueType>[] valueGenerators;		
	private ArrayList<ArrayList<IGenerator<Boolean>>> adjacency; // [from][to] 
	
	private IIterationLink mainIterator;
	
	public GraphGenerator(IGenerator<ValueType>... valueGenerators) {
		LinkCreator linkCreator = new LinkCreator(new CombinatoricLink());
		
		int nodeCount = valueGenerators.length;
		this.valueGenerators = valueGenerators;		
		for (IGenerator valueGen : valueGenerators) {
			linkCreator.add(valueGen);
		}		
		
		adjacency = new ArrayList<ArrayList<IGenerator<Boolean>>>(nodeCount); 
		
		int i, j;
		for (i = 0; i < nodeCount; i++) {	
			adjacency.add(new ArrayList<IGenerator<Boolean>>(nodeCount));
			for (j = 0; j < nodeCount; j++) {
				adjacency.get(i).add(new Chain<Boolean>(false, true));
				linkCreator.add(adjacency.get(i).get(j));
			}
		}
		
		this.mainIterator = linkCreator.getLink();
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		return mainIterator;
	}
	
	@Override
	protected Graph<ValueType> generateCurrent() {
		Graph<ValueType> newGraph = new Graph<ValueType>();
		
		// Add nodes to the graph
		for (IGenerator<ValueType> valueGen : valueGenerators) {
			newGraph.addNode(valueGen.current());
		}
				
		int fromIndex, toIndex;
		for (fromIndex = 0; fromIndex < adjacency.size(); fromIndex++) {
			for (toIndex = 0; toIndex < adjacency.size(); toIndex++) {
				// if nodes are currently linked
				if (adjacency.get(fromIndex).get(toIndex).current() == true) {
					newGraph.linkNodes(valueGenerators[fromIndex].current(), valueGenerators[toIndex].current());
				}				
			}
		}
		
		return newGraph;		
	}
}
