package org.testorrery.examples;

import org.testorrery.Chain;
import org.testorrery.CompositeGenerator;
import org.testorrery.ICopyableGenerator;
import org.testorrery.IRecursive;
import org.testorrery.Literal;
import org.testorrery.Recurse;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

/**
 * Generates all lists up to and including the given length
 */
public class ListGenerator<ValueType> 
	extends CompositeGenerator<List<ValueType>> 
	implements IRecursive<List<ValueType>> {
	
	private int length;
	private ICopyableGenerator<ValueType> valueGenerator;
	private Chain<List<ValueType>> nodeGenerator;
	
	public ListGenerator(ICopyableGenerator<ValueType> valueGen, int length) {
		if (length < 1) {
			throw new java.lang.IllegalArgumentException("List length must be >= 1");
		}
		
		this.valueGenerator = valueGen;
		this.length = length;
		
		nodeGenerator = new Chain<List<ValueType>>();
		nodeGenerator.add(new Literal<List<ValueType>>(null));
		if (length > 1) {
			nodeGenerator.add(new Recurse<List<ValueType>>(this));
		}
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(nodeGenerator, valueGenerator);
	}

	@Override
	protected List<ValueType> generateCurrent() {
		return new List<ValueType>(valueGenerator.current(), nodeGenerator.current());
	}
	
	public ListGenerator<ValueType> recurse() {
		return new ListGenerator<ValueType>(valueGenerator.copy(), length - 1);
	}	
}
