package org.testorrery.examples;

import org.testorrery.Chain;
import org.testorrery.CompositeGenerator;
import org.testorrery.ICopyableGenerator;
import org.testorrery.IRecursive;
import org.testorrery.Literal;
import org.testorrery.Recurse;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

/**
 * Generates all trees up to the given depth 
 * 
 * @author <a href="mailto:bdaniel3@uiuc.edu">Brett Daniel</a>
 */
public class TreeGenerator<ValueType> 
	extends CompositeGenerator<Tree<ValueType>> 
	implements IRecursive<Tree<ValueType>> {
	
	private int depth;
	private ICopyableGenerator<ValueType> valueGenerator;
	private Chain<Tree<ValueType>> leftNodeGenerator;
	private Chain<Tree<ValueType>> rightNodeGenerator;

	public TreeGenerator(ICopyableGenerator<ValueType> valueGen, int depth) {
		if (depth < 1) {
			throw new java.lang.IllegalArgumentException("Tree depth must be >= 1");
		}
		
		this.valueGenerator = valueGen;
		this.depth = depth;		
		this.leftNodeGenerator = makeNodeGenerator(depth);
		this.rightNodeGenerator = makeNodeGenerator(depth);
	}

	private Chain<Tree<ValueType>> makeNodeGenerator(int depth) {
		Chain<Tree<ValueType>> nodeGen = new Chain<Tree<ValueType>>();
		nodeGen.add(new Literal<Tree<ValueType>>(null));
		if (depth > 1) {
			nodeGen.add(new Recurse<Tree<ValueType>>(this));
		}
		return nodeGen;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(
				valueGenerator, 
				leftNodeGenerator, 
				rightNodeGenerator);
	}
	
	@Override
	protected Tree<ValueType> generateCurrent() {
		return new Tree<ValueType>(
				valueGenerator.current(), 
				leftNodeGenerator.current(), 
				rightNodeGenerator.current());
	}	

	/**
	 * Returns a recursed tree generator 
	 */
	public TreeGenerator<ValueType> recurse() {
		return new TreeGenerator<ValueType>(valueGenerator.copy(), depth - 1);
	}	
}
