package org.testorrery.link;

import org.testorrery.IGenerator;

/**
 * Abstract base class for link nodes with two children
 */
public abstract class BinaryLink implements IIterationLink {
	
	private IIterationLink left;
	private IIterationLink right;
	
	public IIterationLink getLeft() {
		return left;
	}
	public void setLeft(IIterationLink left) {
		this.left = left;
	}
	
	public IIterationLink getRight() {
		return right;
	}
	public void setRight(IIterationLink right) {
		this.right = right;
	}
	
	public BinaryLink() {}
	
	public BinaryLink(IIterationLink left, IIterationLink right) {
		this.left = left;
		this.right = right;
	}
	
	public BinaryLink(IGenerator leftGen, IGenerator rightGen) {
		this(new GeneratorLink(leftGen), new GeneratorLink(rightGen));
	}
	/**
	 * Resets this link and all child links to their initial state. Overridable.
	 *
	 */
	public void reset() {
		left.reset();
		right.reset();
	}
	
	/**
	 * @return true if this link can be iterated
	 */
	public abstract boolean hasNext();
	
	/**
	 * Iterates this link. The specific form of iteration is determined by the derived class
	 *
	 */
	public abstract void next();
	
	/**
	 * Used to create a new instance of this type of BinaryLink without knowing the concrete type
	 */
	public abstract BinaryLink createNewInstance(IIterationLink left, IIterationLink right);
	
}
