package org.testorrery.link;


/**
 * Iteration tree visitor that creates the iteration expression for the tree
 */
public class ExpressionVisitor implements IIteratorVisitor {
	/**
	 * A generator's index
	 */
	private int genCount;
	/**
	 * The working expression
	 */
	private StringBuffer expression;

	public ExpressionVisitor() {
	}
	
	public void visit(BinaryLink link) {
		appendBinaryLink(link, '?');
	}
	
	public void visit(CombinatoricLink link) {
		appendBinaryLink(link, LinkExpressionParser.COMBINATORIC_OPERATOR);
	}

	public void visit(SequentialLink link) {
		appendBinaryLink(link, LinkExpressionParser.SEQUENTIAL_OPERATOR);
	}

	public void appendBinaryLink(BinaryLink link, char opChar) {
		expression.append('(');
		link.getLeft().accept(this);
		expression.append(" ");
		expression.append(opChar);
		expression.append(" ");
		link.getRight().accept(this);
		expression.append(')');	
	}
	
	public void visit(GeneratorLink link) {
		expression.append("$" + genCount++);
		//_expression.append(link.getInnerGenerator().toString()); // useful for debugging
	}
	
	/**
	 * @return The expression created from traversing the given iteration tree
	 */
	public String getExpression(IIterationLink root) {
		genCount = 0;
		expression = new StringBuffer();
		root.accept(this);
		return expression.toString();
	}
}
