package org.testorrery.tests;

import org.junit.Assert;
import org.junit.Test;
import org.testorrery.Chain;
import org.testorrery.GeneratorTester;
import org.testorrery.Literal;


public class ChainTest {
	
	@Test
	public void demo() {
		Chain<String> chain = new Chain<String>();
		chain.add(new Literal<String>("S1"));
		chain.add(new Literal<String>("S2"));
		
		Chain<String> subChain = new Chain<String>();
		subChain.add(new Literal<String>("S3"));
		subChain.add(new Literal<String>("S4"));
		
		chain.add(subChain);
		
		chain.add(new Literal<String>("S5"));
		
		for (String genString : chain) {
			System.out.println(genString);
		}
		
	}
	
	@Test
	public void basicFunctionality() {
		final String[] strings = new String[] {"S1", "S2", "S3", "S4"};		
		Chain<String> chain = new Chain<String>(strings);
		
		GeneratorTester.testExpectedCount(chain, 4);				
	}
	
	/**
	 * Tests that a chain iterates all the way through a contained generator 
	 * before continuing on to the next generator in the chain
	 */
	@Test
	public void compoundChain() {
			
		String[] strings1 = new String[] {"S1", "S2", "S3", "S4"};		
		String[] strings2 = new String[] {"S5", "S6", "S7"};		
		
		Chain<String> chain1 = new Chain<String>(strings1);
		
		Chain<String> chain2 = new Chain<String>(strings2);
		
		chain1.add(chain2); 
		
		int generatedCount = 0;
		for (String genString : chain1) {
			if (generatedCount < strings1.length) {
				Assert.assertEquals(strings1[generatedCount], genString);
			}
			else {
				Assert.assertEquals(strings2[generatedCount - strings1.length], genString);
			}			
			generatedCount++;
		}
		// should iterate through all values of both chains
		Assert.assertEquals(strings1.length + strings2.length, generatedCount);
	}
	
	@Test
	public void emptyChain() {
		Chain<String> emptySeq = new Chain<String>();
		GeneratorTester.testExpectedCount(emptySeq, 0);
	}
	
	@Test
	public void chainOfEmptyGenerators() {
		Chain<String> emptySeq1 = new Chain<String>();
		Chain<String> emptySeq2 = new Chain<String>();
		
		Chain<String> containerSeq = new Chain<String>();
		containerSeq.add(emptySeq1);
		containerSeq.add(emptySeq2);
		
		GeneratorTester.testExpectedCount(containerSeq, 0);
	}
	
	@Test
	public void emptyGeneratorInChain() {
		Chain<Integer> emptySeq = new Chain<Integer>();
		
		Chain<Integer> containerSeq = new Chain<Integer>();
		containerSeq.add(1);
		containerSeq.add(emptySeq);
		containerSeq.add(2);
		
		GeneratorTester.testExpectedObjects(containerSeq, 1, 2);
	}
}
