package org.testorrery.tests;

import org.junit.Assert;
import org.testorrery.IGenerator;
import org.testorrery.link.BinaryLink;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.GeneratorLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.IIteratorVisitor;
import org.testorrery.link.SequentialLink;

/**
 * IIterationLink visitor that asserts that the tree contains the expected generators
 * in the given order (left-to-right). Used to test that an iteration tree is formed correctly.
 * @author Brett Daniel
 *
 */
public class ExpectedGeneratorsVisitor implements IIteratorVisitor {
	private IGenerator[] _expectedGenerators;	
	private int _curIndex;
	
	public ExpectedGeneratorsVisitor() {	
	}

	public void visit(BinaryLink link) {
		link.getLeft().accept(this);
		link.getRight().accept(this);
	}

	public void visit(CombinatoricLink link) {
		visit((BinaryLink)link);
	}

	public void visit(SequentialLink link) {
		visit((BinaryLink)link);
	}

	public void visit(GeneratorLink link) {
		if (_curIndex == _expectedGenerators.length) {
			Assert.fail("Extra generators found");
		}
		Assert.assertEquals("Unexpected generator found.", _expectedGenerators[_curIndex++], link.getInnerGenerator());
	}
	
	/**
	 * Asserts that the given iteration tree has the expected generators in the given order
	 */
	public void assertFoundAll(IIterationLink root, IGenerator... expectedIterators) {
		_curIndex = 0;
		_expectedGenerators = expectedIterators;
		root.accept(this);		
		Assert.assertEquals("Not all generators were found.", _expectedGenerators.length, _curIndex);
	}

}