package org.testorrery.tests;

import org.junit.Before;
import org.junit.Test;
import org.testorrery.Chain;
import org.testorrery.Filter;
import org.testorrery.GeneratorTester;
import org.testorrery.IPredicate;
import org.testorrery.examples.PairGenerator;

public class FilterTest {
	final static Integer TEST_VALUE = 2;
	Chain<Integer> testSequence = new Chain<Integer>(1, TEST_VALUE, 3);
	private Filter<Integer> filteredSequence;
	
	@Before
	public void setUp() {
		filteredSequence = new Filter<Integer>(testSequence);
	}
	
	@Test
	public void noPredicates() {
		// filtered iteration should echo test sequence
		GeneratorTester.testExpectedStrings(filteredSequence, "1", TEST_VALUE.toString(), "3");
	}
	
	@Test
	public void filterByPredicate() {				
		// filter by test value
		filteredSequence.registerPredicate(
				new IPredicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.equals(TEST_VALUE);
					}
				});
		
		GeneratorTester.testExpectedStrings(filteredSequence, TEST_VALUE.toString());		
	}
	
	@Test
	public void multiplePredicates() {				
		// Accept only those values lower than testValue + 1
		filteredSequence.registerPredicate(
				new IPredicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.intValue() < TEST_VALUE + 1;
					}
				});
		// Accept only those values greater than testValue - 1
		filteredSequence.registerPredicate(
				new IPredicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.intValue() > TEST_VALUE - 1;
					}
				});
		
		// Make sure only the test value was generated
		GeneratorTester.testExpectedStrings(filteredSequence, TEST_VALUE.toString());		
	}
	
	@Test
	public void noPredicatesSatisfied() {
		filteredSequence.registerPredicate(
				new IPredicate<Integer>() {
					public boolean evaluate(Integer testObject) {						
						return false; // no values are accepted
					}
				});
		GeneratorTester.testExpectedCount(filteredSequence, 0);
	}
	
	@Test
	public void linkedCombinatorically() {		
		// filter by test value
		filteredSequence.registerPredicate(
				new IPredicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.equals(TEST_VALUE);
					}
				});
		Chain<Integer> dummySequence = new Chain<Integer>(1, 2, 3);
		
		// make filtered generator part of a larger generator 
		PairGenerator<Integer, Integer> pair = new PairGenerator<Integer, Integer>(dummySequence, filteredSequence);
		GeneratorTester.testExpectedStrings(pair, 
				"[1," + TEST_VALUE.toString() + ']',
				"[2," + TEST_VALUE.toString() + ']',
				"[3," + TEST_VALUE.toString() + ']');
		
		// and in reverse 
		PairGenerator<Integer, Integer> pair2 = new PairGenerator<Integer, Integer>(filteredSequence, dummySequence);
		GeneratorTester.testExpectedStrings(pair2, 
				'[' + TEST_VALUE.toString() + ",1]",
				'[' + TEST_VALUE.toString() + ",2]",
				'[' + TEST_VALUE.toString() + ",3]");
	}
}
