package org.testorrery.tests;

import org.junit.Test;
import org.testorrery.Chain;
import org.testorrery.GeneratorTester;
import org.testorrery.ObjectIndexPair;

public class GeneratorTesterTest {
	@Test()
	public void expectedObjectCount() {
		Chain<Character> seq = new Chain<Character>('a','b','c');
		GeneratorTester.testExpectedCount(seq, 3);		
	}
	
	@Test(expected=java.lang.AssertionError.class)
	public void expectedObjectCount_InvalidCount() {
		Chain<Character> seq = new Chain<Character>('a','b','c');
		GeneratorTester.testExpectedCount(seq, 200);		
	}
		
	@Test()
	public void expectedObjectArray() {
		final Character[] vals = new Character[] {'a','b','c'};
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedObjects(seq, vals);		
	}
	
	@Test(expected=java.lang.AssertionError.class)
	public void expectedObjectArray_GenerateTooFew() {
		final Character[] vals = new Character[] {'a','b','c'};
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedObjects(seq, 'a', 'b');		
	}
	
	@Test(expected=java.lang.AssertionError.class)
	public void expectedObjectArray_GenerateTooMany() {
		final Character[] vals = new Character[] {'a','b','c'};
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedObjects(seq, 'a','b','c','d');		
	}
	
	@Test()
	public void expectedStringValues() {
		final Integer[] vals = new Integer[] {1,2,3};
		final String[] stringVals = new String[] {"1","2","3"};
		
		Chain<Integer> seq = new Chain<Integer>(vals);
		GeneratorTester.testExpectedStrings(seq, stringVals);		
	}
	
	@Test(expected=java.lang.AssertionError.class)
	public void expectedStringValues_GenerateTooFew() {
		final Integer[] vals = new Integer[] {1,2,3};
		final String[] stringVals = new String[] {"1","2"};
		
		Chain<Integer> seq = new Chain<Integer>(vals);
		GeneratorTester.testExpectedStrings(seq, stringVals);		
	}
	
	@Test(expected=java.lang.AssertionError.class)
	public void expectedStringValues_GenerateTooMany() {
		final Integer[] vals = new Integer[] {1,2,3};
		final String[] stringVals = new String[] {"1","2","3","4"};
		
		Chain<Integer> seq = new Chain<Integer>(vals);
		GeneratorTester.testExpectedStrings(seq, stringVals);		
	}
	
	@Test() 
	public void expectedAtIndex() {
		final Character[] vals = new Character[] {'a', 'b', 'c'};
		
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedIndices(seq, new ObjectIndexPair(vals[1], 1));
	}
	
	@Test() 
	public void expectedAtIndices() {
		final Character[] vals = new Character[] {'a', 'b', 'c', 'd', 'e'};
		
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedIndices(seq, new ObjectIndexPair(vals[1], 1), new ObjectIndexPair(vals[3], 3));
	}
	
	@Test(expected=AssertionError.class) 
	public void expectedAtIndices_NonexistentIndex() {
		final Character[] vals = new Character[] {'a', 'b', 'c', 'd', 'e'};
		
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedIndices(seq, new ObjectIndexPair(vals[1], 1), new ObjectIndexPair(vals[3], 50));
	}
	
	@Test(expected=IllegalArgumentException.class) 
	public void expectedAtIndices_InvalidIndex() {
		final Character[] vals = new Character[] {'a', 'b', 'c', 'd', 'e'};
		
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedIndices(seq, new ObjectIndexPair(vals[1], 1), new ObjectIndexPair(vals[3], -2));
	}
	
	@Test(expected=IllegalArgumentException.class) 
	public void expectedAtIndices_DuplicateIndex() {
		final Character[] vals = new Character[] {'a', 'b', 'c', 'd', 'e'};
		
		Chain<Character> seq = new Chain<Character>(vals);
		GeneratorTester.testExpectedIndices(seq, new ObjectIndexPair(vals[1], 1), new ObjectIndexPair(vals[3], 1));
	}
}
