package org.testorrery.tests;

import org.junit.Test;
import org.testorrery.Chain;
import org.testorrery.GeneratorTester;
import org.testorrery.RuntimeGenerator;

public class RuntimeGeneratorTest {
	
	@Test
	public void publicField_PrimitiveType() throws SecurityException, NoSuchFieldException  {
		RuntimeGenerator<PubFiePriTyp> rtGen = 
			new RuntimeGenerator<PubFiePriTyp>(PubFiePriTyp.class);
		rtGen.setGenerator("field", new Chain<Integer>(1,2,3));
		
		GeneratorTester.testExpectedStrings(rtGen, "1", "2", "3");			
	}
	
	public static class PubFiePriTyp {
		public int field;
		@Override
		public String toString() {
			return String.valueOf(field);
		}
	}
	
	@Test
	public void publicField_ObjectType() throws SecurityException, NoSuchFieldException {
		RuntimeGenerator<PubFieObjTyp> rtGen = 
			new RuntimeGenerator<PubFieObjTyp>(PubFieObjTyp.class);
		rtGen.setGenerator("field", new Chain<String>("A","B","C"));
		
		GeneratorTester.testExpectedStrings(rtGen, "A", "B", "C");	
	}
	
	public static class PubFieObjTyp {
		public String field;
		@Override
		public String toString() {
			return field;
		}
	}
	
	@Test
	public void accessorMethod_PrimitiveType() throws SecurityException, NoSuchFieldException {
		RuntimeGenerator<AccMetPriTyp> rtGen = 
			new RuntimeGenerator<AccMetPriTyp>(AccMetPriTyp.class);
		rtGen.setGenerator("field", new Chain<Integer>(1,2,3));
		
		GeneratorTester.testExpectedStrings(rtGen, "1", "2", "3");			
	}	
	public static class AccMetPriTyp {
		private int field;
		
		public void setField(int field) {
			this.field = field;
		}
		
		@Override
		public String toString() {
			return String.valueOf(field);
		}
	}
	
	@Test
	public void accessorMethod_ObjectType() throws SecurityException, NoSuchFieldException {
		RuntimeGenerator<PubFieObjTyp> rtGen = 
			new RuntimeGenerator<PubFieObjTyp>(PubFieObjTyp.class);
		rtGen.setGenerator("field", new Chain<String>("A","B","C"));
		
		GeneratorTester.testExpectedStrings(rtGen, "A", "B", "C");		
	}
	
	public static class AccMetObjTyp {
		private String field;
		
		public void setField(String field) {
			this.field = field;
		}
		
		@Override
		public String toString() {
			return field;
		}
	}
	
	@Test
	public void multipleFields() throws SecurityException, NoSuchFieldException {
		RuntimeGenerator<MultiField> rtGen = 
			new RuntimeGenerator<MultiField>(MultiField.class);
		rtGen.setGenerator("field1", new Chain<String>("A", "B"));
		rtGen.setGenerator("field2", new Chain<String>("C", "D"));
		GeneratorTester.testExpectedStrings(rtGen, "AC", "AD", "BC", "BD");		
	}
	
	public static class MultiField {
		private String field1;
		private String field2;
		
		public void setField1(String field1) {
			this.field1 = field1;
		}
		public void setField2(String field2) {
			this.field2 = field2;
		}
		
		@Override
		public String toString() {			
			return field1 + field2;
		}
	}
	
	@Test
	public void nullableField() throws SecurityException, NoSuchFieldException {
		RuntimeGenerator<Nullable> rtGen = 
			new RuntimeGenerator<Nullable>(Nullable.class);
		rtGen.setGenerator("field", new Chain<String>("A", null)); // nullable value
		GeneratorTester.testExpectedStrings(rtGen, "A", "<NULL>");	
	}
	public static class Nullable {
		private String field;
		public void setField(String field) {
			this.field = field;
		}
		@Override
		public String toString() {	
			if (field == null) return "<NULL>";
			return field;
		}
	}
		
	@Test(expected=NoSuchFieldException.class)
	public void invalidField() throws SecurityException, NoSuchFieldException {
		RuntimeGenerator<PubFieObjTyp> rtGen = 
			new RuntimeGenerator<PubFieObjTyp>(PubFieObjTyp.class);
		rtGen.setGenerator("adsfasdfsf", new Chain<String>("A","B","C"));
	}
}
