package org.testorrery.tests.link;

import org.junit.Assert;
import org.junit.Test;
import org.testorrery.Chain;
import org.testorrery.IGenerator;
import org.testorrery.Literal;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.GeneratorLink;

public class CombinatoricLinkTest {
	
	@Test
	public void testInitialization() {
		IGenerator<Integer> gen1 = new Literal<Integer>(1);
		IGenerator<Character> gen2 = new Literal<Character>('a');
		
		CombinatoricLink link = new CombinatoricLink(gen1, gen2);
		
		link.next();
		Assert.assertEquals(1, gen1.current().intValue());
		Assert.assertEquals('a', gen2.current().charValue());		
	}
	
	@Test()
	public void testIteration() {
		IGenerator<String> gen1 = new Chain<String>("string1", "string2", "string3");
		IGenerator<Integer> gen2 = new Chain<Integer>(1, 2, 3, 4);
		
		CombinatoricLink link = new CombinatoricLink(gen1, gen2);
		
		LinkTestUtil.iterateLink(link, 3*4, gen1, gen2);
	}
	
	@Test()
	public void testNestedIteration() {
		IGenerator<String> gen1 = new Chain<String>("string1", "string2");
		IGenerator<Integer> gen2 = new Chain<Integer>(1, 2, 3);
		IGenerator<Character> gen3 = new Chain<Character>('a', 'b', 'c', 'd');
		
		CombinatoricLink link = 
			new CombinatoricLink(
					new GeneratorLink(gen1), 
					new CombinatoricLink(
							new GeneratorLink(gen2),
							new GeneratorLink(gen3)));
		
		LinkTestUtil.iterateLink(link, 2 * 3 * 4, gen1, gen2, gen3);
	}
	
	@Test()
	public void testDeepNesting() {
		CombinatoricLink link = 
			new CombinatoricLink(
					new CombinatoricLink(
							new CombinatoricLink(
									new GeneratorLink(new Chain<String>("1", "2")), 
									new CombinatoricLink(
											new GeneratorLink(new Chain<String>("3", "4")), 
											new GeneratorLink(new Chain<String>("5", "6")))),
							new GeneratorLink(new Chain<String>("7", "8"))),
					new GeneratorLink(new Chain<String>("9", "10")));
		
		LinkTestUtil.iterateLink(link, 2 * 2 * 2 * 2 * 2);
	}
}
