package org.testorrery.tests.link;

import org.junit.Assert;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;

/** 
 * Utility class for testing links
 * @author Brett Daniel
 *
 */
public class LinkTestUtil {
	
	/**
	 * Iterates the given link to verify that it works correctly
	 * @param targetLink The link to verify
	 * @param expectedCount The expected number of iterations
	 * @param registeredGenerators The generators registered to the given link (optional, used for printing)
	 */
	public static void iterateLink(IIterationLink targetLink, int expectedCount, IGenerator... registeredGenerators) {
		int runCount = 2; // loop twice to test reset()
		int count;
		
		while (runCount-- > 0) {
			count = 0;
			targetLink.reset();
			while (targetLink.hasNext() && count <= expectedCount) {
				targetLink.next();
				if (runCount == 1 && registeredGenerators.length > 0) {
					for (IGenerator gen : registeredGenerators) {
						System.out.print(gen.current());
						System.out.print("\t");
					}
					System.out.print("\n");
				}
				count++;			
			}
			Assert.assertEquals(expectedCount, count);
		}
	}

}
