package edu.uiuc.astgen;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;

import org.testorrery.CompositeGenerator;

/**
 * Base class for generators that generate objects derived from ASTNode.
 *
 * @param <ASTNodeType> The type of ASTNode generated by this generator
 */
public abstract class ASTNodeGeneratorBase<ASTNodeType extends ASTNode> extends CompositeGenerator<ASTNodeType> {
	private static AST defaultAST = AST.newAST(AST.JLS3);	
	
	public static AST ast() {
		return defaultAST;
	}
	
	private AST ast;
	
	public ASTNodeGeneratorBase() {
		ast = defaultAST;
	}
	
	public AST getAST() {
		return ast;
	}
	
	/**
	 * Sets this generator's AST to something other than the default.
	 * Note that the <em>caller</em> should call <code>setAST</code> on all child generators. 
	 */
	public void setAST(AST ast) {
		this.ast = ast;
	}
	
	@Override
	public ASTNodeType current() {		
		ASTNodeType current = super.current();
		if (current != null && current.getParent() != null) {
			// Return a copy of the current node to prevent parent conflicts
			return (ASTNodeType)ASTNode.copySubtree(getAST(), current);
		}
		return current;
	}
	
}
