package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.testorrery.IGenerator;
import org.testorrery.Literal;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;

public class ArrayFieldDeclarationGenerator extends
		ASTNodeGeneratorBase<FieldDeclaration> {

	private IGenerator<? extends Type> typeGen;
	private IGenerator<? extends SimpleName> nameGen;
	private IGenerator<Integer> dimGen;

	public void setDimGen(IGenerator<Integer> dimGen) {
		this.dimGen = dimGen;
	}

	public void setNameGen(IGenerator<? extends SimpleName> nameGen) {
		this.nameGen = nameGen;
	}

	public void setTypeGen(IGenerator<? extends Type> typeGen) {
		this.typeGen = typeGen;
	}	
	
	@Override
	protected IIterationLink getInitializedIterator() {
		AST ast = ASTNodeGeneratorBase.ast();
		if (typeGen == null) {
			typeGen = new ASTNodeLiteral<Type>(ast.newPrimitiveType(PrimitiveType.INT));
		}
		if (nameGen == null) {
			nameGen = new ASTNodeLiteral<SimpleName>(ast.newSimpleName("theField"));
		}
		if (dimGen == null) {
			dimGen = new Literal<Integer>(0);
		}
		return LinkCreator.createCombinatoricLink(typeGen, nameGen, dimGen);
	}

	@Override
	protected FieldDeclaration generateCurrent() {
		AST ast = ASTNodeGeneratorBase.ast();
		VariableDeclarationFragment vdf = ast.newVariableDeclarationFragment();
		vdf.setExtraDimensions(dimGen.current());
		vdf.setName(nameGen.current());
		FieldDeclaration fd = ast.newFieldDeclaration(vdf);
		fd.setType(typeGen.current());
		return fd;
	}

}
