package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.Type;
import org.testorrery.IGenerator;
import org.testorrery.Literal;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;

public class ArrayTypeGenerator extends ASTNodeGeneratorBase<Type> {	

	private static final AST ast = ast();
	
	private IGenerator<? extends Type> typeGen;
	private IGenerator<Integer> subscriptNumberGen;
	
	@Override
	protected IIterationLink getInitializedIterator() {
		if (typeGen == null) {
			typeGen = new ASTNodeLiteral<Type>(ast.newPrimitiveType(PrimitiveType.INT));
		}
		if (subscriptNumberGen == null) {
			subscriptNumberGen = new Literal<Integer>(1);
		}
		return LinkCreator.createCombinatoricLink(typeGen, subscriptNumberGen);
	}
	
	public void setSubscriptNumberGen(IGenerator<Integer> subscriptNumberGen) {
		this.subscriptNumberGen = subscriptNumberGen;
	}
	
	public void setTypeGen(IGenerator<? extends Type> typeGen) {
		this.typeGen = typeGen;
	}

	@Override
	protected Type generateCurrent() {
		Type baseType = typeGen.current();
		int subscripts = subscriptNumberGen.current();
		
		return getArrayType(baseType, subscripts);
	}
	
	private Type getArrayType(Type baseType, int subscripts) {
		if (subscripts == 0) {
			return baseType;
		}
		return ast.newArrayType(getArrayType(baseType, subscripts - 1));
	}

}
