package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.BooleanLiteral;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.NumberLiteral;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class AssignmentExpressionForVariableGenerator extends ASTNodeGeneratorBase<ExpressionStatement>{
	private ASTNodeGeneratorBase<SingleVariableDeclaration> variableDeclaration;
	
	public AssignmentExpressionForVariableGenerator(ASTNodeGeneratorBase<SingleVariableDeclaration> variableGen)  {
		super();
		variableDeclaration = variableGen;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(variableDeclaration);
	}
	
	@Override
	protected ExpressionStatement generateCurrent() {
		SingleVariableDeclaration variable = variableDeclaration.current();
		AST ast = super.getAST();
		SimpleName name = variable.getName();
		if (name.getParent() != null)
			name = (SimpleName)name.copySubtree(ast, name);

		Assignment assignment = ast.newAssignment();
		assignment.setLeftHandSide(name);
		
		Expression expression=null;
		// later move those generators to (static) fields
		if (variable.getType().toString().equals("boolean")) {
			BooleanLiteralGenerator b = new BooleanLiteralGenerator();
			b.next();
			expression = (BooleanLiteral)b.current();
		} else if (variable.getType().toString().equals("byte")) {
			NumberLiteralGenerator b = new NumberLiteralGenerator(1, PrimitiveType.BYTE);
			b.next();
			expression = (NumberLiteral)b.current();
		} 	
		assignment.setRightHandSide(expression);
		ExpressionStatement expressionStatement = ast.newExpressionStatement(assignment);
		return expressionStatement;
	}	
	
}
