package edu.uiuc.astgen.basic;


import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.BooleanLiteral;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.NumberLiteral;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class AssignmentExpressionGenerator extends ASTNodeGeneratorBase<ExpressionStatement>{

	private ASTNodeGeneratorBase<FieldDeclaration> fieldDeclaration;

	public AssignmentExpressionGenerator(ASTNodeGeneratorBase<FieldDeclaration> fieldGen)  {
		super();
		fieldDeclaration = fieldGen;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(fieldDeclaration);
	}

	@Override
	protected ExpressionStatement generateCurrent() {
		FieldDeclaration f = fieldDeclaration.current();
		
		AST ast = super.getAST();
		VariableDeclarationFragment vdf = (VariableDeclarationFragment)f.fragments().get(0);
		SimpleName name = vdf.getName();		
		
		if (name.getParent() != null)
			name = (SimpleName)name.copySubtree(ast, name);

		Assignment assignment = ast.newAssignment();
		assignment.setLeftHandSide(name);
		
		Expression expression=null;
		// later move those generators to (static) fields
		if (f.getType().toString().equals("boolean")) {
			BooleanLiteralGenerator b = new BooleanLiteralGenerator();
			b.next();
			expression = (BooleanLiteral)b.current();
		} else if (f.getType().toString().equals("byte")) {
			NumberLiteralGenerator b = new NumberLiteralGenerator(1, PrimitiveType.BYTE);
			b.next();
			expression = (NumberLiteral)b.current();
		} 
		System.out.println(f.getType()+ " - "+ expression);
		assignment.setRightHandSide(expression);
		ExpressionStatement expressionStatement = ast.newExpressionStatement(assignment);
		return expressionStatement;
	}	
}
