package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.Expression;
import org.testorrery.IGenerator;
import org.testorrery.Literal;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class AssignmentGenerator extends ASTNodeGeneratorBase<Assignment> {

	private ASTNodeGeneratorBase<? extends Expression> leftGenerator;
	private IGenerator<Assignment.Operator> opGenerator;
	private ASTNodeGeneratorBase<? extends Expression> rightGenerator;
	
	public AssignmentGenerator() {
		leftGenerator = null;
		rightGenerator = null;
		opGenerator = new Literal<Assignment.Operator>(Assignment.Operator.ASSIGN);
	}
	
	public AssignmentGenerator(
			ASTNodeGeneratorBase<? extends Expression> leftGenerator, 
			ASTNodeGeneratorBase<? extends Expression> rightGenerator) {
		this();
		this.leftGenerator = leftGenerator;
		this.rightGenerator = rightGenerator;
	}
	
	
	public ASTNodeGeneratorBase<? extends Expression> getLeftGenerator() {
		return leftGenerator;
	}
	public void setLeftGenerator(
			ASTNodeGeneratorBase<? extends Expression> leftGenerator) {
		this.leftGenerator = leftGenerator;
	}
	
	public IGenerator<Assignment.Operator> getOperatorGenerator() {
		return opGenerator;
	}
	public void setOperatorGenerator(IGenerator<Assignment.Operator> opGenerator) {
		this.opGenerator = opGenerator;
	}
	
	public ASTNodeGeneratorBase<? extends Expression> getRightGenerator() {
		return rightGenerator;
	}
	public void setRightGenerator(
			ASTNodeGeneratorBase<? extends Expression> rightGenerator) {
		this.rightGenerator = rightGenerator;
	}

	@Override
	protected Assignment generateCurrent() {
		Assignment assignment = super.getAST().newAssignment();
		if (leftGenerator == null) {
			assignment.setLeftHandSide(super.getAST().newNullLiteral());
		}
		else {
			assignment.setLeftHandSide(leftGenerator.current());				
		}
		assignment.setOperator(opGenerator.current());
		if (rightGenerator == null) {
			assignment.setRightHandSide(super.getAST().newNullLiteral());
		}
		else {
			assignment.setRightHandSide(rightGenerator.current());				
		}
		
		return assignment;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(leftGenerator, opGenerator, rightGenerator);
	}

}
