package edu.uiuc.astgen.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ClassInstanceCreation;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.SimpleType;
import org.eclipse.jdt.core.dom.Type;
import org.testorrery.Chain;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class ClassInstanceCreationGenerator extends ASTNodeGeneratorBase<ClassInstanceCreation>{
	private IGenerator<? extends Expression> optionalExpressionDeclaration;
	//private IGenerator<? extends Name> typeName;
	private IGenerator<? extends Type> type;
	private ArrayList<IGenerator<? extends Type>> typeArguments;
	private ArrayList<IGenerator<? extends Expression>> arguments;
	
	public ClassInstanceCreationGenerator (IGenerator<? extends Type> typeGen) {
		super();		
		type = typeGen;
		arguments = new ArrayList<IGenerator<? extends Expression>>();
		typeArguments = new ArrayList<IGenerator<? extends Type>>();
	}

	public void addTypeArgumentGenerator(IGenerator<? extends Type> typeArgumentGen) {
		typeArguments.add(typeArgumentGen);
	}
	
	public void addArgumentGenerator(IGenerator<? extends Expression> argumentGen) {
		arguments.add(argumentGen);
	}
		
	@Override
	protected ClassInstanceCreation generateCurrent() {
		ClassInstanceCreation classInstance = super.getAST().newClassInstanceCreation();
		if (optionalExpressionDeclaration != null)
			classInstance.setExpression(optionalExpressionDeclaration.current());
		//classInstance.setName(typeName.current());
		Type typeDeclaration = type.current();
		if (typeDeclaration.getParent() != null)
			typeDeclaration = (Type)typeDeclaration.copySubtree(getAST(), typeDeclaration); 		
		classInstance.setType(typeDeclaration);		
		if (arguments != null) {
			for (IGenerator<? extends Expression> expressionGen : arguments) {
				classInstance.arguments().add(expressionGen.current());
			}		
		}
		if (typeArguments != null) {
			for (IGenerator<? extends Type> typeExpressionGen : typeArguments) {
				classInstance.typeArguments().add(typeExpressionGen.current());
			}
		}	
		return classInstance;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator linkCreator = new LinkCreator(new CombinatoricLink());
		if (optionalExpressionDeclaration != null)
			linkCreator.add(optionalExpressionDeclaration);
		//linkCreator.add(typeName);
		linkCreator.add(type);
		if (typeArguments != null)
			linkCreator.addAll(typeArguments);
		if (arguments != null)
			linkCreator.addAll(arguments);
		return linkCreator.getLink();
	}

	public void setOptionalExpressionDeclaration(
			IGenerator<? extends Expression> optionalExpressionDeclaration) {
		this.optionalExpressionDeclaration = optionalExpressionDeclaration;
	}
	
	public static ClassInstanceCreationGenerator getNewDefaultClassInstanceCreationGenerator() {
		AST ast = ast();
		Chain<SimpleType> chainSimpleType = new Chain<SimpleType>(ast.newSimpleType(ast.newSimpleName("Exception")));		
		Chain<String> chainOfStrings = new Chain<String>();
		chainOfStrings.add("\"name1\"");
		chainOfStrings.add("\"name2\"");
		chainOfStrings.add("\"name3\"");
		StringLiteralGenerator stringLiteralGen = new StringLiteralGenerator(chainOfStrings);
		ClassInstanceCreationGenerator classInstanceGen = new ClassInstanceCreationGenerator(chainSimpleType);
		classInstanceGen.addArgumentGenerator(stringLiteralGen);
		return classInstanceGen;
	}

}
