package edu.uiuc.astgen.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.PackageDeclaration;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;

/**
 * Generates a compilation unit with a series of type declarations
 */
public class CompilationUnitGenerator extends
		ASTNodeGeneratorBase<CompilationUnit> {

	private ASTNodeGeneratorBase<PackageDeclaration> _packageGen;
	private ArrayList<IGenerator<? extends AbstractTypeDeclaration>> _typeGenList;
	
	public CompilationUnitGenerator() {
		this(new ASTNodeLiteral<PackageDeclaration>(null));
	}
	
	public CompilationUnitGenerator(ASTNodeGeneratorBase<PackageDeclaration> packageGen) {
		super();
		_typeGenList = new ArrayList<IGenerator<? extends AbstractTypeDeclaration>>();
		_packageGen = packageGen;
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator linkCreator = new LinkCreator(new CombinatoricLink());
		linkCreator.add(_packageGen);
		for (IGenerator gen : _typeGenList) {
			linkCreator.add(gen);
		}
		return linkCreator.getLink();
	}
	
	public void setPackageNameGenerator(ASTNodeGeneratorBase<PackageDeclaration> packageGen) {
		_packageGen = packageGen;
	}
	
	public <DeclarationType extends AbstractTypeDeclaration> void addDeclarationGenerator(IGenerator<DeclarationType> declarationGen) {
		_typeGenList.add(declarationGen);
	}

	@Override
	protected CompilationUnit generateCurrent() {
		CompilationUnit cu = super.getAST().newCompilationUnit();
		PackageDeclaration packageName = _packageGen.current();
		if (packageName != null) { 
			cu.setPackage(packageName);
		}
		for (IGenerator<? extends AbstractTypeDeclaration> gen : _typeGenList) {
			cu.types().add(gen.current());
		}		
		return cu;
	}

}
