package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.EnhancedForStatement;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class EnhancedForStatementGenerator extends ASTNodeGeneratorBase<EnhancedForStatement>{
	private IGenerator<SingleVariableDeclaration> parameter;
	private IGenerator<? extends Expression> expression;
	private IGenerator<? extends Block> block;
	
	public EnhancedForStatementGenerator(IGenerator<SingleVariableDeclaration> parameter, IGenerator<? extends Expression> expression) {
		super();
		this.parameter = parameter;
		this.expression = expression;
	}	
	
	@Override
	protected EnhancedForStatement generateCurrent() {
		EnhancedForStatement enhancedFor = super.getAST().newEnhancedForStatement();
		enhancedFor.setParameter(parameter.current());
		enhancedFor.setExpression(expression.current());
		if (block != null)
			enhancedFor.setBody(block.current());
		return enhancedFor;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(new CombinatoricLink());
		lc.add(parameter);
		lc.add(expression);
		if (block != null)
			lc.add(block);
		return lc.getLink();
	}

	public void setBlock(IGenerator<? extends Block> block) {
		this.block = block;
	}

	public static EnhancedForStatementGenerator getNewDefaultEnhancedForStatementGenerator() {
		SimpleNameGenerator parameterGen = new SimpleNameGenerator("parameter");
		SimpleNameGenerator nameGen = new SimpleNameGenerator("v1");
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(
				PrimitiveType.BOOLEAN, PrimitiveType.BYTE);//		
		SingleVariableDeclarationGenerator variableGen = new SingleVariableDeclarationGenerator(nameGen, primitiveTypeGen);	
		
		PrimitiveTypeGenerator primitiveTypeGen3 = new PrimitiveTypeGenerator(PrimitiveType.BYTE);//		
		SimpleNameGenerator nameGen3 = new SimpleNameGenerator("v1");
		SingleVariableDeclarationGenerator variableGen3 = new SingleVariableDeclarationGenerator(nameGen3, primitiveTypeGen3);
		AssignmentExpressionForVariableGenerator statementGen =new AssignmentExpressionForVariableGenerator(variableGen3);
		
		BlockGenerator blockGen = new BlockGenerator();
		blockGen.addStatementGenerator(statementGen);
		
		EnhancedForStatementGenerator enhancedForGen = new EnhancedForStatementGenerator(variableGen, parameterGen);
		enhancedForGen.setBlock(blockGen);
		return enhancedForGen;
	}

}
