package edu.uiuc.astgen.basic;

import java.util.LinkedList;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ForStatement;
import org.eclipse.jdt.core.dom.Statement;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * 
 * @author Kely
 * This class requires these 4 fields: initializers, condition, updaters, and a block statement
 * There are 2 methods that allow to add expressions to initializers and updaters
 */
public class ForStatementGenerator extends ASTNodeGeneratorBase<ForStatement>{	
	private java.util.LinkedList<ASTNodeGeneratorBase<? extends Expression>> initializersDeclaration;
	private java.util.LinkedList<ASTNodeGeneratorBase<? extends Expression>> updatersDeclaration;
	private ASTNodeGeneratorBase<? extends Statement> statementDeclaration;
	private ASTNodeGeneratorBase<? extends Expression> conditionDeclaration;
	
	public ForStatementGenerator(
			ASTNodeGeneratorBase<? extends Expression> conditionGen, 
			ASTNodeGeneratorBase<? extends Statement> statementGen) {
		conditionDeclaration = conditionGen;
		statementDeclaration = statementGen;	
		initializersDeclaration = new LinkedList<ASTNodeGeneratorBase<? extends Expression>>();
		updatersDeclaration = new LinkedList<ASTNodeGeneratorBase<? extends Expression>>();
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(new CombinatoricLink());
		lc.add(conditionDeclaration);
		lc.add(statementDeclaration);
		lc.addAll(initializersDeclaration);
		lc.addAll(updatersDeclaration);
		return lc.getLink();
	}
	
	public void addInitializerExpression(ASTNodeGeneratorBase<? extends Expression> newExpressionGen) {
		initializersDeclaration.add(newExpressionGen);
	}
	
	public void addUpdaterExpression(ASTNodeGeneratorBase<? extends Expression> newExpressionGen) {
		updatersDeclaration.add(newExpressionGen);
	}
	
	@Override
	protected ForStatement generateCurrent() {		
		ForStatement forStatement  = super.getAST().newForStatement();	
		//System.out.println("if expression: "+ infixDeclaration.current());
		for (ASTNodeGeneratorBase<? extends Expression> expressionGen : initializersDeclaration) {
			forStatement.initializers().add(expressionGen.current());	
		}
		for (ASTNodeGeneratorBase<? extends Expression> expressionGen : updatersDeclaration) {
			forStatement.updaters().add(expressionGen.current());	
		}		
		//System.out.println("after setting the variable");
		forStatement.setExpression(conditionDeclaration.current());
		forStatement.setBody(statementDeclaration.current());		
		return forStatement;		
	}
}
