package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.NumberLiteral;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.ReturnStatement;
import org.eclipse.jdt.core.dom.SimpleType;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.testorrery.Chain;
import org.testorrery.CompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;

public class GetterSetterGenerator extends CompositeGenerator<MethodDeclaration> {
	Chain<TypeOfMethod> typeFieldGetterSetterChain = new Chain<TypeOfMethod>(new Setter(), new SetterWithReturnType(), new Getter()); // new GetterSetter()
	private IGenerator<Modifier> visibilityModifierGen;
			
	private String fieldName;
	private IGenerator<? extends Type> typeGen;
	AST ast = ASTNodeGeneratorBase.ast();

	public GetterSetterGenerator (String fieldName) {
		this.fieldName = fieldName;
		this.visibilityModifierGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, (Modifier.ModifierKeyword)null);
		this.typeGen = new ASTNodeLiteral<SimpleType>(ast.newSimpleType(ast.newName("Integer")));
	}
	
	@Override
	protected MethodDeclaration generateCurrent() {
		TypeOfMethod typeFieldGetterSetterBuilder = typeFieldGetterSetterChain.current();
		Type fieldType = typeGen.current();		
		Modifier modGetterSetter = visibilityModifierGen.current();
		
		MethodDeclaration getter_setter = typeFieldGetterSetterBuilder.build(fieldName, fieldType, modGetterSetter);
		return getter_setter;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator linkCreator = new LinkCreator(new CombinatoricLink());
		linkCreator.add(typeGen);
		linkCreator.add(visibilityModifierGen);
		linkCreator.add(typeFieldGetterSetterChain);		
		return linkCreator.getLink();
	}

	private static abstract class TypeOfMethod {
		public static AST ast = ASTNodeGeneratorBase.ast();

		abstract MethodDeclaration build(String fieldName, Type fieldType, Modifier modMethod);

	}

	private class Getter extends TypeOfMethod {
		@Override
		MethodDeclaration build(String fieldName, Type fieldType, Modifier modMethod) {
			MethodDeclaration getter = ast.newMethodDeclaration();
			if (modMethod != null)
				getter.modifiers().add(modMethod);
			if (fieldName.length() == 1)
				getter.setName(ast.newSimpleName("get" + fieldName.toUpperCase().charAt(0)));
			else 
				getter.setName(ast.newSimpleName("get" + fieldName.toUpperCase().charAt(0) + fieldName.substring(1)));
			getter.setReturnType2((Type)ASTNode.copySubtree(ast, fieldType));
			ReturnStatement returnStatement = ast.newReturnStatement();
			returnStatement.setExpression(ast.newSimpleName(fieldName));
			
			Block methBody = ast.newBlock();
			methBody.statements().add((ReturnStatement)ASTNode.copySubtree(ast, returnStatement));
			getter.setBody(methBody);			
			//getter.getBody().statements().add(returnStatement);
			return getter;
		}
	}

	private class Setter extends TypeOfMethod {
		@Override
		MethodDeclaration build(String fieldName, Type fieldType, Modifier modMethod) {
			MethodDeclaration setter = ast.newMethodDeclaration();
			if (modMethod != null)
				setter.modifiers().add(modMethod);
			if (fieldName.length() == 1)
				setter.setName(ast.newSimpleName("set" + fieldName.toUpperCase().charAt(0)));
			else 
				setter.setName(ast.newSimpleName("set" + fieldName.toUpperCase().charAt(0) + fieldName.substring(1)));
			
			SingleVariableDeclaration svd = ast.newSingleVariableDeclaration();
			svd.setName(ast.newSimpleName(fieldName));
			svd.setType((Type)ASTNode.copySubtree(ast, fieldType));
			setter.parameters().add(svd);

			Assignment assignment = ast.newAssignment();
			FieldAccess fieldAccess = ast.newFieldAccess();
			fieldAccess.setExpression(ast.newThisExpression());
			fieldAccess.setName(ast.newSimpleName(fieldName));
			assignment.setLeftHandSide(fieldAccess);
			assignment.setRightHandSide(ast.newSimpleName(fieldName));
			
			ExpressionStatement expStat = ast.newExpressionStatement(assignment);
			Block methBody = ast.newBlock();
			methBody.statements().add((ExpressionStatement)ASTNode.copySubtree(ast, expStat));
			setter.setBody(methBody);
			//setter.getBody().statements().add(assignment);
			return setter;
		}
	}

	private class SetterWithReturnType extends TypeOfMethod {

		@Override
		MethodDeclaration build(String fieldName, Type fieldType, Modifier modMethod) {
			MethodDeclaration setter = ast.newMethodDeclaration();
			if (modMethod != null)
				setter.modifiers().add(modMethod);
			if (fieldName.length() == 1)
				setter.setName(ast.newSimpleName("set" + fieldName.toUpperCase().charAt(0)));
			else 
				setter.setName(ast.newSimpleName("set" + fieldName.toUpperCase().charAt(0) + fieldName.substring(1)));
			
			SingleVariableDeclaration svd = ast.newSingleVariableDeclaration();
			svd.setName(ast.newSimpleName(fieldName));
			svd.setType((Type)ASTNode.copySubtree(ast, fieldType));
			setter.parameters().add(svd);

			Assignment assignment = ast.newAssignment();
			FieldAccess fieldAccess = ast.newFieldAccess();
			fieldAccess.setExpression(ast.newThisExpression());
			fieldAccess.setName(ast.newSimpleName(fieldName));
			assignment.setLeftHandSide(fieldAccess);
			assignment.setRightHandSide(ast.newSimpleName(fieldName));
			
			ExpressionStatement expStat = ast.newExpressionStatement(assignment);
			Block methBody = ast.newBlock();
			methBody.statements().add((ExpressionStatement)ASTNode.copySubtree(ast, expStat));
			ReturnStatement retStat = ast.newReturnStatement();
			NumberLiteral numLit = ast.newNumberLiteral();
			numLit.setToken("2");
			retStat.setExpression(numLit);
			methBody.statements().add(retStat);
			setter.setBody(methBody);
			setter.setReturnType2(ast.newPrimitiveType(PrimitiveType.INT));
			//setter.getBody().statements().add(assignment);
			return setter;
		}

	}

	public void setTypeGen(IGenerator<? extends Type> typeGen) {
		this.typeGen = typeGen;
	}

	public IGenerator<? extends Type> getTypeGen() {
		return typeGen;
	}

	public void setVisibilityModifierGen(IGenerator<Modifier> visibilityModifierGen) {
		this.visibilityModifierGen = visibilityModifierGen;
	}
}
