package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.IfStatement;
import org.eclipse.jdt.core.dom.Statement;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class IfStatementGenerator extends ASTNodeGeneratorBase<IfStatement>{
	
	private ASTNodeGeneratorBase<? extends Expression> expressionDeclaration;
	private ASTNodeGeneratorBase<? extends Statement> thenStatementDeclaration;
	private ASTNodeGeneratorBase<? extends Statement> elseOptionalStatementDeclaration;
	
	public IfStatementGenerator(
			ASTNodeGeneratorBase<? extends Expression> expressionGen, 
			ASTNodeGeneratorBase<? extends Statement> thenStatementGen, 
			ASTNodeGeneratorBase<? extends Statement> elseOptionalGen) {
		super();
		expressionDeclaration = expressionGen;
		thenStatementDeclaration = thenStatementGen;
		elseOptionalStatementDeclaration = elseOptionalGen;
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(new CombinatoricLink());
		lc.add(expressionDeclaration);
		lc.add(thenStatementDeclaration);
		lc.add(elseOptionalStatementDeclaration);
		return lc.getLink();
	}
	
	@Override
	protected IfStatement generateCurrent() {
		IfStatement ifStatement  = super.getAST().newIfStatement();	
		//System.out.println("if expression: "+ expressionDeclaration.current());
		ifStatement.setExpression(expressionDeclaration.current());
		ifStatement.setThenStatement(thenStatementDeclaration.current());	
		ifStatement.setElseStatement(elseOptionalStatementDeclaration.current());	
		return ifStatement;		
	}
}
