package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.BooleanLiteral;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.InfixExpression;
import org.eclipse.jdt.core.dom.NumberLiteral;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.testorrery.link.GeneratorLink;
import org.testorrery.link.IIterationLink;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * 
 * @author Kely
 * This class requires: an OperatorGenerator, statementGenerator 
 */
public class InfixExpressionGenerator extends ASTNodeGeneratorBase<InfixExpression>{
	private ASTNodeGeneratorBase<SingleVariableDeclaration> variableDeclaration;
	
	public InfixExpressionGenerator(
			ASTNodeGeneratorBase<SingleVariableDeclaration> variableGen) {
		super();
		variableDeclaration = variableGen;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return new GeneratorLink(variableDeclaration);
	}
	
	@Override
	protected InfixExpression generateCurrent() {
		SingleVariableDeclaration variable = variableDeclaration.current();
		AST ast = super.getAST();
		InfixExpression infixExpression = ast.newInfixExpression();
		
		
		SimpleName name = variable.getName();
		if (name.getParent() != null)
			name = (SimpleName)name.copySubtree(ast, name);

		infixExpression.setLeftOperand(name);
		
		Expression expression=null;
		// later move those generators to (static) fields
		if (variable.getType().toString().equals("boolean")) {
			BooleanLiteralGenerator b = new BooleanLiteralGenerator();
			b.next();
			expression = (BooleanLiteral)b.current();
		} else if (variable.getType().toString().equals("byte")) {
			NumberLiteralGenerator b = new NumberLiteralGenerator(1, PrimitiveType.BYTE);
			b.next();
			expression = (NumberLiteral)b.current();
		}
		infixExpression.setOperator(InfixExpression.Operator.EQUALS);
		infixExpression.setRightOperand(expression);
			
		return infixExpression;		
	}

	public void reset() {
		super.reset();
		variableDeclaration.reset();
	}	
}
