package edu.uiuc.astgen.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.BodyDeclaration;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class InterfaceGenerator extends ASTNodeGeneratorBase<TypeDeclaration> {
	private ArrayList<ASTNodeGeneratorBase<? extends BodyDeclaration>> bodyDeclarationGenerators;
	private ASTNodeGeneratorBase<SimpleName> identifierGen;
	private ASTNodeGeneratorBase<Modifier> visibilityModifierGen;
	private ASTNodeGeneratorBase<Modifier> interfaceModifierGen;
	
	public InterfaceGenerator(ASTNodeGeneratorBase<SimpleName> identifierGen) {
		this(identifierGen, new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD));
	}

	public InterfaceGenerator(
			ASTNodeGeneratorBase<SimpleName> identifierGen, 
			ASTNodeGeneratorBase<Modifier> visibilityModifierGen) {
		this(identifierGen, visibilityModifierGen, new ModifierGenerator((Modifier.ModifierKeyword)null));
	}
	
	public InterfaceGenerator(
			ASTNodeGeneratorBase<SimpleName> identifierGen, 
			ASTNodeGeneratorBase<Modifier> visibilityModifierGen, 
			ASTNodeGeneratorBase<Modifier> classModifierGen) {
		super();

		this.identifierGen = identifierGen;
		this.visibilityModifierGen = visibilityModifierGen;
		interfaceModifierGen = classModifierGen;	
		bodyDeclarationGenerators = new ArrayList<ASTNodeGeneratorBase<? extends BodyDeclaration>>();	
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(new CombinatoricLink());
		lc.add(identifierGen);
		lc.add(visibilityModifierGen);
		lc.add(interfaceModifierGen);
		lc.addAll(bodyDeclarationGenerators);
		return lc.getLink();
	}
	
	public void addBodyDeclarationGenerator(ASTNodeGeneratorBase<? extends BodyDeclaration> bodyDeclarationGenerator) {
		bodyDeclarationGenerators.add(bodyDeclarationGenerator);
	}	

	@Override
	protected TypeDeclaration generateCurrent() {
		TypeDeclaration toReturn = super.getAST().newTypeDeclaration();
		toReturn.setInterface(true);
		toReturn.setName(identifierGen.current());
		
		Modifier visibilityModifier = visibilityModifierGen.current();
		if (visibilityModifier != null) {
			toReturn.modifiers().add(visibilityModifier);
		}
		
		Modifier classModifier = interfaceModifierGen.current();
		if (classModifier != null) {
			toReturn.modifiers().add(classModifier);	
		}
		
		for (IGenerator<? extends BodyDeclaration> bodyDeclGen : bodyDeclarationGenerators) {
			toReturn.bodyDeclarations().add(bodyDeclGen.current());
		}
		return toReturn;

	}

}
