package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
import org.testorrery.link.GeneratorLink;
import org.testorrery.link.IIterationLink;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;

/**
 * Generates modifiers used in various contexts.
 */
public class ModifierGenerator extends ASTNodeGeneratorBase<Modifier> {
	
	public static ModifierGenerator createVisibilityModifier() {
		return new ModifierGenerator(
				(ModifierKeyword)null, 
				ModifierKeyword.PUBLIC_KEYWORD, 
				ModifierKeyword.PROTECTED_KEYWORD, 
				ModifierKeyword.PRIVATE_KEYWORD);
	}

	public static ModifierGenerator createLocalVariableModifier() {
		return new ModifierGenerator(
				(ModifierKeyword)null, 
				ModifierKeyword.FINAL_KEYWORD);
	}

	public static ModifierGenerator createClassModifier() {
		return new ModifierGenerator(
				(ModifierKeyword)null, 
				ModifierKeyword.STATIC_KEYWORD,
				ModifierKeyword.ABSTRACT_KEYWORD,
				ModifierKeyword.FINAL_KEYWORD,
				ModifierKeyword.NATIVE_KEYWORD,
				ModifierKeyword.SYNCHRONIZED_KEYWORD,
				ModifierKeyword.TRANSIENT_KEYWORD,
				ModifierKeyword.VOLATILE_KEYWORD,
				ModifierKeyword.STRICTFP_KEYWORD);		
	}

	public static ModifierGenerator createMethodModifier() {
		return new ModifierGenerator(
				(ModifierKeyword)null, 
				ModifierKeyword.STATIC_KEYWORD,
				ModifierKeyword.ABSTRACT_KEYWORD,
				ModifierKeyword.FINAL_KEYWORD,
				ModifierKeyword.NATIVE_KEYWORD,
				ModifierKeyword.SYNCHRONIZED_KEYWORD,
				ModifierKeyword.TRANSIENT_KEYWORD,
				ModifierKeyword.VOLATILE_KEYWORD,
				ModifierKeyword.STRICTFP_KEYWORD);		
	}

	public static ModifierGenerator createFieldModifier() {
		return new ModifierGenerator(
				(ModifierKeyword)null, 
				ModifierKeyword.STATIC_KEYWORD,
				ModifierKeyword.FINAL_KEYWORD,
				ModifierKeyword.TRANSIENT_KEYWORD,
				ModifierKeyword.VOLATILE_KEYWORD);
	}
	
	private ASTNodeGeneratorBase<Modifier> _modifierGen;
	
	public ModifierGenerator(ModifierKeyword... keywords) {
		super();
		ASTNodeChain<Modifier> modifierSequence = new ASTNodeChain<Modifier>();
		Modifier modifier;
		for (ModifierKeyword keyword : keywords) {
			if (keyword == null) {
				modifier = null;
			}
			else {
				modifier = super.getAST().newModifier(keyword);
			}
			modifierSequence.add(new ASTNodeLiteral<Modifier>(modifier));
		}
		_modifierGen = modifierSequence;
	}
	
	public ModifierGenerator(ASTNodeGeneratorBase<Modifier> modifierGen) {
		super();
		_modifierGen = modifierGen;
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		return new GeneratorLink(_modifierGen);
	}
	
	@Override
	protected Modifier generateCurrent() {
		return _modifierGen.current();
	}
}
