package edu.uiuc.astgen.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SimpleType;
import org.eclipse.jdt.core.dom.StringLiteral;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.testorrery.Chain;
import org.testorrery.CompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;
import edu.uiuc.astgen.fieldreferences.FieldReferenceType;


public class MultipleFieldDeclarationGenerator extends CompositeGenerator<ArrayList<FieldDeclaration>> {
	Chain<TypeOfFieldDeclaration> typeFieldDeclarationChain = new Chain<TypeOfFieldDeclaration>(new FieldsTogether(), new FieldsSeparate());
	Chain<TypeOfFieldReference> typeFieldReferenceChain = new Chain<TypeOfFieldReference>(new FieldsReferenced());//, new FieldsNotReferenced());
	
	private IGenerator<FieldReferenceType> fieldRefTypeGen;
	private IGenerator<? extends Type> typeGen;
	
	public String f;
	public String g;
	public TypeDeclaration typeDeclaration;
	public boolean singleClass;
	
	public MultipleFieldDeclarationGenerator (String f, String g, TypeDeclaration typeDecl, boolean singleClass) {
		this.f = f;
		this.g = g;		
		this.typeDeclaration = typeDecl;
		this.singleClass = singleClass;
		this.typeGen = new ASTNodeLiteral<SimpleType>(ast.newSimpleType(ast.newName("Object")));
	}	

	public void setTypeGen(IGenerator<? extends Type> typeGen) {
		this.typeGen = typeGen;
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		if (fieldRefTypeGen == null) {
			if (!singleClass) {
				fieldRefTypeGen = new Chain<FieldReferenceType>(
						FieldReferenceType.NEW_INSTANCE, 
						FieldReferenceType.QUALIFIED_THIS, 
						FieldReferenceType.THIS,
						FieldReferenceType.UNQUALIFIED,
						FieldReferenceType.SUPER);
			} else {
				fieldRefTypeGen = new Chain<FieldReferenceType>(
						FieldReferenceType.NEW_INSTANCE, 
						FieldReferenceType.QUALIFIED_THIS, 
						FieldReferenceType.THIS,
						FieldReferenceType.UNQUALIFIED);				
			}
		}
		
		LinkCreator linkCreator = new LinkCreator(new CombinatoricLink());
		linkCreator.add(typeGen);
		linkCreator.add(fieldRefTypeGen);
		linkCreator.add(typeFieldDeclarationChain);
		linkCreator.add(typeFieldReferenceChain);
		return linkCreator.getLink();
	}
	
	AST ast = ASTNodeGeneratorBase.ast();
	
	@Override
	protected ArrayList<FieldDeclaration> generateCurrent() {				
		TypeOfFieldDeclaration typeFieldDeclBuilder = typeFieldDeclarationChain.current();
		TypeOfFieldReference typeFieldRefBuilder = typeFieldReferenceChain.current();
		Type fieldType = typeGen.current();
		FieldReferenceType fieldRefType;
		Expression fieldRef = null;
		
		fieldRefType = fieldRefTypeGen.current();
		// build field reference (i.e. field, this.field, super.field, etc.)
		fieldRef = fieldRefType.getBuilder().build(ast.newSimpleName(f), typeDeclaration);			
							
		ArrayList<FieldDeclaration> listFieldDeclarations = typeFieldDeclBuilder.build(f, g, fieldType);		
		ArrayList<FieldDeclaration> listFieldDeclarations2 = typeFieldRefBuilder.build(listFieldDeclarations, fieldRef);		
		return listFieldDeclarations2;
	}
	
	private static abstract class TypeOfFieldReference {
		AST ast = ASTNodeGeneratorBase.ast();
		abstract ArrayList<FieldDeclaration> build(ArrayList<FieldDeclaration> listFields, Expression fieldReference);		
	}
	
	private class FieldsReferenced extends TypeOfFieldReference {
		@Override
		ArrayList<FieldDeclaration> build(ArrayList<FieldDeclaration> listFields, Expression fieldReference) {
			ArrayList<FieldDeclaration> listReferencedFields = new ArrayList<FieldDeclaration>();
			FieldDeclaration field;
			VariableDeclarationFragment vdf_g;
			FieldDeclaration auxField = listFields.get(0);
			VariableDeclarationFragment auxVdf = (VariableDeclarationFragment) auxField.fragments().get(0);			
			SimpleName reference_f = auxVdf.getName(); //ast.newSimpleName(listFields.get(0).fragments().get(0).toString());
			if (listFields.size() == 1) {
				vdf_g = (VariableDeclarationFragment) listFields.get(0).fragments().get(1);
				if (fieldReference != null)
					vdf_g.setInitializer((Expression)ASTNode.copySubtree(ast, fieldReference));
				else 
					vdf_g.setInitializer((SimpleName)ASTNode.copySubtree(ast, reference_f));
				field = listFields.get(0);
				field.fragments().remove(1);
				field.fragments().add(vdf_g);
				listReferencedFields.add(field);
			}	
			else if (listFields.size() == 2) {
				vdf_g = (VariableDeclarationFragment) listFields.get(1).fragments().get(0);
				if (fieldReference != null)
					vdf_g.setInitializer((Expression)ASTNode.copySubtree(ast, fieldReference));
				else
					vdf_g.setInitializer((SimpleName)ASTNode.copySubtree(ast, reference_f));
				field = listFields.get(1);
				field.fragments().clear();
				field.fragments().add(vdf_g);
				listReferencedFields.add(listFields.get(0));
				listReferencedFields.add(field);
			}
			return listReferencedFields;
		}		
	}
	
	private class FieldsNotReferenced extends TypeOfFieldReference {
		@Override
		ArrayList<FieldDeclaration> build(ArrayList<FieldDeclaration> listFields, Expression fieldReference) {
			return listFields;
		}		
	}
		
	private static abstract class TypeOfFieldDeclaration {
		public static AST ast = ASTNodeGeneratorBase.ast();
		//abstract ArrayList<FieldDeclaration> build(String field1, String field2, Type type, Expression reference);
		abstract ArrayList<FieldDeclaration> build(String field1, String field2, Type type);
		
		public static VariableDeclarationFragment vdf_F;
		public static VariableDeclarationFragment vdf_G;
		
		public static void initializeFields(String field1, String field2) {
			vdf_F = ast.newVariableDeclarationFragment();
			vdf_F.setName(ast.newSimpleName(field1));
			vdf_G = ast.newVariableDeclarationFragment();
			vdf_G.setName(ast.newSimpleName(field2));			
		}		
	}
	
	private class FieldsTogether extends TypeOfFieldDeclaration {
		@Override
		ArrayList<FieldDeclaration> build(String field1, String field2, Type type) {
			initializeFields(field1, field2);
			ArrayList<FieldDeclaration> listFields = new ArrayList<FieldDeclaration>();
			VariableDeclarationFragment vdf_f1 = vdf_F;
			if (type.toString().equals("int") || type.toString().equals("Integer")){
				vdf_f1.setInitializer(ast.newNumberLiteral("10"));
			} else if (type.toString().equals("boolean") || type.toString().equals("Boolean")) {
				vdf_f1.setInitializer(ast.newBooleanLiteral(true));
			} else if (type.toString().equals("String")) {
				StringLiteral s = ast.newStringLiteral();
				s.setEscapedValue("\"abcd\"");
				vdf_f1.setInitializer(s);
			}
			FieldDeclaration fieldDeclaration1 = ast.newFieldDeclaration(vdf_f1);
			//fieldDeclaration1.setType(ast.newPrimitiveType(PrimitiveType.INT));
			fieldDeclaration1.setType((Type)ASTNode.copySubtree(ast, type));
			VariableDeclarationFragment vdf_f2 = vdf_G;
			//vdf_f2.setInitializer(reference);
			
			fieldDeclaration1.fragments().add(vdf_f2);
			listFields.add(fieldDeclaration1);
			return listFields;
		}
	}
	
	private class FieldsSeparate extends TypeOfFieldDeclaration {
		@Override
		ArrayList<FieldDeclaration> build(String field1, String field2, Type type) {
			ArrayList<FieldDeclaration> listFields = new ArrayList<FieldDeclaration>();
			VariableDeclarationFragment vdf_f1 = vdf_F;
			if (vdf_f1.getParent() != null) { 
				vdf_f1 = (VariableDeclarationFragment)vdf_f1.copySubtree(ast, vdf_f1);
				if (type.toString().equals("int") || type.toString().equals("Integer")){
					vdf_f1.setInitializer(ast.newNumberLiteral("10"));
				} else if (type.toString().equals("boolean") || type.toString().equals("Boolean")) {
					vdf_f1.setInitializer(ast.newBooleanLiteral(true));
				} else if (type.toString().equals("String")) {
					StringLiteral s = ast.newStringLiteral();
					s.setEscapedValue("\"abcd\"");
					vdf_f1.setInitializer(s);
				}			}
			FieldDeclaration fieldDeclaration1 = ast.newFieldDeclaration(vdf_f1);
			//fieldDeclaration1.setType(ast.newPrimitiveType(PrimitiveType.INT));
			fieldDeclaration1.setType((Type)ASTNode.copySubtree(ast, type));
			
			VariableDeclarationFragment vdf_f2 = vdf_G;
			//vdf_f2.setInitializer(reference);
			if (vdf_f2.getParent() != null) vdf_f2 = (VariableDeclarationFragment)vdf_f2.copySubtree(ast, vdf_f2);			
			FieldDeclaration fieldDeclaration2 = ast.newFieldDeclaration(vdf_f2);			
			//fieldDeclaration2.setType(ast.newPrimitiveType(PrimitiveType.INT));
			fieldDeclaration2.setType((Type)ASTNode.copySubtree(ast, type));
			
			listFields.add(fieldDeclaration1);
			listFields.add(fieldDeclaration2);
			return listFields;
		}
	}

	public void setFieldRefTypeGen(IGenerator<FieldReferenceType> fieldRefTypeGen) {
		this.fieldRefTypeGen = fieldRefTypeGen;
	}	

}
