package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.PostfixExpression;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.testorrery.Chain;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class PostFixExpressionGenerator extends ASTNodeGeneratorBase<PostfixExpression>{
	private ASTNodeGeneratorBase<SingleVariableDeclaration> variableDeclaration;
	private Chain<PostfixExpression.Operator> operatorGenerator;
	
	public PostFixExpressionGenerator(
			ASTNodeGeneratorBase<SingleVariableDeclaration> variableGen) {
		super();
		variableDeclaration = variableGen;
		operatorGenerator = new Chain<PostfixExpression.Operator>();		
		operatorGenerator.add(PostfixExpression.Operator.INCREMENT);
		operatorGenerator.add(PostfixExpression.Operator.DECREMENT);				
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(operatorGenerator, variableDeclaration);
	}
	
	@Override
	protected PostfixExpression generateCurrent() {
		SingleVariableDeclaration variable = variableDeclaration.current();
		AST ast = super.getAST();
		
		
		PostfixExpression postfixExpression = ast.newPostfixExpression();
				
		SimpleName name = variable.getName();
		if (name.getParent() != null)
			name = (SimpleName)name.copySubtree(ast, name);

		postfixExpression.setOperand(name);		
		postfixExpression.setOperator(operatorGenerator.current());		
		return postfixExpression;		
	}		
}
