package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.PrimitiveType;
import org.testorrery.Chain;
import org.testorrery.Literal;
import org.testorrery.link.GeneratorLink;
import org.testorrery.link.IIterationLink;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * Generates Java's primitive types
 */
public class PrimitiveTypeGenerator extends ASTNodeGeneratorBase<PrimitiveType> {
	
	public static final PrimitiveType.Code[] primitiveTypes = new PrimitiveType.Code[] {
		PrimitiveType.BOOLEAN,
		PrimitiveType.INT,
		PrimitiveType.BYTE,
		PrimitiveType.SHORT,
		PrimitiveType.CHAR,
		PrimitiveType.LONG,
		PrimitiveType.FLOAT,
		PrimitiveType.DOUBLE		
	};
	
	public static PrimitiveTypeGenerator getAllTypes() {
		return new PrimitiveTypeGenerator(primitiveTypes);
	}
		
	private Chain<PrimitiveType.Code> _typeGenerator;
	
	public PrimitiveTypeGenerator() {
		this(primitiveTypes);
	}
	
	public PrimitiveTypeGenerator(PrimitiveType.Code...codes) {
		super();
		_typeGenerator = new Chain<PrimitiveType.Code>();
		for (PrimitiveType.Code code : codes) {
			_typeGenerator.add(new Literal<PrimitiveType.Code>(code));
		}			
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return new GeneratorLink(_typeGenerator);
	}

	@Override
	protected PrimitiveType generateCurrent() {
		return super.getAST().newPrimitiveType(_typeGenerator.current());
	}
}
