package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;

/**
 * Generates local variable declarations
 */
public class SingleVariableDeclarationGenerator extends
		ASTNodeGeneratorBase<SingleVariableDeclaration> {
	
	private IGenerator<SimpleName> nameGen;
	private IGenerator<Modifier> localVariableModifierGen; // null or "final"
	private IGenerator<? extends Type> typeGen;
	
	public SingleVariableDeclarationGenerator(String name) {
		nameGen = new ASTNodeLiteral<SimpleName>(ast().newSimpleName(name));
	}
		
	public SingleVariableDeclarationGenerator(IGenerator<SimpleName> nameGen, IGenerator<? extends Type> typeGen) {
		this.nameGen = nameGen;
		this.typeGen = typeGen;
	}

	public SingleVariableDeclarationGenerator(IGenerator<SimpleName> nameGen, IGenerator<? extends Type> typeGen, IGenerator<Modifier> localVariableModifierGen) {
		this(nameGen, typeGen);
		this.localVariableModifierGen = localVariableModifierGen;
	}

	public void setLocalVariableModifierGen(
			IGenerator<Modifier> localVariableModifierGen) {
		this.localVariableModifierGen = localVariableModifierGen;
	}
	
	public void setTypeGen(IGenerator<? extends Type> typeGen) {
		this.typeGen = typeGen;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		if (localVariableModifierGen == null) {
			localVariableModifierGen = new ASTNodeLiteral<Modifier>(null);
		}
		if (typeGen == null) {
			typeGen = new ASTNodeLiteral<Type>(ast().newSimpleType(ast().newSimpleName("Object")));
		}
		return LinkCreator.createCombinatoricLink(nameGen, localVariableModifierGen, typeGen);
	}

	@Override
	protected SingleVariableDeclaration generateCurrent() {		
		SingleVariableDeclaration localVariable = super.getAST().newSingleVariableDeclaration();
		localVariable.setName(nameGen.current());
		localVariable.setType(typeGen.current());
		Modifier modifier = localVariableModifierGen.current();
		if (modifier != null) {
			localVariable.modifiers().add(modifier);
		}
		
		return localVariable;
	}
}
