package edu.uiuc.astgen.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.SwitchCase;
import org.eclipse.jdt.core.dom.SwitchStatement;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class SwitchStatementGenerator extends ASTNodeGeneratorBase<SwitchStatement>{
	private ASTNodeGeneratorBase<? extends Expression> switchExpressionDeclaration;
	private ArrayList<IGenerator<? extends Statement>> statements;
	
	public SwitchStatementGenerator(ASTNodeGeneratorBase<? extends Expression> expressionGen) {
		super();
		switchExpressionDeclaration = expressionGen;
		statements = new ArrayList<IGenerator<? extends Statement>>();
	}	

	public void addStatement(IGenerator<? extends Statement> statementDeclarationGenerator) {
		statements.add(statementDeclarationGenerator);
	}
	
	@Override
	protected SwitchStatement generateCurrent() {
		SwitchStatement switchStatement = super.getAST().newSwitchStatement();
		switchStatement.setExpression(switchExpressionDeclaration.current());
		for (IGenerator<? extends Statement> statementGen : statements) {
			switchStatement.statements().add(statementGen.current());			
		}		
		return switchStatement;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator linkCreator = new LinkCreator(new CombinatoricLink());
		linkCreator.add(switchExpressionDeclaration);
		linkCreator.addAll(statements);
		return linkCreator.getLink();
	}
	
	public static SwitchStatementGenerator getNewDefaultSwitchStatementGenerator(){
		SimpleNameGenerator simpleNameGen = new SimpleNameGenerator("month");
		NumberLiteralGenerator numberGen = new NumberLiteralGenerator(2, PrimitiveType.BYTE);
		SwitchCaseGenerator switchCaseGen = new SwitchCaseGenerator(numberGen);
		
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);		
		AssignmentExpressionGenerator assignmentExpression = new AssignmentExpressionGenerator(fieldDeclGen);		
		
		ASTNodeChain<SwitchCase> statementsSequence = new ASTNodeChain<SwitchCase>();		
		statementsSequence.add(switchCaseGen);
		ASTNodeChain<ExpressionStatement> varStatementsSequence = new ASTNodeChain<ExpressionStatement>();		
		varStatementsSequence.add(assignmentExpression);		
		
		SwitchStatementGenerator switchStatementGen = new SwitchStatementGenerator(simpleNameGen);
		switchStatementGen.addStatement(switchCaseGen);
		switchStatementGen.addStatement(varStatementsSequence);
		return switchStatementGen;
	}

}
