package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.BooleanLiteral;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.NumberLiteral;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.VariableDeclarationExpression;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.ASTNodeLiteral;

public class VariableDeclarationExpressionGenerator extends ASTNodeGeneratorBase<VariableDeclarationExpression>{

	private ASTNodeGeneratorBase<SimpleName> _nameGen;
	private ASTNodeGeneratorBase<Modifier> _localVariableModifierGen; // null or "final"
	private ASTNodeGeneratorBase<? extends Type> _typeGen;
	
	public VariableDeclarationExpressionGenerator(
			ASTNodeGeneratorBase<SimpleName> nameGen,
			ASTNodeGeneratorBase<? extends Type> typeGen) {
		this(nameGen, typeGen, new ASTNodeLiteral<Modifier>((Modifier)null));
	}
	
	public VariableDeclarationExpressionGenerator(
			ASTNodeGeneratorBase<SimpleName> nameGen, 
			ASTNodeGeneratorBase<? extends Type> typeGen, 
			ASTNodeGeneratorBase<Modifier> localVariableModifierGen) {
		super();
		
		_nameGen = nameGen;
		_typeGen = typeGen;
		_localVariableModifierGen = localVariableModifierGen;		
	}	
	
	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(
				_nameGen,
				_typeGen,
				_localVariableModifierGen);		
	}

	@Override
	protected VariableDeclarationExpression generateCurrent() {
		VariableDeclarationFragment variableDeclarationFragment = super.getAST().newVariableDeclarationFragment();
		variableDeclarationFragment.setName(_nameGen.current());
		VariableDeclarationExpression variableDeclarationExp = super.getAST().newVariableDeclarationExpression(variableDeclarationFragment);
		variableDeclarationExp.setType(_typeGen.current());
		
		Modifier modifier = _localVariableModifierGen.current();
		if (modifier != null) {
			variableDeclarationExp.modifiers().add(modifier);
		}
		
		// later move those generators to (static) fields
		Expression expression=null;
		if (variableDeclarationExp.getType().toString().equals("boolean")) {
			BooleanLiteralGenerator b = new BooleanLiteralGenerator();
			b.next();
			expression = (BooleanLiteral)b.current();
		} else if (variableDeclarationExp.getType().toString().equals("byte")) {
			NumberLiteralGenerator b = new NumberLiteralGenerator(1, PrimitiveType.BYTE);
			b.next();
			expression = (NumberLiteral)b.current();
		} 	
		variableDeclarationFragment.setInitializer(expression);		
		return variableDeclarationExp;
	}

}
