package edu.uiuc.astgen.basic;

import java.util.List;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class VariableDeclarationStatementGenerator extends ASTNodeGeneratorBase<VariableDeclarationStatement>{
	private  ASTNodeGeneratorBase<SingleVariableDeclaration> localVariableGenerator;
	private ASTNodeGeneratorBase<SimpleName> identifierGen;
	
	public VariableDeclarationStatementGenerator(ASTNodeGeneratorBase<SimpleName> identifierGen, ASTNodeGeneratorBase<SingleVariableDeclaration> localGen) {
		super();
		this.identifierGen = identifierGen;
		localVariableGenerator = localGen;
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(
				identifierGen,
				localVariableGenerator);		
	}

	@Override
	protected VariableDeclarationStatement generateCurrent() {
		AST ast = super.getAST();
		VariableDeclarationFragment vdf = ast.newVariableDeclarationFragment();
		vdf.setName(identifierGen.current());
		VariableDeclarationStatement vds = ast.newVariableDeclarationStatement(vdf);
		SingleVariableDeclaration variable = (SingleVariableDeclaration)localVariableGenerator.current();
		ASTNode type = variable.getType();
		if (type.getParent() != null)
			type = type.copySubtree(getAST(), type); 
		vds.setType((Type)type);				
		List modifiers = variable.modifiers();
		if (!modifiers.isEmpty()) {
			ASTNode modifier = (ASTNode)modifiers.get(0);
			if (modifier.getParent() != null)
				modifier = modifier.copySubtree(ast, modifier);
			vds.modifiers().add(modifier);
		}
		return vds;		
	}
}
