package edu.uiuc.astgen.classrelationships;

import org.eclipse.jdt.core.dom.TypeDeclaration;

/**
 * A {@link RelationshipSpecification} that combines some number of sub-specifications
 */
public class CompositeRelationship implements RelationshipSpecification {
	private RelationshipSpecification[] relationshipSpecifications;
	
	public CompositeRelationship(RelationshipSpecification...relationshipSpecifications) {
		this.relationshipSpecifications = relationshipSpecifications;
	}
	
	private final RelationshipBuilder builder = new RelationshipBuilder() {
		public void build(TypeDeclaration a, TypeDeclaration b) {
			for (RelationshipSpecification relationshipSpecification : relationshipSpecifications) {
				relationshipSpecification.getBuilder().build(a, b);
			}
		}
	};
	
	public RelationshipBuilder getBuilder() {
		return builder;
	}

	public String getDescription() {
		StringBuilder sb = new StringBuilder();
		for (RelationshipSpecification relationshipSpecification : relationshipSpecifications) {
			sb.append(relationshipSpecification.getDescription()).append('\n');
		}
		return sb.toString();		
	}

	public RelationshipDirection getDirection() {
		return RelationshipDirection.UNSPECIFIED;
	}

	public Reversibility getReversibility() {
		for (RelationshipSpecification relationshipSpecification : relationshipSpecifications) {
			if (relationshipSpecification.getReversibility() == Reversibility.IS_REVERSIBLE) {
				return Reversibility.IS_REVERSIBLE;
			}
		}
		return Reversibility.NOT_REVERSIBLE;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (RelationshipSpecification relationshipSpecification : relationshipSpecifications) {
			sb.append(relationshipSpecification.getDescription()).append(' ');
			sb.append(relationshipSpecification.getDirection()).append('\n');			
		}
		return sb.toString();
	}

}
