package edu.uiuc.astgen.classrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * Enum of ways in which one class can be related to another by inheritance
 */
public enum InheritanceType implements RelationshipSpecification {
	/**
	 * class A {}
	 * class B {}
	 */
	NO_INHERITANCE(new NoInheritanceBuilder(), RelationshipDirection.UNRELATED, Reversibility.NOT_REVERSIBLE),	
	/**
	 * class A {}
	 * class B extends A {}
	 */
	SUBCLASS(new SubclassBuilder(), RelationshipDirection.B_TO_A, Reversibility.IS_REVERSIBLE);
	
	private static class NoInheritanceBuilder implements RelationshipBuilder{
		public void build(TypeDeclaration a, TypeDeclaration b) {
			// do nothing
		}
	}
	
	private static class SubclassBuilder implements RelationshipBuilder{
		public void build(TypeDeclaration a, TypeDeclaration b) {
			// make B a subclass of A
			AST ast = ASTNodeGeneratorBase.ast();
			b.setSuperclassType(ast.newSimpleType(ast.newSimpleName(a.getName().toString())));
		}
	}
	
	private RelationshipBuilder builder;
	private RelationshipDirection direction;
	private Reversibility reversibility;
	
	private InheritanceType(
			RelationshipBuilder builder,
			RelationshipDirection direction,
			Reversibility reversibility) {
		this.builder = builder;
		this.direction = direction;
		this.reversibility = reversibility;
	}	
	public RelationshipBuilder getBuilder() {
		return builder;	
	}
	public String getDescription() {
		return this.toString();
	}
	public Reversibility getReversibility() {
		return this.reversibility;
	}
	public RelationshipDirection getDirection() {
		return this.direction;
	}
	
}