
package edu.uiuc.astgen.classrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * Enum of ways in which it is possible to place a class in relation to another.
 */
public enum LocationType implements RelationshipSpecification {
	/**
	  * class A {}
	  * class B {}
	  */
	SEPARATE(new SeparateBuilder(), RelationshipDirection.UNRELATED, Reversibility.NOT_REVERSIBLE), 
	/**
	 * class A {
	 *   class B {}
	 * }	 
	 */
	INNER(new InnerBuilder(), RelationshipDirection.B_TO_A, Reversibility.IS_REVERSIBLE), 
	/**
	 * class A {
	 *   void dummyMethod() {
	 *     class B {}
	 *   }
	 * }
	 */
	LOCAL(new LocalBuilder(), RelationshipDirection.B_TO_A, Reversibility.IS_REVERSIBLE);
	
	private static class SeparateBuilder implements RelationshipBuilder {
		public void build(TypeDeclaration a, TypeDeclaration b) {
			// do nothing
		}
	}
	
	private static class InnerBuilder implements RelationshipBuilder {
		public void build(TypeDeclaration a, TypeDeclaration b) {	
			a.bodyDeclarations().add(b);
		}
	}
		
	private static class LocalBuilder implements RelationshipBuilder {
		public void build(TypeDeclaration a, TypeDeclaration b) {	
			AST ast = ASTNodeGeneratorBase.ast();			
			
			MethodDeclaration methDecl = BuilderUtil.getExistingDummyMethod(a);
			methDecl.getBody().statements().add(ast.newTypeDeclarationStatement(b));;
		}
	}
	
	private RelationshipBuilder builder;
	private RelationshipDirection direction;
	private Reversibility reversibility;
	
	private LocationType(
			RelationshipBuilder builder,
			RelationshipDirection direction,
			Reversibility reversibility) {
		this.builder = builder;
		this.direction = direction;
		this.reversibility = reversibility;
	}	
	public RelationshipBuilder getBuilder() {
		return builder;	
	}
	public String getDescription() {
		return this.toString();
	}
	public Reversibility getReversibility() {
		return this.reversibility;
	}
	public RelationshipDirection getDirection() {
		return this.direction;
	}

}


