package edu.uiuc.astgen.classrelationships;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.testorrery.CompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.CombinatoricLink;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * Generates a list of type declarations
 */
public class MultiClassRelationshipGenerator extends CompositeGenerator<List<TypeDeclaration>> {
	private IGenerator<TypeDeclaration>[] classGens;
	private IGenerator<RelationshipSpecification>[][] builderGens;
	
	public MultiClassRelationshipGenerator(SpecificationCallback builderCallback, IGenerator<TypeDeclaration>... classGens) {
		this.classGens = classGens;
		int i, j;
		builderGens = new IGenerator[classGens.length][classGens.length];
		
		for (i = 0; i < classGens.length; i++) {
			for (j = 0; j < classGens.length; j++) {
				builderGens[i][j] = builderCallback.makeSpecificationGenerator();
			}
		}
	}	
	
	public static interface SpecificationCallback {
		IGenerator<RelationshipSpecification> makeSpecificationGenerator();
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(new CombinatoricLink());
		if (classGens.length < 2) {
			throw new IllegalArgumentException("Two or more class generators must be defined");
		}
		for (IGenerator<TypeDeclaration> classGen : classGens) {
			lc.add(classGen);
		}
		
		for (IGenerator[] b1 : builderGens) {
			for (IGenerator b2 : b1) {
				lc.add(b2);
			}
		}
		return lc.getLink();
	}

	@Override
	protected List<TypeDeclaration> generateCurrent() {
		int i, j;
		AST ast = ASTNodeGeneratorBase.ast();

		LinkedList<TypeDeclaration> toReturn = new LinkedList<TypeDeclaration>();
		
		// generate classes
		TypeDeclaration[] classes = new TypeDeclaration[classGens.length];
		for (i = 0; i < classGens.length; i++) {
			classes[i] = (TypeDeclaration)ASTNode.copySubtree(ast, classGens[i].current());
		}
		
		// generate relationships
		RelatedPair[][] currentRelationships = new RelatedPair[classGens.length][classGens.length];
		for (i = 0; i < classGens.length; i++) {
			for (j = 0; j < classGens.length; j++) {			
				try {
					builderGens[i][j].current().getBuilder().build(classes[i], classes[j]);
					currentRelationships[i][j] = new RelatedPair(classes[i], classes[j], builderGens[i][j].current());
				}
				catch (IllegalArgumentException e) {
					
				}
			}
		}
		
		// find only the top-level classes (those that are not inner or local
//		boolean isTop;		
//		for (i = 0; i < classGens.length; i++) {
//			isTop = true;
//			
//			for (j = 0; j < classGens.length; i++) {
//				if (currentRelationships[i][j].getLocationType() != ClassLocationType.SEPARATE && 
//						currentRelationships[i][j].getLocationTypeDirection() == RelationshipDirection.A_TO_B) {
//					isTop = false;
//					break;
//				}
//			}
//			
//			if (isTop) {
//				toReturn.add(classGens[i].current());
//			}
//		}
		
		return Arrays.asList(classes);
	}
}
