package edu.uiuc.astgen.classrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * Container for two {@link TypeDeclaration}s that are related by a particular {@link RelationshipSpecification}.
 * 
 * @see RelatedPairGenerator
 */
public class RelatedPair {
		
	private TypeDeclaration classA;
	private TypeDeclaration classB;
	private RelationshipSpecification relSpec;
		
	public RelatedPair(TypeDeclaration classA, TypeDeclaration classB, RelationshipSpecification relationshipSpecification) {
		this.classA = classA;
		this.classB = classB;
		this.relSpec = relationshipSpecification;
	}
	
	public TypeDeclaration getClassA() {
		return classA;
	}

	public TypeDeclaration getClassB() {
		return classB;
	}
	
	public RelationshipSpecification getRelationshipSpecification() {
		return relSpec;
	}
	
	public RelatedPair getCopy() {
		AST ast = ASTNodeGeneratorBase.ast(); 
		TypeDeclaration copyA = (TypeDeclaration)ASTNode.copySubtree(ast, this.classA);
		TypeDeclaration copyB = (TypeDeclaration)ASTNode.copySubtree(ast, this.classB);
		
		return new RelatedPair(copyA, copyB, this.relSpec);
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		if (relSpec != null) {
			sb.append(relSpec.toString());
		}
		sb.append("// CLASS A\n");
		sb.append(classA.toString());
		sb.append("// CLASS B\n");
		sb.append(classB.toString());
		return sb.toString();
	}
}
