package edu.uiuc.astgen.classrelationships;

import org.eclipse.jdt.core.dom.TypeDeclaration;

/**
 * Wraps another relationship specification and reverses it
 * Note that this class does not actually care if the wrapped {@link RelationshipSpecification} is reversible.
 * 
 */
public class ReversedRelationship implements RelationshipSpecification {
	private RelationshipSpecification innerSpec;
	
	public ReversedRelationship(RelationshipSpecification innerSpec) {
		this.innerSpec = innerSpec;
	}
	
	public RelationshipSpecification getInnerSpecification() {
		return innerSpec;
	}
	
	public RelationshipBuilder getBuilder() {
		return new RelationshipBuilder() {
			public void build(TypeDeclaration a, TypeDeclaration b) {
				innerSpec.getBuilder().build(b, a); // reverse the parameters of the call
			}
		};
	}

	public String getDescription() {
		return innerSpec.getDescription();
	}

	public RelationshipDirection getDirection() {
		return innerSpec.getDirection().getReverse();
	}

	public Reversibility getReversibility() {
		return innerSpec.getReversibility();
	}
	
	@Override
	public String toString() {
		return getDescription() + ' ' + getDirection();
	}

}
