package edu.uiuc.astgen.fieldreferences;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
import org.testorrery.Chain;
import org.testorrery.FilteredCompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.classrelationships.InheritanceType;
import edu.uiuc.astgen.classrelationships.RelationshipSpecification;
import edu.uiuc.astgen.classrelationships.RelationshipSpecificationGenerator;

public class DualClass2FieldsChildDeclarationGenerator extends FilteredCompositeGenerator<CompilationUnit>{
	private IGenerator<FieldReferenceType> fieldRefTypeGen;
	private RelationshipSpecificationGenerator relationshipSpecificationGen;
	private IGenerator<FieldDeclaration> fieldDeclGen;
	
	private String className1;
	private String className2;
	private String fieldName2;
	private static AST ast = ASTNodeGeneratorBase.ast();
	
	public DualClass2FieldsChildDeclarationGenerator(IGenerator<FieldDeclaration> fieldDeclGen, String className1, String className2, String fieldName2) {
		this.fieldDeclGen = fieldDeclGen;
		relationshipSpecificationGen = new RelationshipSpecificationGenerator();
		relationshipSpecificationGen.setLocationAll();
		
		InheritanceType[] inheritanceTypes = new InheritanceType[1];
		inheritanceTypes[0] = InheritanceType.SUBCLASS;
		//relationshipSpecificationGen.setInheritanceAll();
		relationshipSpecificationGen.setInheritance(inheritanceTypes);
		this.className1 = className1;
		this.className2 = className2;
		this.fieldName2 = fieldName2;
	}

	@Override
	protected boolean iterationIsValid() {
		RelationshipSpecification relSpec = relationshipSpecificationGen.current();
		if (relSpec.toString().indexOf("SEPARATE UNRELATED") > 0 ){
			System.out.println("iteration is not valid");
			return false;
		}	
		return true;	
	}

	@Override
	protected CompilationUnit generateCurrent() {
		RelationshipSpecification relSpec = relationshipSpecificationGen.current();
		//TypedFieldExpressionSpec expressionSpec = expressionSpecGen.current();
		FieldReferenceType fieldRefType = fieldRefTypeGen.current();
		//FieldDeclaration fieldDecl = (FieldDeclaration)ASTNode.copySubtree(ast, expressionSpec.getFieldDecl());
		FieldDeclaration fieldDecl = fieldDeclGen.current();
		
		// build first class
		TypeDeclaration classA = ast.newTypeDeclaration();
		classA.setName(ast.newSimpleName(className1));
		classA.modifiers().add(ast.newModifier(ModifierKeyword.PUBLIC_KEYWORD)); // public since class A is the main type in the compilation unit
		
		// build second class
		TypeDeclaration classB = ast.newTypeDeclaration();
		classB.setName(ast.newSimpleName(className2));
		
		//ArrayList<FieldDeclaration> listFieldDecl = listFieldDeclGen.current();		
		//classA.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(0)));
		//if (listFieldDecl.size() == 2) classB.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(1)));

		// add field declaration to A
		classA.bodyDeclarations().add(fieldDecl);

		VariableDeclarationFragment vdf = (VariableDeclarationFragment) fieldDecl.fragments().get(0);
		SimpleName fieldName =  (SimpleName) ASTNode.copySubtree(ast, vdf.getName());

		// build field reference (i.e. field, this.field, super.field, etc.)
		Expression fieldRef = fieldRefType.getBuilder().build(fieldName, classA);
		VariableDeclarationFragment vdf2 = ast.newVariableDeclarationFragment();
		vdf2.setName(ast.newSimpleName(fieldName2));		
		vdf2.setInitializer(fieldRef);
		FieldDeclaration fieldDecl2 = ast.newFieldDeclaration(vdf2);
		fieldDecl2.setType((Type) ASTNode.copySubtree(ast, fieldDecl.getType()));
		//System.out.println("fieldRed: "+ fieldRef);
				
		classB.bodyDeclarations().add(fieldDecl2);
		// link the classes together
		relSpec.getBuilder().build(classA, classB);		
		
		CompilationUnit cu = ast.newCompilationUnit();
		cu.types().add(classA);
		if (classB.getParent() == null) {
			cu.types().add(classB);
		}
		
		return cu;
		
	}

	@Override
	protected IIterationLink getInitializedIterator() {	
		fieldRefTypeGen = new Chain<FieldReferenceType>(FieldReferenceType.values());
		return LinkCreator.createCombinatoricLink(
				relationshipSpecificationGen,
				fieldRefTypeGen,
				fieldDeclGen);
	}


	public void setRelationshipSpecificationGen(
			RelationshipSpecificationGenerator relationshipSpecificationGen) {
		this.relationshipSpecificationGen = relationshipSpecificationGen;
	}

}
