package edu.uiuc.astgen.fieldreferences;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
import org.testorrery.FilteredCompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.classrelationships.RelationshipSpecification;
import edu.uiuc.astgen.classrelationships.RelationshipSpecificationGenerator;

public class DualClass2FieldsParentDeclarationGenerator extends FilteredCompositeGenerator<CompilationUnit>{
	private IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen;	
	private RelationshipSpecificationGenerator relationshipSpecificationGen;
	
	private String className1;
	private String className2;
	private static AST ast = ASTNodeGeneratorBase.ast();
	
	public DualClass2FieldsParentDeclarationGenerator(String className1, String className2, IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen) {
		this.listFieldDeclGen = listFieldDeclGen;
		relationshipSpecificationGen = new RelationshipSpecificationGenerator();
		relationshipSpecificationGen.setLocationAll();
		relationshipSpecificationGen.setInheritanceAll();
		this.className1 = className1;
		this.className2 = className2;
	}	

	@Override
	protected boolean iterationIsValid() {		
		return true;	
	}

	@Override
	protected CompilationUnit generateCurrent() {
		RelationshipSpecification relSpec = relationshipSpecificationGen.current();
		
		// build first class
		TypeDeclaration classA = ast.newTypeDeclaration();
		classA.setName(ast.newSimpleName(className1));
		classA.modifiers().add(ast.newModifier(ModifierKeyword.PUBLIC_KEYWORD)); // public since class A is the main type in the compilation unit
		
		// build second class
		TypeDeclaration classB = ast.newTypeDeclaration();
		classB.setName(ast.newSimpleName(className2));
		
		ArrayList<FieldDeclaration> listFieldDecl = listFieldDeclGen.current();		
		classA.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(0)));
		if (listFieldDecl.size() == 2) classB.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(1)));
		
		String dummyMethodName = "dummyMethod";
		String dummyVarName = "dummyVariable";	
		
		MethodDeclaration methDecl = ast.newMethodDeclaration();
		methDecl.setName(ast.newSimpleName(dummyMethodName));
		FieldDeclaration f = listFieldDecl.get(0);
		VariableDeclarationFragment vdf = (VariableDeclarationFragment) f.fragments().get(0);
		SimpleName fieldName =  (SimpleName) ASTNode.copySubtree(ast, vdf.getName());
		
				
		// link the classes together
		relSpec.getBuilder().build(classA, classB);		
		
		CompilationUnit cu = ast.newCompilationUnit();
		cu.types().add(classA);
		if (classB.getParent() == null) {
			cu.types().add(classB);
		}
		
		return cu;
		
	}

	@Override
	protected IIterationLink getInitializedIterator() {	
		return LinkCreator.createCombinatoricLink(
				relationshipSpecificationGen,
				listFieldDeclGen);
	}


	public void setRelationshipSpecificationGen(
			RelationshipSpecificationGenerator relationshipSpecificationGen) {
		this.relationshipSpecificationGen = relationshipSpecificationGen;
	}
}