package edu.uiuc.astgen.fieldreferences;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ArrayType;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.ClassInstanceCreation;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.QualifiedName;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.StringLiteral;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
import org.testorrery.Chain;
import org.testorrery.FilteredCompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.basic.GetterSetterGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.classrelationships.InheritanceType;
import edu.uiuc.astgen.classrelationships.LocationType;
import edu.uiuc.astgen.classrelationships.RelationshipSpecification;
import edu.uiuc.astgen.classrelationships.RelationshipSpecificationGenerator;

public class DualClassFieldGetterSetter extends FilteredCompositeGenerator<CompilationUnit>{
	private IGenerator<MethodDeclaration> getterSetterGen;
	private RelationshipSpecificationGenerator relationshipSpecificationGen;
	Chain<Integer> positionGen = new Chain<Integer>(1, 2);
	Chain<MainMethodDeclaration> methodDeclarationChain = new Chain<MainMethodDeclaration>(new SimpleDeclaration());
	private IGenerator<Modifier> fieldVisibilityModifierGen;
	
	//private IGenerator<? extends Type> typeGen;
	
	private String className1;
	private String className2;
	private String fieldName;
	private static AST ast = ASTNodeGeneratorBase.ast();

	public DualClassFieldGetterSetter(String className1, String className2, String fieldName) {
		relationshipSpecificationGen = new RelationshipSpecificationGenerator();
		InheritanceType[] inheritanceTypes = new InheritanceType[1];
		inheritanceTypes[0] = InheritanceType.SUBCLASS;
		relationshipSpecificationGen.setInheritance(inheritanceTypes);
		
		LocationType[] locationTypes = new LocationType[2];
		locationTypes[0] = LocationType.INNER;
		locationTypes[1] = LocationType.SEPARATE;
		relationshipSpecificationGen.setLocation(locationTypes);
		
		this.fieldVisibilityModifierGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, (Modifier.ModifierKeyword)null);
		this.className1 = className1;
		this.className2 = className2;
		this.fieldName = fieldName;
	}	

	@Override
	protected boolean iterationIsValid() {		
		return true;
	}

	@Override
	protected CompilationUnit generateCurrent() {
		MethodDeclaration getterSetter = getterSetterGen.current();
		RelationshipSpecification relSpec = relationshipSpecificationGen.current();
		MainMethodDeclaration mainMet = methodDeclarationChain.current();
		Modifier fieldModifier = fieldVisibilityModifierGen.current();
		Integer position = positionGen.current();
		//Type fieldType = typeGen.current();
		
		// build first class
		TypeDeclaration classA = ast.newTypeDeclaration();
		classA.setName(ast.newSimpleName(className1));
		classA.modifiers().add(ast.newModifier(ModifierKeyword.PUBLIC_KEYWORD)); // public since class A is the main type in the compilation unit
		
		// build second class
		TypeDeclaration classB = ast.newTypeDeclaration();
		classB.setName(ast.newSimpleName(className2));
		
		VariableDeclarationFragment vdf = ast.newVariableDeclarationFragment();
		vdf.setName(ast.newSimpleName(fieldName));
		FieldDeclaration fieldDecl = ast.newFieldDeclaration(vdf);
		GetterSetterGenerator getSetGen = (GetterSetterGenerator) getterSetterGen;		
		fieldDecl.setType(getSetGen.getTypeGen().current());
		if (fieldModifier != null)
			fieldDecl.modifiers().add(fieldModifier);
		classA.bodyDeclarations().add(ASTNode.copySubtree(ast, fieldDecl));		
		
		// Main method
		MethodDeclaration mainMethod = mainMet.build(className2, getSetGen.getTypeGen().current(), getterSetter.getName(), position, relSpec.toString());
		classA.bodyDeclarations().add(ASTNode.copySubtree(ast, mainMethod));
		if (position == 1) {
			classA.bodyDeclarations().add(ASTNode.copySubtree(ast, getterSetter));			
		}	
		else { 
			classB.bodyDeclarations().add(ASTNode.copySubtree(ast, getterSetter));
		}		
	
		// link the classes together
		System.out.println("relSpec: "+ relSpec);
		relSpec.getBuilder().build(classA, classB);		
		
		CompilationUnit cu = ast.newCompilationUnit();
		cu.types().add(classA);
		if (classB.getParent() == null) {
			cu.types().add(classB);
		}
		
		return cu;

	}

	@Override
	protected IIterationLink getInitializedIterator() {	
		return LinkCreator.createCombinatoricLink(				
				relationshipSpecificationGen,		
				fieldVisibilityModifierGen,
				getterSetterGen,
				methodDeclarationChain,
				positionGen);
	}


	public void setRelationshipSpecificationGen(RelationshipSpecificationGenerator relationshipSpecificationGen) {
		this.relationshipSpecificationGen = relationshipSpecificationGen;
	}

	public void setGetterSetterGen(IGenerator<MethodDeclaration> getterSetterGen) {
		this.getterSetterGen = getterSetterGen;
	}
	
	//
	private static abstract class MainMethodDeclaration {
		public static AST ast = ASTNodeGeneratorBase.ast();
		abstract MethodDeclaration build(String className, Type type, SimpleName methodName, int position,  String relationType);
		
	}
	
	private class SimpleDeclaration extends MainMethodDeclaration {
		@Override
		MethodDeclaration build(String className, Type type, SimpleName methodName, int position, String relationType) {
			
			MethodDeclaration mainMethod = ast.newMethodDeclaration();
			mainMethod.setName(ast.newSimpleName("main"));
			mainMethod.modifiers().add(ast.newModifier(ModifierKeyword.PUBLIC_KEYWORD));
			mainMethod.modifiers().add(ast.newModifier(ModifierKeyword.STATIC_KEYWORD));
			SingleVariableDeclaration svd = ast.newSingleVariableDeclaration();
			svd.setName(ast.newSimpleName("args"));
			ArrayType arrayString = ast.newArrayType(ast.newSimpleType(ast.newName("String")));
			svd.setType(arrayString);
			mainMethod.parameters().add(svd);
			
			// This part should change depending on the relationType
			VariableDeclarationStatement vds = null;
			if (relationType.indexOf("INNER B_TO_A") == 0){
				VariableDeclarationFragment vdf2 = ast.newVariableDeclarationFragment();
				vdf2.setName(ast.newSimpleName(className2.toLowerCase()));
				
				ClassInstanceCreation classInstA = ast.newClassInstanceCreation();
				classInstA.setType(ast.newSimpleType(ast.newName(className1)));
				
				ClassInstanceCreation classInstB = ast.newClassInstanceCreation();
				classInstB.setType(ast.newSimpleType(ast.newName(className2)));
				classInstB.setExpression(classInstA);
				vdf2.setInitializer(classInstB);
				vds = ast.newVariableDeclarationStatement(vdf2);
				vds.setType(ast.newSimpleType(ast.newName(className2)));
			} else if (relationType.indexOf("SEPARATE UNRELATED") == 0){
				VariableDeclarationFragment vdf2 = ast.newVariableDeclarationFragment();
				vdf2.setName(ast.newSimpleName(className2.toLowerCase()));
				ClassInstanceCreation classInst = ast.newClassInstanceCreation();
				classInst.setType(ast.newSimpleType(ast.newName(className2)));
				vdf2.setInitializer(classInst);
				vds = ast.newVariableDeclarationStatement(vdf2);
				vds.setType(ast.newSimpleType(ast.newName(className2)));				
			} 
			
			MethodInvocation methInv = ast.newMethodInvocation();
			Expression initializer =null;
			methInv.setName((SimpleName) ASTNode.copySubtree(ast, methodName));
			if (type.toString().equals("int") || type.toString().equals("Integer")){
				initializer = ast.newNumberLiteral("10");
			} else if (type.toString().equals("boolean") || type.toString().equals("Boolean")) {
				initializer = ast.newBooleanLiteral(true);
			} else if (type.toString().equals("String")) {
				StringLiteral s = ast.newStringLiteral();
				s.setEscapedValue("\"abcd\"");
				initializer = s;
			}	
						
			SimpleName s = ast.newSimpleName(className2.toLowerCase());
			methInv.setExpression(s);
			ExpressionStatement es = null;
			
			if (methodName.getIdentifier().startsWith("set")){
				methInv.arguments().add(initializer);
				es = ast.newExpressionStatement(methInv);
			}	
			else {
				QualifiedName qualifiedName = ast.newQualifiedName(ast.newName("System"), ast.newSimpleName("out"));
				MethodInvocation methodInv = ast.newMethodInvocation();
				methodInv.setExpression(qualifiedName);
				methodInv.setName(ast.newSimpleName("println"));
				methodInv.arguments().add(methInv);
				es = ast.newExpressionStatement(methodInv);
			}
			
			
			Block methBody = ast.newBlock();		
			methBody.statements().add((VariableDeclarationStatement)ASTNode.copySubtree(ast, vds));
			methBody.statements().add((ExpressionStatement)ASTNode.copySubtree(ast, es));
			mainMethod.setBody(methBody);
			return mainMethod;
		}
	}

	public void setFieldVisibilityModifierGen(
			IGenerator<Modifier> fieldVisibilityModifierGen) {
		this.fieldVisibilityModifierGen = fieldVisibilityModifierGen;
	}

}
