package edu.uiuc.astgen.fieldreferences;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
import org.testorrery.FilteredCompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.classrelationships.RelationshipSpecification;
import edu.uiuc.astgen.classrelationships.RelationshipSpecificationGenerator;
import edu.uiuc.astgen.nestedexpressions.NestedExpressionBuilder;
import edu.uiuc.astgen.nestedexpressions.NestedExpressionBuilderGenerator;
import edu.uiuc.astgen.nestedexpressions.NotNestedBuilder;
import edu.uiuc.astgen.nestedexpressions.ParenthesesBuilder;
import edu.uiuc.astgen.statements.ExpressionStatementBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilderGenerator;

public class DualClassTwoFieldReferenceGenerator extends FilteredCompositeGenerator<CompilationUnit>{
	private IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen;
	private IGenerator<NestedExpressionBuilder> methodExprGen;
	private IGenerator<NestedExpressionBuilder> nestedExprGen;
	private IGenerator<StatementWithExpressionBuilder> exprInStmtGen;	
	private RelationshipSpecificationGenerator relationshipSpecificationGen;
	
	private String className1;
	private String className2;
	private static AST ast = ASTNodeGeneratorBase.ast();
	
	public DualClassTwoFieldReferenceGenerator(String className1, String className2, IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen) {
		this.listFieldDeclGen = listFieldDeclGen;
		relationshipSpecificationGen = new RelationshipSpecificationGenerator();
		relationshipSpecificationGen.setLocationAll();
		relationshipSpecificationGen.setInheritanceAll();
		this.className1 = className1;
		this.className2 = className2;
	}	

	@Override
	protected boolean iterationIsValid() {
		NestedExpressionBuilder methExprBuilder = methodExprGen.current();
		NestedExpressionBuilder nestedExprBuilder = nestedExprGen.current();
		StatementWithExpressionBuilder exprInStmtBuilder = exprInStmtGen.current();		
		
		if (nestedExprBuilder.isAssignment()) {
			// cannot assign to a method
			return false;
		}
		if (!nestedExprBuilder.applicableTo(methExprBuilder.outputType())) {
			// if output type of nested expression does not match the input type of the outer expression
			return false;
		}
		if (!exprInStmtBuilder.applicableTo(nestedExprBuilder.outputType())) {
			// if output type of outer expression does not match the input type of the statement
			return false;
		}
		if (methExprBuilder instanceof ParenthesesBuilder 
				&& exprInStmtBuilder instanceof ExpressionStatementBuilder) {
			// disallow parenthesized expression statement
			// (expr);
			return false;
		}
		if (methExprBuilder instanceof NotNestedBuilder 
				&& nestedExprBuilder instanceof ParenthesesBuilder
				&& exprInStmtBuilder instanceof ExpressionStatementBuilder) {
			// disallow parenthesized expression statement
			// (expr);
			return false;
		}
		return true;	
	}

	@Override
	protected CompilationUnit generateCurrent() {
		NestedExpressionBuilder methodExprBuilder = methodExprGen.current();
		NestedExpressionBuilder nestedExprBuilder = nestedExprGen.current();
		StatementWithExpressionBuilder exprInStmtBuilder = exprInStmtGen.current();
		RelationshipSpecification relSpec = relationshipSpecificationGen.current();
		
		// build first class
		TypeDeclaration classA = ast.newTypeDeclaration();
		classA.setName(ast.newSimpleName(className1));
		classA.modifiers().add(ast.newModifier(ModifierKeyword.PUBLIC_KEYWORD)); // public since class A is the main type in the compilation unit
		
		// build second class
		TypeDeclaration classB = ast.newTypeDeclaration();
		classB.setName(ast.newSimpleName(className2));
		
		ArrayList<FieldDeclaration> listFieldDecl = listFieldDeclGen.current();		
		classA.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(0)));
		if (listFieldDecl.size() == 2) classB.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(1)));
		
		String dummyMethodName = "dummyMethod";
		String dummyVarName = "dummyVariable";	
		
		MethodDeclaration methDecl = ast.newMethodDeclaration();
		methDecl.setName(ast.newSimpleName(dummyMethodName));
		FieldDeclaration f = listFieldDecl.get(0);
		VariableDeclarationFragment vdf = (VariableDeclarationFragment) f.fragments().get(0);
		SimpleName fieldName =  (SimpleName) ASTNode.copySubtree(ast, vdf.getName());
		
		// Creating an expression that use field f
		Expression methExpr = methodExprBuilder.build(fieldName, ast.newSimpleName("unneeded")); 
		Expression nestedExpr = nestedExprBuilder.build(methExpr, ast.newSimpleName(dummyVarName));
		Statement stmt = exprInStmtBuilder.build(nestedExpr);
		methDecl.setBody(ast.newBlock());
		List methStmts = methDecl.getBody().statements();
		
		// Declaring a local variable
		VariableDeclarationFragment varDeclFrag = ast.newVariableDeclarationFragment();
		varDeclFrag.setName(ast.newSimpleName(dummyVarName));
		varDeclFrag.setInitializer(ast.newNullLiteral());
		VariableDeclarationStatement dummyVarDecl = ast.newVariableDeclarationStatement(varDeclFrag);
		dummyVarDecl.setType((Type)ASTNode.copySubtree(ast, ast.newSimpleType(ast.newSimpleName("Integer"))));
		
		//methStmts.add(dummyVarDecl);						
		methStmts.add(stmt);
		classA.bodyDeclarations().add(methDecl);
				
		// link the classes together
		relSpec.getBuilder().build(classA, classB);		
		
		CompilationUnit cu = ast.newCompilationUnit();
		cu.types().add(classA);
		if (classB.getParent() == null) {
			cu.types().add(classB);
		}
		
		return cu;
		
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		if (methodExprGen == null) {			
			methodExprGen = NestedExpressionBuilderGenerator.parenNoParenAllOperandTypes();
		}
		if (nestedExprGen == null) {
			nestedExprGen = NestedExpressionBuilderGenerator.allOperandTypes();
		}
		if (exprInStmtGen == null) {
			exprInStmtGen = StatementWithExpressionBuilderGenerator.allExpressionTypes();
		}	
	
		return LinkCreator.createCombinatoricLink(
				relationshipSpecificationGen,
				listFieldDeclGen,
				methodExprGen,
				nestedExprGen,
				exprInStmtGen);
	}

	public void setExprInStmtGen(
			IGenerator<StatementWithExpressionBuilder> exprInStmtGen) {
		this.exprInStmtGen = exprInStmtGen;
	}

	public void setMethodExprGen(IGenerator<NestedExpressionBuilder> methodExprGen) {
		this.methodExprGen = methodExprGen;
	}

	public void setNestedExprGen(IGenerator<NestedExpressionBuilder> nestedExprGen) {
		this.nestedExprGen = nestedExprGen;
	}

	public void setRelationshipSpecificationGen(
			RelationshipSpecificationGenerator relationshipSpecificationGen) {
		this.relationshipSpecificationGen = relationshipSpecificationGen;
	}

}
