package edu.uiuc.astgen.fieldreferences;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ArrayType;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.ClassInstanceCreation;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.InfixExpression;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.QualifiedName;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.StringLiteral;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
import org.testorrery.Chain;
import org.testorrery.FilteredCompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class SingleClass2FieldsOptInitializedGenerator extends FilteredCompositeGenerator<TypeDeclaration>{
	private IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen;
	Chain<MainMethodDeclaration> methodDeclarationChain = new Chain<MainMethodDeclaration>(new SimpleDeclaration());
	
	private String className;
	private static AST ast = ASTNodeGeneratorBase.ast();
	
	public SingleClass2FieldsOptInitializedGenerator(String className, IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen) {
		this.className = className;
		this.listFieldDeclGen = listFieldDeclGen;
	}
	
	@Override
	protected boolean iterationIsValid() {
			return true;	
	}

	@Override
	protected TypeDeclaration generateCurrent() {		
		MainMethodDeclaration mainMet = methodDeclarationChain.current();
		TypeDeclaration toReturn = ast.newTypeDeclaration();
		toReturn.setName(ast.newSimpleName(className));
		ArrayList<FieldDeclaration> listFieldDecl = listFieldDeclGen.current();		
		toReturn.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(0)));
		if (listFieldDecl.size() == 2) toReturn.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(1)));
		toReturn.modifiers().add(ast.newModifier(Modifier.ModifierKeyword.PUBLIC_KEYWORD));
		
		String dummyMethodName = "dummyMethod";
		String dummyVarName = "dummyVariable";	
		// Main method
		MethodDeclaration mainMethod = mainMet.build(className, listFieldDecl);
		toReturn.bodyDeclarations().add(ASTNode.copySubtree(ast, mainMethod));
		
		
		MethodDeclaration methDecl = ast.newMethodDeclaration();
		methDecl.setName(ast.newSimpleName(dummyMethodName));
		FieldDeclaration f = listFieldDecl.get(0);
		VariableDeclarationFragment vdf = (VariableDeclarationFragment) f.fragments().get(0);
		SimpleName fieldName =  (SimpleName) ASTNode.copySubtree(ast, vdf.getName());
		
		return toReturn;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		
		return LinkCreator.createCombinatoricLink(
				listFieldDeclGen, methodDeclarationChain);
	}

	private static abstract class MainMethodDeclaration {
		public static AST ast = ASTNodeGeneratorBase.ast();
		//abstract MethodDeclaration build(String className, Type type, SimpleName methodName, int position,  String relationType);
		abstract MethodDeclaration build(String className, ArrayList<FieldDeclaration> listFields);
		
	}
	
	private class SimpleDeclaration extends MainMethodDeclaration {
		@Override
		MethodDeclaration build(String className, ArrayList<FieldDeclaration> listFields) {
			
			MethodDeclaration mainMethod = ast.newMethodDeclaration();
			mainMethod.setName(ast.newSimpleName("main"));
			mainMethod.modifiers().add(ast.newModifier(ModifierKeyword.PUBLIC_KEYWORD));
			mainMethod.modifiers().add(ast.newModifier(ModifierKeyword.STATIC_KEYWORD));
			SingleVariableDeclaration svd = ast.newSingleVariableDeclaration();
			svd.setName(ast.newSimpleName("args"));
			ArrayType arrayString = ast.newArrayType(ast.newSimpleType(ast.newName("String")));
			svd.setType(arrayString);
			mainMethod.parameters().add(svd);
			
			// This part should change depending on the relationType
			VariableDeclarationStatement vds = null;
			VariableDeclarationFragment vdf2 = ast.newVariableDeclarationFragment();
			vdf2.setName(ast.newSimpleName(className.toLowerCase()));
			ClassInstanceCreation classInst = ast.newClassInstanceCreation();
			classInst.setType(ast.newSimpleType(ast.newName(className)));
			vdf2.setInitializer(classInst);
			vds = ast.newVariableDeclarationStatement(vdf2);
			vds.setType(ast.newSimpleType(ast.newName(className)));				
			 			
			MethodInvocation methInv = ast.newMethodInvocation();
			Expression initializer =null;
						
			SimpleName s = ast.newSimpleName(className.toLowerCase());
			methInv.setExpression(s);
			ExpressionStatement es = null;
			
			SimpleName fName, gName;
			if (listFields.size() == 1) {
				FieldDeclaration field = listFields.get(0);
				VariableDeclarationFragment vdf_f = (VariableDeclarationFragment) field.fragments().get(0);
				VariableDeclarationFragment vdf_g= (VariableDeclarationFragment) field.fragments().get(1);
				fName = vdf_f.getName(); 
				gName = vdf_g.getName();
			} else {
				FieldDeclaration fieldF = listFields.get(0);
				VariableDeclarationFragment vdf_f = (VariableDeclarationFragment) fieldF.fragments().get(0);
				FieldDeclaration fieldG = listFields.get(1);
				VariableDeclarationFragment vdf_g= (VariableDeclarationFragment) fieldG.fragments().get(0);
				fName = vdf_f.getName(); 
				gName = vdf_g.getName();				
			}
			
			QualifiedName qualifiedName = ast.newQualifiedName(ast.newName("System"), ast.newSimpleName("out"));
			MethodInvocation methodInv = ast.newMethodInvocation();
			methodInv.setExpression(qualifiedName);
			methodInv.setName(ast.newSimpleName("println"));
			InfixExpression infixExp = ast.newInfixExpression();
			StringLiteral stLit = ast.newStringLiteral();
			stLit.setLiteralValue("a." + fName);
			infixExp.setLeftOperand(stLit);
			infixExp.setOperator(InfixExpression.Operator.PLUS);
			QualifiedName qualName = ast.newQualifiedName(ast.newName("a"), (SimpleName)ASTNode.copySubtree(ast, fName));
			infixExp.setRightOperand(qualName);
			StringLiteral stLit2 = ast.newStringLiteral();
			stLit2.setLiteralValue("a." + gName);
			infixExp.extendedOperands().add(stLit2);
			QualifiedName qualName2 = ast.newQualifiedName(ast.newName("a"), (SimpleName)ASTNode.copySubtree(ast, gName));
			infixExp.extendedOperands().add(qualName2);
			methodInv.arguments().add(infixExp);
			
			es = ast.newExpressionStatement(methodInv);
			
			
			
			Block methBody = ast.newBlock();		
			methBody.statements().add((VariableDeclarationStatement)ASTNode.copySubtree(ast, vds));
			methBody.statements().add((ExpressionStatement)ASTNode.copySubtree(ast, es));
			mainMethod.setBody(methBody);
			return mainMethod;
		}
	}	

}
