package edu.uiuc.astgen.fieldreferences;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
import org.testorrery.FilteredCompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.nestedexpressions.NestedExpressionBuilder;
import edu.uiuc.astgen.nestedexpressions.NestedExpressionBuilderGenerator;
import edu.uiuc.astgen.nestedexpressions.NotNestedBuilder;
import edu.uiuc.astgen.nestedexpressions.ParenthesesBuilder;
import edu.uiuc.astgen.statements.ExpressionStatementBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilderGenerator;

public class SingleClass2FieldsReferenceGenerator extends FilteredCompositeGenerator<TypeDeclaration>{
	private IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen;

	private IGenerator<NestedExpressionBuilder> methodExprGen;
	private IGenerator<NestedExpressionBuilder> nestedExprGen;
	private IGenerator<StatementWithExpressionBuilder> exprInStmtGen;
	

	private static AST ast = ASTNodeGeneratorBase.ast();
	
	public SingleClass2FieldsReferenceGenerator(IGenerator<ArrayList<FieldDeclaration>> listFieldDeclGen) {
		this.listFieldDeclGen = listFieldDeclGen;
	}
	
	@Override
	protected boolean iterationIsValid() {
		NestedExpressionBuilder methExprBuilder = methodExprGen.current();
		NestedExpressionBuilder nestedExprBuilder = nestedExprGen.current();
		StatementWithExpressionBuilder exprInStmtBuilder = exprInStmtGen.current();		
		

			if (nestedExprBuilder.isAssignment()) {
				// cannot assign to a method
				return false;
			}
			if (!nestedExprBuilder.applicableTo(methExprBuilder.outputType())) {
				// if output type of nested expression does not match the input type of the outer expression
				return false;
			}
			if (!exprInStmtBuilder.applicableTo(nestedExprBuilder.outputType())) {
				// if output type of outer expression does not match the input type of the statement
				return false;
			}
			if (methExprBuilder instanceof ParenthesesBuilder 
					&& exprInStmtBuilder instanceof ExpressionStatementBuilder) {
				// disallow parenthesized expression statement
				// (expr);
				return false;
			}
			if (methExprBuilder instanceof NotNestedBuilder 
					&& nestedExprBuilder instanceof ParenthesesBuilder
					&& exprInStmtBuilder instanceof ExpressionStatementBuilder) {
				// disallow parenthesized expression statement
				// (expr);
				return false;
			}
			return true;	
	}

	@Override
	protected TypeDeclaration generateCurrent() {
		NestedExpressionBuilder methodExprBuilder = methodExprGen.current();
		NestedExpressionBuilder nestedExprBuilder = nestedExprGen.current();
		StatementWithExpressionBuilder exprInStmtBuilder = exprInStmtGen.current();
		
		TypeDeclaration toReturn = ast.newTypeDeclaration();
		toReturn.setName(ast.newSimpleName("A"));
		ArrayList<FieldDeclaration> listFieldDecl = listFieldDeclGen.current();		
		toReturn.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(0)));
		if (listFieldDecl.size() == 2) toReturn.bodyDeclarations().add((FieldDeclaration) ASTNode.copySubtree(ast, listFieldDecl.get(1)));
		toReturn.modifiers().add(ast.newModifier(Modifier.ModifierKeyword.PUBLIC_KEYWORD));
		
		String dummyMethodName = "dummyMethod";
		String dummyVarName = "dummyVariable";	
		
		MethodDeclaration methDecl = ast.newMethodDeclaration();
		methDecl.setName(ast.newSimpleName(dummyMethodName));
		FieldDeclaration f = listFieldDecl.get(0);
		VariableDeclarationFragment vdf = (VariableDeclarationFragment) f.fragments().get(0);
		SimpleName fieldName =  (SimpleName) ASTNode.copySubtree(ast, vdf.getName());
		
		// Creating an expression that use field f
		Expression methExpr = methodExprBuilder.build(fieldName, ast.newSimpleName("unneeded")); 
		Expression nestedExpr = nestedExprBuilder.build(methExpr, ast.newSimpleName(dummyVarName));
		Statement stmt = exprInStmtBuilder.build(nestedExpr);
		methDecl.setBody(ast.newBlock());
		List methStmts = methDecl.getBody().statements();
		
		// Declaring a local variable
		VariableDeclarationFragment varDeclFrag = ast.newVariableDeclarationFragment();
		varDeclFrag.setName(ast.newSimpleName(dummyVarName));
		varDeclFrag.setInitializer(ast.newNullLiteral());
		VariableDeclarationStatement dummyVarDecl = ast.newVariableDeclarationStatement(varDeclFrag);
		dummyVarDecl.setType((Type)ASTNode.copySubtree(ast, ast.newSimpleType(ast.newSimpleName("Integer"))));
		
		//methStmts.add(dummyVarDecl);						
		methStmts.add(stmt);
		toReturn.bodyDeclarations().add(methDecl);
		
		return toReturn;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		if (methodExprGen == null) {					
			methodExprGen = NestedExpressionBuilderGenerator.parenNoParenAllOperandTypes();			
		}
		if (nestedExprGen == null) {
			nestedExprGen = NestedExpressionBuilderGenerator.allOperandTypes();
		}
		if (exprInStmtGen == null) {
			exprInStmtGen = StatementWithExpressionBuilderGenerator.allExpressionTypes();
		}		
		
		return LinkCreator.createCombinatoricLink(
				listFieldDeclGen,
				methodExprGen,
				nestedExprGen,
				exprInStmtGen);
	}

}
