package edu.uiuc.astgen.fieldreferences;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.StringLiteral;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
import org.testorrery.Chain;
import org.testorrery.FilteredCompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.examples.Pair;
import org.testorrery.examples.PairGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.classrelationships.InheritanceType;
import edu.uiuc.astgen.classrelationships.RelationshipSpecification;
import edu.uiuc.astgen.classrelationships.RelationshipSpecificationGenerator;

public class TripleClass2FieldsChildDeclarationGenerator extends FilteredCompositeGenerator<CompilationUnit>{
	private IGenerator<FieldReferenceType> fieldRefTypeGen;
	private IGenerator<FieldDeclaration> fieldDeclGen;
	private RelationshipSpecificationGenerator relationshipSpecificationGen;
		
	Chain<ThirdClassDeclaration> classDeclarationGen = new Chain<ThirdClassDeclaration>(new InsideFirstClass(), new OutsideFirstClass());
	
	PairGenerator<RelationshipSpecification, ThirdClassDeclaration> pairGen; 
			
	
	private String className1;
	private String className2;
	private String className3;
	private String fieldName2;
	private static AST ast = ASTNodeGeneratorBase.ast();
	
	public TripleClass2FieldsChildDeclarationGenerator(IGenerator<FieldDeclaration> fieldDeclGen, String className1, String className2, String className3, String fieldName2) {
		this.fieldDeclGen = fieldDeclGen;
		relationshipSpecificationGen = new RelationshipSpecificationGenerator();
		relationshipSpecificationGen.setLocationAll();
		
		InheritanceType[] inheritanceTypes = new InheritanceType[1];
		inheritanceTypes[0] = InheritanceType.SUBCLASS;
		//relationshipSpecificationGen.setInheritanceAll();
		relationshipSpecificationGen.setInheritance(inheritanceTypes);
		
		pairGen = new PairGenerator<RelationshipSpecification, ThirdClassDeclaration>(relationshipSpecificationGen, classDeclarationGen);
		this.className1 = className1;
		this.className2 = className2;
		this.className3 = className3;
		this.fieldName2 = fieldName2;
	}

	@Override
	protected boolean iterationIsValid() {		
		return true;	
	}

	@Override
	protected CompilationUnit generateCurrent() {
		//RelationshipSpecification relSpec = relationshipSpecificationGen.current();
		Pair<RelationshipSpecification, ThirdClassDeclaration> pairRel_Lit = pairGen.current();
		//TypedFieldExpressionSpec expressionSpec = expressionSpecGen.current();
		FieldReferenceType fieldRefType = fieldRefTypeGen.current();
		//FieldDeclaration fieldDecl = (FieldDeclaration)ASTNode.copySubtree(ast, expressionSpec.getFieldDecl());
		FieldDeclaration fieldDecl = fieldDeclGen.current();
		
		// build first class
		TypeDeclaration classA = ast.newTypeDeclaration();
		classA.setName(ast.newSimpleName(className1));
		classA.modifiers().add(ast.newModifier(ModifierKeyword.PUBLIC_KEYWORD)); // public since class A is the main type in the compilation unit
		
		// build second class
		TypeDeclaration classB = ast.newTypeDeclaration();
		classB.setName(ast.newSimpleName(className2));
		
		// build third class
		TypeDeclaration classC = ast.newTypeDeclaration();
		classC.setName(ast.newSimpleName(className3));		

		// add field declaration to A
		classA.bodyDeclarations().add(fieldDecl);
		
		classC.bodyDeclarations().add(ASTNode.copySubtree(ast, fieldDecl));

		VariableDeclarationFragment vdf = (VariableDeclarationFragment) fieldDecl.fragments().get(0);
		SimpleName fieldName =  (SimpleName) ASTNode.copySubtree(ast, vdf.getName());

		// build field reference (i.e. field, this.field, super.field, etc.)
		Expression initializer =null;
		if (fieldRefType.toString().equals("NO_REFERENCE_TYPE")){			
			if (fieldDecl.getType().toString().equals("int") || fieldDecl.getType().toString().equals("Integer")){
				initializer = ast.newNumberLiteral("10");			
			} else if (fieldDecl.getType().toString().equals("boolean") || fieldDecl.getType().toString().equals("Boolean")) {
				initializer = ast.newBooleanLiteral(true);
			} else if (fieldDecl.getType().toString().equals("String")) {
				StringLiteral s = ast.newStringLiteral();
				s.setEscapedValue("\"abcd\"");
				initializer = s;
			} else {
				initializer = ast.newNullLiteral();
			}			
		} else {
			initializer = fieldRefType.getBuilder().build(fieldName, classA);
		}
		//Expression fieldRef = fieldRefType.getBuilder().build(fieldName, classA);
		
		VariableDeclarationFragment vdf2 = ast.newVariableDeclarationFragment();
		vdf2.setName(ast.newSimpleName(fieldName2));		
		vdf2.setInitializer(initializer);
		FieldDeclaration fieldDecl2 = ast.newFieldDeclaration(vdf2);
		fieldDecl2.setType((Type) ASTNode.copySubtree(ast, fieldDecl.getType()));
		//System.out.println("fieldRed: "+ fieldRef);
				
		classB.bodyDeclarations().add(fieldDecl2);
		// link the classes together
		pairRel_Lit.left.getBuilder().build(classA, classB);
		RelationshipSpecification relSpec = pairRel_Lit.left;
		System.out.println(relSpec);
		pairRel_Lit.right.build(classA, classC, classB);
		//relSpec.getBuilder().build(classA, classB);		
		
		CompilationUnit cu = ast.newCompilationUnit();
		cu.types().add(classA);
		if (classB.getParent() == null) {
			cu.types().add(classB);
		}
		if (classC.getParent() == null) {
			cu.types().add(classC);
		}
		
		return cu;
		
	}

	@Override
	protected IIterationLink getInitializedIterator() {	
		fieldRefTypeGen = new Chain<FieldReferenceType>(FieldReferenceType.values());
		return LinkCreator.createCombinatoricLink(
				//relationshipSpecificationGen,
				pairGen,
				fieldRefTypeGen,
				fieldDeclGen);
	}


	public void setRelationshipSpecificationGen(
			RelationshipSpecificationGenerator relationshipSpecificationGen) {
		this.relationshipSpecificationGen = relationshipSpecificationGen;
	}

	private static abstract class ThirdClassDeclaration {
		public static AST ast = ASTNodeGeneratorBase.ast();
		abstract void build(TypeDeclaration classA, TypeDeclaration classC, TypeDeclaration classB);
		
	}
	
	private class InsideFirstClass extends ThirdClassDeclaration {
		@Override
		void build(TypeDeclaration classA, TypeDeclaration classC, TypeDeclaration classB) {			
			classC.modifiers().add(ast.newModifier(Modifier.ModifierKeyword.STATIC_KEYWORD));
			classA.bodyDeclarations().add(classC);

			classB.setSuperclassType(ast.newSimpleType(ast.newName(className1 + "." + className3)));
		}
	}		

	private class OutsideFirstClass extends ThirdClassDeclaration {
		@Override
		void build(TypeDeclaration classA, TypeDeclaration classC, TypeDeclaration classB) {
			classB.setSuperclassType(ast.newSimpleType(ast.newName(className3)));
		}
	}		
	

}
