package edu.uiuc.astgen.methodrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public abstract class MethodRelationshipBuilder {
	protected static final AST ast = ASTNodeGeneratorBase.ast();
	
	public MethodDeclaration build(
			String newMethodName, 
			MethodDeclaration originalMethDecl, 
			TypeDeclaration typeDecl) {
		MethodDeclaration methDecl = ast.newMethodDeclaration();
		methDecl.setBody(ast.newBlock());
		buildMethod(newMethodName, originalMethDecl, methDecl);
		typeDecl.bodyDeclarations().add(methDecl);	
		
		return methDecl;
	}
	
	protected abstract void buildMethod(
			String newMethodName,
			MethodDeclaration originalMethDecl,
			MethodDeclaration newMethDecl);

	public boolean willRemoveParam() {
		return false;
	}

	public boolean requiresSuper() {
		return false;
	}
}

class SeparateBuilder extends MethodRelationshipBuilder {
	@Override
	protected void buildMethod(String newMethodName, MethodDeclaration originalMethDecl, MethodDeclaration newMethDecl) {
		newMethDecl.setName(ast.newSimpleName(newMethodName));
		newMethDecl.setReturnType2((Type)ASTNode.copySubtree(ast, originalMethDecl.getReturnType2()));
	}
}

class OverrideBuilder extends MethodRelationshipBuilder {
	@Override
	protected void buildMethod(String newMethodName, MethodDeclaration originalMethDecl, MethodDeclaration newMethDecl) {
		newMethDecl.setName(ast.newSimpleName(originalMethDecl.getName().toString()));
		for (Object paramObj : originalMethDecl.parameters()) {
			newMethDecl.parameters().add(ASTNode.copySubtree(ast, (ASTNode)paramObj));
		}
		newMethDecl.setReturnType2((Type)ASTNode.copySubtree(ast, originalMethDecl.getReturnType2()));
	}
	@Override
	public boolean requiresSuper() {
		return true;
	}
}

class OverloadAddBuilder extends MethodRelationshipBuilder {
	@Override
	protected void buildMethod(String newMethodName, MethodDeclaration originalMethDecl, MethodDeclaration newMethDecl) {
		new OverrideBuilder().buildMethod(newMethodName, originalMethDecl, newMethDecl);
		
		SingleVariableDeclaration newParam = ast.newSingleVariableDeclaration();
		newParam.setName(ast.newSimpleName("newArg"));
		newParam.setType(ast.newSimpleType(ast.newSimpleName("Object")));			
		newMethDecl.parameters().add(newParam);
	}
}

class OverloadRemoveBuilder extends MethodRelationshipBuilder {
	@Override
	protected void buildMethod(String newMethodName, MethodDeclaration originalMethDecl, MethodDeclaration newMethDecl) {
		new OverrideBuilder().buildMethod(newMethodName, originalMethDecl, newMethDecl);
		
		if (originalMethDecl.parameters().size() == 0) return;
		newMethDecl.parameters().remove(newMethDecl.parameters().size() - 1);		
	}
	
	@Override
	public boolean willRemoveParam() {
		return true;
	}
}
