package edu.uiuc.astgen.methodrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclarationStatement;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public abstract class StatementInBlockBuilder {
	public abstract Block build(Statement... innerStatements);
	/**
	 * This builder requires any external variables referenced in
	 * innerStatements to be declared final
	 */
	public abstract boolean requiresFinal();
	
	protected static final AST ast = ASTNodeGeneratorBase.ast();
}

class InBlockBuilder extends StatementInBlockBuilder {
	@Override
	public Block build(Statement... innerStatements) {
		Block block = ast.newBlock();
		for (Statement innerStatement : innerStatements) {
			block.statements().add(innerStatement);
		}
		return block;
	}
	@Override
	public boolean requiresFinal() {
		return false;
	}
}

class InSubBlockBuilder extends StatementInBlockBuilder {
	@Override
	public Block build(Statement... innerStatements) {
		Block block = ast.newBlock();
		block.statements().add(new InBlockBuilder().build(innerStatements));
		return block;
	}
	@Override
	public boolean requiresFinal() {
		return false;
	}
}

class InLocalClassBuilder extends StatementInBlockBuilder {
	@Override
	public Block build(Statement... innerStatements) {
		Block methBody = new InBlockBuilder().build(innerStatements);
		
		MethodDeclaration methDecl = ast.newMethodDeclaration();
		methDecl.setName(ast.newSimpleName("localM"));
		methDecl.setReturnType2(ast.newPrimitiveType(PrimitiveType.VOID));
		methDecl.setBody(methBody);
		
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName("Local"));	
		typeDecl.bodyDeclarations().add(methDecl);
		
		TypeDeclarationStatement localDecl = ast.newTypeDeclarationStatement(typeDecl);
		
		Block block = ast.newBlock();
		block.statements().add(localDecl);
		
		return block;
	}
	@Override
	public boolean requiresFinal() {
		return true;
	}
}
