package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;
import org.testorrery.Chain;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

public class AssignmentBuilderGenerator extends BuilderGenerator {
	private Chain<BinaryOperandLocation> locationGen;
	private Chain<Assignment.Operator> operatorGen; // determined by super.operandType
	
	public AssignmentBuilderGenerator(OperandType operandType) {
		super(operandType);
	}
	
	/**
	 * Set the possible locations where the target expression can be
	 */
	public void setLocations(BinaryOperandLocation... locations) {
		this.locationGen = new Chain<BinaryOperandLocation>(locations);
	}
	public void setLocationsAll() {
		setLocations(BinaryOperandLocation.values());	
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		if (locationGen == null) {
			setLocationsAll();	
		}
		
		operatorGen = new Chain<Assignment.Operator>(getOperandType().getAssignmentOperators());
		
		return LinkCreator.createCombinatoricLink(locationGen, operatorGen);
	}
	
	@Override
	protected NestedExpressionBuilder generateCurrent() {	
		final BinaryOperandLocation location = locationGen.current();
		final Assignment.Operator operator = operatorGen.current();
		return new NestedExpressionBuilder() {
			public Expression build(Expression innerExpression, Name dummyVariable) {
				
				BinaryOperandLocation.LocationPair locations = 
					location.getLocations(innerExpression, dummyVariable);
				
				// build assignment expression
				Assignment assignExpr = getDefaultAST().newAssignment();
				assignExpr.setLeftHandSide(locations.leftHandSide);
				assignExpr.setRightHandSide(locations.rightHandSide);
				assignExpr.setOperator(operator);
				return assignExpr;
			}
			@Override
			public boolean applicableTo(OperandType operandType) {
				return operandType == getOperandType();
			}
			@Override
			public boolean isAssignment() {
				return location != BinaryOperandLocation.RIGHT_ONLY;
			}
			@Override
			public OperandType outputType() {
				return getOperandType();
			}
		};
	}
}