package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

/**
 * Enum of possible locations at which a target expression can be located in a binary expression.
 */
public enum BinaryOperandLocation {
	LEFT_ONLY(new LeftBuilder()),
	RIGHT_ONLY(new RightBuilder()),
	BOTH_SIDES(new BothBuilder());
	
	public static class LocationPair {
		public final Expression leftHandSide;
		public final Expression rightHandSide;
		public LocationPair(Expression lhs, Expression rhs) {
			this.leftHandSide = lhs;
			this.rightHandSide = rhs;
		}
	}
	
	private static interface LocationBuilder {
		LocationPair getLocations(Expression innerExpression, Name dummyVariable);
	}
	
	private static class LeftBuilder implements LocationBuilder {
		public LocationPair getLocations(Expression innerExpression, Name dummyVariable) {
			return new LocationPair(innerExpression, dummyVariable);
		}
	}
	
	private static class RightBuilder implements LocationBuilder {
		public LocationPair getLocations(Expression innerExpression, Name dummyVariable) {
			return new LocationPair(dummyVariable, innerExpression);
		}
	}
	
	private static class BothBuilder implements LocationBuilder {
		public LocationPair getLocations(Expression innerExpression, Name dummyVariable) {
			return new LocationPair(
					innerExpression, 
					(Expression)ASTNode.copySubtree(ASTNodeGeneratorBase.ast(), innerExpression));
		}
	}
	
	private LocationBuilder builder;
	private BinaryOperandLocation(LocationBuilder builder) {
		this.builder = builder;
	}
	public LocationPair getLocations(Expression innerExpression, Name dummyVariable) {
		return builder.getLocations(innerExpression, dummyVariable);
	}
}