package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.InfixExpression;
import org.eclipse.jdt.core.dom.Name;
import org.testorrery.Chain;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

public class InfixBuilderGenerator extends BuilderGenerator {
	private Chain<BinaryOperandLocation> locationGen;
	private Chain<InfixExpression.Operator> operatorGen; // determined by super.operandType
	
	public InfixBuilderGenerator(OperandType operandType) {
		super(operandType);
	}
	
	/**
	 * Set the possible locations where the target expression can be
	 */
	public void setLocations(BinaryOperandLocation... locations) {
		this.locationGen = new Chain<BinaryOperandLocation>(locations);
	}
	public void setLocationsAll() {
		setLocations(BinaryOperandLocation.values());	
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		if (locationGen == null) {
			setLocationsAll();	
		}
		
		operatorGen = new Chain<InfixExpression.Operator>(getOperandType().getInfixOperators());
		
		return LinkCreator.createCombinatoricLink(locationGen, operatorGen);
	}
	
	@Override
	protected NestedExpressionBuilder generateCurrent() {	
		final InfixExpression.Operator operator = operatorGen.current();
		final BinaryOperandLocation location = locationGen.current();
		return new NestedExpressionBuilder() {
			public Expression build(Expression innerExpression, Name dummyVariable) {
				BinaryOperandLocation.LocationPair locations = 
					location.getLocations(innerExpression, dummyVariable);
				
				// build Infix expression
				InfixExpression infixExpr = getDefaultAST().newInfixExpression();
				infixExpr.setLeftOperand(locations.leftHandSide);
				infixExpr.setRightOperand(locations.rightHandSide);
				infixExpr.setOperator(operator);
				return infixExpr;
			}
			public boolean applicableTo(OperandType operandType) {
				return operandType == getOperandType();
			}
			@Override
			public OperandType outputType() {
				if (operator == InfixExpression.Operator.EQUALS 
						|| operator == InfixExpression.Operator.NOT_EQUALS
						|| operator == InfixExpression.Operator.LESS
						|| operator == InfixExpression.Operator.LESS_EQUALS
						|| operator == InfixExpression.Operator.GREATER
						|| operator == InfixExpression.Operator.GREATER_EQUALS
						|| operator == InfixExpression.Operator.NOT_EQUALS) {
					return OperandType.BOOLEAN;
				}
				return getOperandType();
			}
		};
	}
}