package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;

/**
 * Interface for a builder that consumes an expression and a dummyVariable and creates
 *  another expression containing the given expression.
 */
public abstract class NestedExpressionBuilder {
	/**
	 * Builds an expression containging the given expression
	 * @param innerExpression The expression to nest within the given expression
	 * @param dummyVariable A dummy variable defined elsewhere in the scope that is used to ensure expressions are compileable
	 * @return An expression containing the given expression
	 */
	public abstract Expression build(Expression innerExpression, Name dummyVariable);	
	public abstract boolean applicableTo(OperandType operandType);
	public abstract OperandType outputType();

	public boolean isAssignment() {
		return false;
	}
}
