package edu.uiuc.astgen.nestedexpressions;

import org.testorrery.Chain;
import org.testorrery.IGenerator;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

public class NestedExpressionBuilderGenerator extends BuilderGenerator {
	private Chain<NestedExpressionBuilder> builderSeq;	
	
	public NestedExpressionBuilderGenerator(OperandType operandType) {
		super(operandType);
	}
	
	public static IGenerator<NestedExpressionBuilder> allOperandTypes() {
		Chain<NestedExpressionBuilder> builderChain = new Chain<NestedExpressionBuilder>();
		builderChain.add(new NestedExpressionBuilderGenerator(OperandType.VOID));
		builderChain.add(new NestedExpressionBuilderGenerator(OperandType.NUMERIC));
		builderChain.add(new NestedExpressionBuilderGenerator(OperandType.BOOLEAN));
		builderChain.add(new NestedExpressionBuilderGenerator(OperandType.OBJECT));
		return builderChain;
	}
	
	public static IGenerator<NestedExpressionBuilder> parenNoParenAllOperandTypes() {
		Chain<NestedExpressionBuilder> builderChain = new Chain<NestedExpressionBuilder>();
		builderChain.add(new NotNestedBuilder(OperandType.VOID));
		builderChain.add(new NotNestedBuilder(OperandType.NUMERIC));
		builderChain.add(new NotNestedBuilder(OperandType.BOOLEAN));
		builderChain.add(new NotNestedBuilder(OperandType.OBJECT));
		builderChain.add(new ParenthesesBuilder(OperandType.VOID));
		builderChain.add(new ParenthesesBuilder(OperandType.NUMERIC));
		builderChain.add(new ParenthesesBuilder(OperandType.BOOLEAN));
		builderChain.add(new ParenthesesBuilder(OperandType.OBJECT));
		return builderChain;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		builderSeq = new Chain<NestedExpressionBuilder>();
		
		builderSeq.add(new NotNestedBuilder(getOperandType()));	
		builderSeq.add(new ParenthesesBuilder(getOperandType()));
		if (getOperandType() != OperandType.VOID) {
			builderSeq.add(new AssignmentBuilderGenerator(getOperandType()));
			builderSeq.add(new ConditionalBuilderGenerator(getOperandType()));	
			builderSeq.add(new InfixBuilderGenerator(getOperandType()));
			builderSeq.add(new PrefixBuilderGenerator(getOperandType()));
			builderSeq.add(new PostfixBuilderGenerator(getOperandType()));
			// TODO: instanceof
			// TODO: cast
		}
		
		return LinkCreator.createCombinatoricLink(builderSeq);
	}
	
	@Override
	protected NestedExpressionBuilder generateCurrent() {
		return builderSeq.current();
	}

}
