package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;
import org.testorrery.CompositeGenerator;
import org.testorrery.IGenerator;
import org.testorrery.Literal;
import org.testorrery.link.IIterationLink;
import org.testorrery.link.LinkCreator;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class NestedExpressionGenerator extends CompositeGenerator<Expression> {
	private IGenerator<? extends Expression> targetExpressionGen;
	private IGenerator<? extends Name> dummyVariableGen;
	
	private OperandType operandType;		
	private IGenerator<NestedExpressionBuilder> builderGen; // depends on operandType
	
	public void setTargetExpressionGen(IGenerator<? extends Expression> targetExpressionGen) {
		this.targetExpressionGen = targetExpressionGen;
	}
	
	public void setDummyVariableGen(IGenerator<? extends Name> dummyVariableGen) {
		this.dummyVariableGen = dummyVariableGen;
	}
	
	public void setOperandType(OperandType operandType) {
		this.operandType = operandType;
	}
		
	@Override
	protected IIterationLink getInitializedIterator() {		
		if (targetExpressionGen == null) {
			targetExpressionGen = new Literal<Expression>(ast.newSimpleName("target"));
		}
		if (dummyVariableGen == null) {
			dummyVariableGen = new Literal<Name>(ast.newSimpleName("dummy"));
		}
		if (operandType== null) {
			operandType = OperandType.OBJECT;
		}
		
		builderGen = new NestedExpressionBuilderGenerator(operandType);
				
		return LinkCreator.createLinkFromExpression("$0 * ($1 + $2)", builderGen, targetExpressionGen, dummyVariableGen);
	}
	
	AST ast = ASTNodeGeneratorBase.ast();
	
	@Override
	protected Expression generateCurrent() {
		Expression targetExpression = (Expression)ASTNode.copySubtree(ast, targetExpressionGen.current());
		Name dummyVariable = (Name)ASTNode.copySubtree(ast, dummyVariableGen.current());
		return builderGen.current().build(targetExpression, dummyVariable);
	}
}
