package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldDeclaration;

import edu.uiuc.astgen.methodrelationships.SingleClassMethodReferenceGenerator;

/**
 * Holds a type-correct field reference expression and other metadata.
 * 
 * This TypedFieldExpressionSpec stuff is a nasty hack to get expressions to be type-correct.
 * See {@link SingleClassMethodReferenceGenerator#iterationIsValid for a better way to do it.
 */
public class TypedFieldExpressionSpec {
	private final FieldDeclaration fieldDecl;
	private final NestedExpressionBuilder expressionBuilder;
	private final Expression initializingExpression;
	
	public TypedFieldExpressionSpec(
			final FieldDeclaration fieldDecl, 
			final NestedExpressionBuilder expressionBuilder, 
			final Expression initializingExpression) {
		this.fieldDecl = fieldDecl;
		this.expressionBuilder = expressionBuilder;
		this.initializingExpression = initializingExpression;
	}

	public NestedExpressionBuilder getExpressionBuilder() {
		return expressionBuilder;
	}

	public FieldDeclaration getFieldDecl() {
		return fieldDecl;
	}

	public Expression getInitializingExpression() {
		return initializingExpression;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("Field: ").append(fieldDecl.toString());
		sb.append("Initializing Expression: ").append(initializingExpression.toString()).append('\n');
		sb.append("Builder: ").append(expressionBuilder.toString());
		return sb.toString();
	}
	
}
