package edu.uiuc.astgen.test.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.junit.Test;

import edu.uiuc.astgen.basic.AssignmentExpressionGenerator;
import edu.uiuc.astgen.basic.BooleanLiteralGenerator;
import edu.uiuc.astgen.basic.ClassGenerator;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationFragmentGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

import org.testorrery.GeneratorTester;

public class AssignmentExpressionGeneratorTest {

	@Test() public void testBasicFunctionality() throws Exception {
		AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
		SimpleNameGenerator nameGen = new SimpleNameGenerator("class1");
		ModifierGenerator visModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator classModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.ABSTRACT_KEYWORD);
		ClassGenerator classGen = new ClassGenerator(nameGen);
		classGen.setVisibilityModifierGen(visModGen);
		classGen.setClassModifierGen(classModGen);
		
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN,
				PrimitiveType.BYTE);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);
		classGen.addBodyDeclarationGenerator(fieldDeclGen);		
		AssignmentExpressionGenerator assignmentExpression = new AssignmentExpressionGenerator(fieldDeclGen);
		GeneratorTester.testExpectedCount(
				assignmentExpression, 
				8);				
	}
}
