package edu.uiuc.astgen.test.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.junit.Test;

import edu.uiuc.astgen.basic.AssignmentExpressionGenerator;
import edu.uiuc.astgen.basic.BlockGenerator;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.SingleVariableDeclarationGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationFragmentGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationStatementGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

import org.testorrery.GeneratorTester;


public class BlockGeneratorTest {

	@Test()
	public void basicFunctionality() {
		AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
		
		SimpleNameGenerator nameGen = new SimpleNameGenerator("theVariable");
		PrimitiveTypeGenerator typeGen = new PrimitiveTypeGenerator(PrimitiveType.INT,
				PrimitiveType.LONG);
		ModifierGenerator modGen = new ModifierGenerator((Modifier.ModifierKeyword) null,
				Modifier.ModifierKeyword.FINAL_KEYWORD);

		SingleVariableDeclarationGenerator svdg = new SingleVariableDeclarationGenerator(
				nameGen, typeGen, modGen);

		SimpleNameGenerator nameGen2 = new SimpleNameGenerator("i");
		VariableDeclarationStatementGenerator vdfg = new VariableDeclarationStatementGenerator(
				nameGen2, svdg);

		BlockGenerator blockGen = new BlockGenerator();
		blockGen.addStatementGenerator(vdfg);
		GeneratorTester.testExpectedCount(blockGen, 4);
	}
	
	@Test()
	public void expressionStatements() {
		AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
		
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);
		//classGen.addBodyDeclarationGenerator(fieldDeclGen);		
		AssignmentExpressionGenerator assignmentExpression = new AssignmentExpressionGenerator(fieldDeclGen);


		BlockGenerator blockGen = new BlockGenerator();
		blockGen.addStatementGenerator(assignmentExpression);
		GeneratorTester.testExpectedCount(blockGen, 1);
	}	
	
}
