package edu.uiuc.astgen.test.basic;

import org.junit.*;
import org.eclipse.jdt.core.dom.*;

import edu.uiuc.astgen.*;
import edu.uiuc.astgen.basic.AssignmentExpressionForVariableGenerator;
import edu.uiuc.astgen.basic.AssignmentExpressionGenerator;
import edu.uiuc.astgen.basic.BlockGenerator;
import edu.uiuc.astgen.basic.ClassGenerator;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.MethodDeclarationGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationFragmentGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

import org.testorrery.GeneratorTester;

public class ClassGeneratorTest {
	
	@Test()
	public void testDefaultConstructor() {
		ClassGenerator classGen = new ClassGenerator("class1");
		// default to narrowest definition of the generator
		GeneratorTester.testExpectedStrings(classGen, "class class1 {\n}\n");
	}
	
	@Test()
	public void testParentClass() {
		ClassGenerator classGen = new ClassGenerator("class1");
		AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
		SimpleType st = ast.newSimpleType(ast.newSimpleName("C"));
		TypeDeclaration t = ast.newTypeDeclaration();
		t.setName(ast.newSimpleName("B"));
		classGen.setParentClass(st);
		// default to narrowest definition of the generator
		GeneratorTester.testExpectedStrings(classGen, "class class1 extends C {\n}\n");
	}	
	
	@Test()
	public void testBasicFunctionality() {
		SimpleNameGenerator nameGen = new SimpleNameGenerator("class1");
		ModifierGenerator visModGen = new ModifierGenerator(
				Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator classModGen = new ModifierGenerator(
				(Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.ABSTRACT_KEYWORD);
		ASTNodeChain<MethodDeclaration> methodSequenceGen = new ASTNodeChain<MethodDeclaration>();
		
		MethodDeclarationGenerator methodGen;		
		methodGen = new MethodDeclarationGenerator(new SimpleNameGenerator("method1"));
		methodGen.setReturnTypeGenerator(new PrimitiveTypeGenerator(PrimitiveType.INT));
		methodSequenceGen.add(methodGen);
		
		methodGen = new MethodDeclarationGenerator(new SimpleNameGenerator("method2"));
		methodGen.setReturnTypeGenerator(new PrimitiveTypeGenerator(PrimitiveType.LONG));
		methodSequenceGen.add(methodGen);
		
		ClassGenerator classGen = new ClassGenerator(nameGen);
		classGen.setVisibilityModifierGen(visModGen);
		classGen.setClassModifierGen(classModGen);
		classGen.addBodyDeclarationGenerator(methodSequenceGen);
		
		GeneratorTester.testExpectedStrings(
				classGen, 
				"public class class1 {\n  public int method1(){\n  }\n}\n",
				"private class class1 {\n  public int method1(){\n  }\n}\n",
				"public abstract class class1 {\n  public int method1(){\n  }\n}\n",
				"private abstract class class1 {\n  public int method1(){\n  }\n}\n",
				"public class class1 {\n  public long method2(){\n  }\n}\n",
				"private class class1 {\n  public long method2(){\n  }\n}\n",
				"public abstract class class1 {\n  public long method2(){\n  }\n}\n",
				"private abstract class class1 {\n  public long method2(){\n  }\n}\n");		
	}
	
	@Test() public void testAdd1InternalClass() {
		SimpleNameGenerator nameGen = new SimpleNameGenerator("class1");
		SimpleNameGenerator nameInternal = new SimpleNameGenerator("internal1");
		ModifierGenerator visModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator classModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.ABSTRACT_KEYWORD);		
		
		ClassGenerator classGen = new ClassGenerator(nameGen);
		classGen.setVisibilityModifierGen(visModGen);
		classGen.setClassModifierGen(classModGen);
		
		ModifierGenerator visModInternal = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD,
				(Modifier.ModifierKeyword)null);
		ModifierGenerator classModInternal = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.ABSTRACT_KEYWORD, // absence of a modifier
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);		
		
		ClassGenerator classInternal = new ClassGenerator(nameInternal);
		classInternal.setVisibilityModifierGen(visModInternal);
		classInternal.setClassModifierGen(classModInternal);
		classGen.addBodyDeclarationGenerator(classInternal);
		
		//ASTNodeGeneratorTestUtil.testBasicFunctionality(classInternal, 1 * 3 * 3);
		GeneratorTester.testExpectedCount(classGen, 1 * 2 * 2 * 9);
		
	}
	
	@Test() public void testAdd2InternalClass() {
		SimpleNameGenerator nameGen = new SimpleNameGenerator("class1");		
		ModifierGenerator visModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator classModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.ABSTRACT_KEYWORD);		
		
		ClassGenerator classGen = new ClassGenerator(nameGen);
		classGen.setVisibilityModifierGen(visModGen);
		classGen.setClassModifierGen(classModGen);
		
		//setting internal class1
		SimpleNameGenerator nameInternal = new SimpleNameGenerator("internal1");
		ModifierGenerator visModInternal = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD,
				(Modifier.ModifierKeyword)null);
		ModifierGenerator classModInternal = new ModifierGenerator((Modifier.ModifierKeyword)null);		
		
		ClassGenerator classInternal = new ClassGenerator(nameInternal);
		classInternal.setVisibilityModifierGen(visModInternal);
		classInternal.setClassModifierGen(classModInternal);
		classGen.addBodyDeclarationGenerator(classInternal);

		//setting internal class2
		SimpleNameGenerator nameInternal2 = new SimpleNameGenerator("internal2");
		ModifierGenerator visModInternal2 = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD,
				(Modifier.ModifierKeyword)null);
		ModifierGenerator classModInternal2 = new ModifierGenerator((Modifier.ModifierKeyword)null);
		
		ClassGenerator classInternal2 = new ClassGenerator(nameInternal2);
		classInternal2.setVisibilityModifierGen(visModInternal2);
		classInternal2.setClassModifierGen(classModInternal2);
		classGen.addBodyDeclarationGenerator(classInternal2);
		
		GeneratorTester.testExpectedCount(classGen, 1 * 2 * 2 * 9);
		
	}
	
	
	@Test() public void testAdd1InternalClassWith1Field() {
		SimpleNameGenerator nameGen = new SimpleNameGenerator("class1");
		SimpleNameGenerator nameInternal = new SimpleNameGenerator("internal1");
		ModifierGenerator visModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator classModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.ABSTRACT_KEYWORD);		

		ModifierGenerator visModInternal = new ModifierGenerator((Modifier.ModifierKeyword)null);
		ModifierGenerator classModInternal = new ModifierGenerator((Modifier.ModifierKeyword)null);		
		
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator visibilityModInternal = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		ModifierGenerator fieldModInternal = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.INT,
				PrimitiveType.LONG);
		PrimitiveTypeGenerator primitiveTypeInternal = new PrimitiveTypeGenerator(PrimitiveType.INT,
				PrimitiveType.LONG);
		
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		VariableDeclarationFragmentGenerator varDeclFragInternal = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theInternalField"));		
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);// 2*2*2*1= 8		
		FieldDeclarationGenerator fieldDeclInternal = new FieldDeclarationGenerator(
				visibilityModInternal, fieldModInternal, primitiveTypeInternal, varDeclFragInternal);// 2*2*2*1=8
				
		ClassGenerator classGen = new ClassGenerator(nameGen);
		classGen.setVisibilityModifierGen(visModGen);
		classGen.setClassModifierGen(classModGen); // 1*2*2=4
		classGen.addBodyDeclarationGenerator(fieldDeclGen);
		ClassGenerator classInternal = new ClassGenerator(nameInternal);
		classInternal.setVisibilityModifierGen(visModInternal);
		classInternal.setClassModifierGen(classModInternal);//1*1*1=1 
		classInternal.addBodyDeclarationGenerator(fieldDeclInternal);
		classGen.addBodyDeclarationGenerator(classInternal);
						
		//ASTNodeGeneratorTestUtil.testBasicFunctionality(classInternal, 8);
		GeneratorTester.testExpectedCount(classGen, 32 * 8);
	}
	
	@Test() public void testAddOuterClass_With_1InternalClassWith1Field() {
		SimpleNameGenerator nameGen = new SimpleNameGenerator("class1");
		SimpleNameGenerator nameInternal = new SimpleNameGenerator("internal1");
		ModifierGenerator visModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator classModGen = new ModifierGenerator((Modifier.ModifierKeyword)null);		

		ModifierGenerator visModInternal = new ModifierGenerator((Modifier.ModifierKeyword)null);
		ModifierGenerator classModInternal = new ModifierGenerator((Modifier.ModifierKeyword)null);		
		
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.INT,
				PrimitiveType.LONG);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		VariableDeclarationFragmentGenerator varDeclFragInternal = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theInternalField"));		
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);// 2*2*2*1= 8		
		FieldDeclarationGenerator fieldDeclInternal = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragInternal);// 2*2*2*1=8
				
		ClassGenerator classGen = new ClassGenerator(nameGen);
		classGen.setVisibilityModifierGen(visModGen);
		classGen.setClassModifierGen(classModGen); // 1*2*2=4
		//classGen.addBodyDeclarationGenerator(fieldDeclGen);
		ClassGenerator classInternal = new ClassGenerator(nameInternal);
		classInternal.setVisibilityModifierGen(visModInternal);
		classInternal.setClassModifierGen(classModInternal);//1*1*1=1 
		classInternal.addBodyDeclarationGenerator(fieldDeclInternal);
		classGen.addBodyDeclarationGenerator(classInternal);
						
		//ASTNodeGeneratorTestUtil.testBasicFunctionality(classInternal, 8);
		GeneratorTester.testExpectedCount(classGen, 8 );
	}	
	

	
}
