package edu.uiuc.astgen.test.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.Name;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SimpleType;
import org.eclipse.jdt.core.dom.StringLiteral;
import org.junit.Test;

import edu.uiuc.astgen.basic.ClassInstanceCreationGenerator;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.StringLiteralGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationFragmentGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;
import org.testorrery.Chain;
import org.testorrery.GeneratorTester;
import org.testorrery.IGenerator;

public class ClassInstanceCreationGeneratorTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testBasicFunctionality() {
		Chain<PrimitiveType> chainPrimitiveType = new Chain<PrimitiveType>(ast.newPrimitiveType(PrimitiveType.BYTE));		
		ClassInstanceCreationGenerator classInstanceGen = new ClassInstanceCreationGenerator(chainPrimitiveType);
		
		GeneratorTester.testExpectedCount(classInstanceGen, 1);
	}
	
	@Test
	public void DefaultClassInstanceCreation() {
		GeneratorTester.testExpectedCount(ClassInstanceCreationGenerator.getNewDefaultClassInstanceCreationGenerator(), 3);
	}	
	
	
	@Test
	public void testClassException() {
		Chain<SimpleType> chainSimpleType = new Chain<SimpleType>(ast.newSimpleType(ast.newSimpleName("Exception")), ast.newSimpleType(ast.newSimpleName("IOException")));		
		Chain<String> chainOfStrings = new Chain<String>();
		chainOfStrings.add("\"name1\"");
		chainOfStrings.add("\"name2\"");
		StringLiteralGenerator stringLiteralGen = new StringLiteralGenerator(chainOfStrings);
		ClassInstanceCreationGenerator classInstanceGen = new ClassInstanceCreationGenerator(chainSimpleType);
		classInstanceGen.addArgumentGenerator(stringLiteralGen);
		
		GeneratorTester.testExpectedCount(classInstanceGen, 4);
	}	
}
