package edu.uiuc.astgen.test.basic;

import org.junit.*;
import org.eclipse.jdt.core.dom.*;

import edu.uiuc.astgen.*;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationFragmentGenerator;

import org.testorrery.GeneratorTester;

public class FieldDeclarationGeneratorTest {
	
	@Test() 
	public void testDefaultConstructor() {
		FieldDeclarationGenerator fieldDecGen = new FieldDeclarationGenerator("theField");
		// default to narrowest definition of the generator
		GeneratorTester.testExpectedStrings(fieldDecGen, "public Object theField;\n");
	}	
	
	@Test
	public void testBasicFunctionality() {
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.INT,
				PrimitiveType.LONG);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);
		
		GeneratorTester.testExpectedStrings(
				fieldDeclGen, 
				"public int theField;\n",
				"public long theField;\n",
				"private int theField;\n",
				"private long theField;\n",
				"public static int theField;\n",
				"public static long theField;\n",
				"private static int theField;\n",
				"private static long theField;\n");
	}
	
	@Test
	public void test() {
		AST ast = AST.newAST(AST.JLS3);
		
		VariableDeclarationFragment vdf = ast.newVariableDeclarationFragment();		
		Type fieldType = ast.newPrimitiveType(PrimitiveType.INT);
		Modifier modifier = ast.newModifier(Modifier.ModifierKeyword.PUBLIC_KEYWORD);

		FieldDeclaration fd = ast.newFieldDeclaration(vdf);
		fd.setType(fieldType);
		fd.modifiers().add(modifier);
		
	}
}
