package edu.uiuc.astgen.test.basic;

import org.junit.*;
import org.eclipse.jdt.core.dom.*;

import edu.uiuc.astgen.*;
import edu.uiuc.astgen.basic.MethodDeclarationGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.SingleVariableDeclarationGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

import org.testorrery.GeneratorTester;

public class MethodDeclarationGeneratorTest {
	
	@Test() 
	public void testBasicFunctionality() {
		AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
		
		SimpleNameGenerator nameGen = new SimpleNameGenerator("theMethod");
		PrimitiveTypeGenerator returnTypeGen = new PrimitiveTypeGenerator(PrimitiveType.INT,
				PrimitiveType.LONG);
		ModifierGenerator visibilityModifierGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD,
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator methodModifierGen = new ModifierGenerator(Modifier.ModifierKeyword.ABSTRACT_KEYWORD,
				Modifier.ModifierKeyword.FINAL_KEYWORD);
		ASTNodeGeneratorBase<Block> methodBodyGen = 
			new ASTNodeLiteral<Block>(ast.newBlock()); // TODO: non-empty block?
		ASTNodeChain<SingleVariableDeclaration> paramSequence = new ASTNodeChain<SingleVariableDeclaration>(new SingleVariableDeclarationGenerator(new SimpleNameGenerator("intParam"), 
			new PrimitiveTypeGenerator(PrimitiveType.INT)),
				new SingleVariableDeclarationGenerator(new SimpleNameGenerator("longParam"), 
						new PrimitiveTypeGenerator(PrimitiveType.LONG))); // chooses either a long or int parameter

		MethodDeclarationGenerator methodGen = new MethodDeclarationGenerator(nameGen);
		methodGen.setReturnTypeGenerator(returnTypeGen);
		methodGen.setVisibilityModifierGenerator(visibilityModifierGen);
		methodGen.setMethodModifierGenerator(methodModifierGen);
		methodGen.setMethodBodyGenerator(methodBodyGen);
		methodGen.addParameterGenerator(paramSequence);
		
		GeneratorTester.testExpectedStrings(
				methodGen,
				"public abstract int theMethod(int intParam){\n}\n",
				"public abstract long theMethod(int intParam){\n}\n",
				"private abstract int theMethod(int intParam){\n}\n",
				"private abstract long theMethod(int intParam){\n}\n",
				"public final int theMethod(int intParam){\n}\n",
				"public final long theMethod(int intParam){\n}\n",
				"private final int theMethod(int intParam){\n}\n",
				"private final long theMethod(int intParam){\n}\n",
				"public abstract int theMethod(long longParam){\n}\n",
				"public abstract long theMethod(long longParam){\n}\n",
				"private abstract int theMethod(long longParam){\n}\n",
				"private abstract long theMethod(long longParam){\n}\n",
				"public final int theMethod(long longParam){\n}\n",
				"public final long theMethod(long longParam){\n}\n",
				"private final int theMethod(long longParam){\n}\n",
				"private final long theMethod(long longParam){\n}\n");
	}
	
}
