package edu.uiuc.astgen.test.basic;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.junit.Test;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.basic.MultipleFieldDeclarationGenerator;
import edu.uiuc.astgen.fieldreferences.FieldReferenceType;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;
import org.testorrery.Chain;
import org.testorrery.GeneratorTester;
import org.testorrery.IGenerator;

public class MultipleFieldDeclarationGeneratorTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testBasicFunctionality() {
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName("A"));
		
		MultipleFieldDeclarationGenerator multiFieldDeclGen = new MultipleFieldDeclarationGenerator("f", "g", typeDecl, true);
		multiFieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				//ast.newPrimitiveType(PrimitiveType.INT)
				//ast.newSimpleType(ast.newSimpleName("String")),
				//ast.newSimpleType(ast.newSimpleName("Integer")),
				//ast.newSimpleType(ast.newSimpleName("Boolean")),
				ast.newSimpleType(ast.newSimpleName("Object"))));
		
		
		GeneratorTester.testExpectedCount(
				multiFieldDeclGen,				
				1 * 2 * 4);
	}
	
	@Test
	public void testBasicFunctionality_WithClassDeclaration() {
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName("A"));
		IGenerator<FieldReferenceType> fieldRefTypeGen = new Chain<FieldReferenceType>(
				FieldReferenceType.NEW_INSTANCE, 
				FieldReferenceType.QUALIFIED_THIS, 
				FieldReferenceType.THIS,
				FieldReferenceType.UNQUALIFIED,
				FieldReferenceType.SUPER);		
		MultipleFieldDeclarationGenerator multiFieldDeclGen = new MultipleFieldDeclarationGenerator("field1", "field2", typeDecl, false);
		multiFieldDeclGen.setFieldRefTypeGen(fieldRefTypeGen);
		multiFieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newSimpleType(ast.newSimpleName("Integer"))));
		
		
		GeneratorTester.testExpectedCount(
				multiFieldDeclGen,				
				1 * 2 * 5);
	}	
}
