package edu.uiuc.astgen.test.basic;

import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.SwitchCase;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.junit.Test;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.basic.AssignmentExpressionGenerator;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.ModifierGenerator;
import edu.uiuc.astgen.basic.NumberLiteralGenerator;
import edu.uiuc.astgen.basic.PrimitiveTypeGenerator;
import edu.uiuc.astgen.basic.SimpleNameGenerator;
import edu.uiuc.astgen.basic.SingleVariableDeclarationGenerator;
import edu.uiuc.astgen.basic.SwitchCaseGenerator;
import edu.uiuc.astgen.basic.SwitchStatementGenerator;
import edu.uiuc.astgen.basic.VariableDeclarationFragmentGenerator;

import org.testorrery.GeneratorTester;

public class SwitchStatementGeneratorTest {
	@Test
	public void DefaulSwitchStatement() {
		GeneratorTester.testExpectedCount(SwitchStatementGenerator.getNewDefaultSwitchStatementGenerator(), 2);
	}
		
	@Test
	public void basicFunctionality_OneSwitchCase() {
		SimpleNameGenerator simpleNameGen = new SimpleNameGenerator("month");
		NumberLiteralGenerator numberGen = new NumberLiteralGenerator(2, PrimitiveType.BYTE);
		SwitchCaseGenerator switchCaseGen = new SwitchCaseGenerator(numberGen);
		
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN,
				PrimitiveType.BYTE);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);		
		AssignmentExpressionGenerator assignmentExpression = new AssignmentExpressionGenerator(fieldDeclGen);		
		
		ASTNodeChain<SwitchCase> statementsSequence = new ASTNodeChain<SwitchCase>();		
		statementsSequence.add(switchCaseGen);
		ASTNodeChain<ExpressionStatement> varStatementsSequence = new ASTNodeChain<ExpressionStatement>();		
		varStatementsSequence.add(assignmentExpression);	
		
		SwitchStatementGenerator switchStatementGen = new SwitchStatementGenerator(simpleNameGen);
		switchStatementGen.addStatement(switchCaseGen);
		switchStatementGen.addStatement(varStatementsSequence);
		GeneratorTester.testExpectedCount(switchStatementGen, 16);
	}	
	
	@Test
	public void TwoSwitchCases() {
		SimpleNameGenerator simpleNameGen = new SimpleNameGenerator("month");
		NumberLiteralGenerator numberGen = new NumberLiteralGenerator(2, PrimitiveType.BYTE);
		SwitchCaseGenerator switchCaseGen = new SwitchCaseGenerator(numberGen);
		
		NumberLiteralGenerator numberGen2 = new NumberLiteralGenerator(1, PrimitiveType.BYTE);
		SwitchCaseGenerator switchCaseGen2 = new SwitchCaseGenerator(numberGen2);
		
		
		ModifierGenerator visibilityModGen = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN,
				PrimitiveType.BYTE);
		VariableDeclarationFragmentGenerator varDeclFragGen = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator(
				visibilityModGen, fieldModGen, primitiveTypeGen, varDeclFragGen);		
		AssignmentExpressionGenerator assignmentExpression = new AssignmentExpressionGenerator(fieldDeclGen);		

		ModifierGenerator visibilityModGen2 = new ModifierGenerator(Modifier.ModifierKeyword.PUBLIC_KEYWORD, 
				Modifier.ModifierKeyword.PRIVATE_KEYWORD);
		ModifierGenerator fieldModGen2 = new ModifierGenerator((Modifier.ModifierKeyword)null,
				Modifier.ModifierKeyword.STATIC_KEYWORD);
		PrimitiveTypeGenerator primitiveTypeGen2 = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN,
				PrimitiveType.BYTE);
		VariableDeclarationFragmentGenerator varDeclFragGen2 = new VariableDeclarationFragmentGenerator(new SimpleNameGenerator("theField"));
		
		FieldDeclarationGenerator fieldDeclGen2 = new FieldDeclarationGenerator(
				visibilityModGen2, fieldModGen2, primitiveTypeGen2, varDeclFragGen2);		
		AssignmentExpressionGenerator assignmentExpression2 = new AssignmentExpressionGenerator(fieldDeclGen2);		
		
		ASTNodeChain<SwitchCase> statementsSequence = new ASTNodeChain<SwitchCase>();		
		statementsSequence.add(switchCaseGen);

		ASTNodeChain<ExpressionStatement> varStatementsSequence = new ASTNodeChain<ExpressionStatement>();		
		varStatementsSequence.add(assignmentExpression);
		
		ASTNodeChain<SwitchCase> statementsSequence2 = new ASTNodeChain<SwitchCase>();		
		statementsSequence.add(switchCaseGen2);

		ASTNodeChain<ExpressionStatement> varStatementsSequence2 = new ASTNodeChain<ExpressionStatement>();		
		varStatementsSequence2.add(assignmentExpression2);		
		
		SwitchStatementGenerator switchStatementGen = new SwitchStatementGenerator(simpleNameGen);
		switchStatementGen.addStatement(switchCaseGen);
		switchStatementGen.addStatement(varStatementsSequence);
		switchStatementGen.addStatement(switchCaseGen2);
		switchStatementGen.addStatement(varStatementsSequence2);
		GeneratorTester.testExpectedCount(switchStatementGen, 128);
	}	
}
