package edu.uiuc.astgen.test.classrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.junit.*;

import edu.uiuc.astgen.classrelationships.BuilderUtil;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

public class BuilderUtilTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testGetDummyMethods() {		
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName("A"));
		
		// existing method should not affect numbering
		MethodDeclaration existingMethod = ast.newMethodDeclaration();
		existingMethod.setName(ast.newSimpleName(BuilderUtil.getDummyMethodName(typeDecl.getName().toString()) + "_ABC"));		
		typeDecl.bodyDeclarations().add(existingMethod);
		
		// add dummy methods
		typeDecl.bodyDeclarations().add(BuilderUtil.getDummyMethod(typeDecl));
		typeDecl.bodyDeclarations().add(BuilderUtil.getDummyMethod(typeDecl));
		typeDecl.bodyDeclarations().add(BuilderUtil.getDummyMethod(typeDecl));
		
		System.out.println(typeDecl.toString());
		Assert.assertEquals(
				"class A {\n  void dummyMethod_A_ABC();\n  public void dummyMethod_A(){\n  }\n  public void dummyMethod_A1(){\n  }\n  public void dummyMethod_A2(){\n  }\n}\n", 
				typeDecl.toString());
	}
	
	@Test
	public void testGetDummyFields() {		
		
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName("A"));
		
		String defaultFieldName = BuilderUtil.getDummyFieldName(typeDecl.getName().toString());		
		
		// existing field should not affect numbering
		VariableDeclarationFragment varDeclFrag = ast.newVariableDeclarationFragment();
		varDeclFrag.setName(ast.newSimpleName(defaultFieldName + "_ABC"));
		FieldDeclaration existingField = ast.newFieldDeclaration(varDeclFrag);	
		typeDecl.bodyDeclarations().add(existingField);
		
		// numbered multiple fragments should affect numbering
		VariableDeclarationFragment multiFrag1 = ast.newVariableDeclarationFragment();
		multiFrag1.setName(ast.newSimpleName(defaultFieldName));
		VariableDeclarationFragment multiFrag2 = ast.newVariableDeclarationFragment();
		multiFrag2.setName(ast.newSimpleName(defaultFieldName + 1));
		FieldDeclaration multiDecl = ast.newFieldDeclaration(multiFrag1);		
		multiDecl.fragments().add(multiFrag2);
		typeDecl.bodyDeclarations().add(multiDecl);
		
		// add dummy fields
		typeDecl.bodyDeclarations().add(BuilderUtil.getDummyField(typeDecl));
		typeDecl.bodyDeclarations().add(BuilderUtil.getDummyField(typeDecl));
		// ...and one with an initializer		
		typeDecl.bodyDeclarations().add(BuilderUtil.getDummyField(typeDecl, ast.newNullLiteral()));
		
		System.out.println(typeDecl.toString());
		Assert.assertEquals(
				"class A {\n  int dummyField_A_ABC;\n  int dummyField_A, dummyField_A1;\n  public int dummyField_A2;\n  public int dummyField_A3;\n  public int dummyField_A4=null;\n}\n", 
				typeDecl.toString());
	}
}
