package edu.uiuc.astgen.test.classrelationships;

import org.junit.*;

import edu.uiuc.astgen.basic.ClassGenerator;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.classrelationships.LocationType;
import edu.uiuc.astgen.classrelationships.RelationshipSpecificationGenerator;
import edu.uiuc.astgen.classrelationships.RelatedPairGenerator;
import edu.uiuc.astgen.classrelationships.InheritanceType;
import edu.uiuc.astgen.classrelationships.ReferenceType;

import org.testorrery.GeneratorTester;

public class RelatedPairGeneratorTest {
	
	@Test
	public void testBasicFunctionality() {	
		RelationshipSpecificationGenerator crbg = new RelationshipSpecificationGenerator();
		crbg.setInheritance(InheritanceType.NO_INHERITANCE, InheritanceType.SUBCLASS);
		crbg.setLocation(LocationType.SEPARATE, LocationType.INNER);
		crbg.setReference(ReferenceType.NO_REFERENCE, ReferenceType.FIELD_TYPE);		
		crbg.setEnableReverse(false);
				
		ClassGenerator class1Gen = new ClassGenerator("A");
		class1Gen.addBodyDeclarationGenerator(new FieldDeclarationGenerator("f"));
				
		ClassGenerator class2Gen = new ClassGenerator("B");
		class2Gen.addBodyDeclarationGenerator(new FieldDeclarationGenerator("g"));
				
		RelatedPairGenerator crg = new RelatedPairGenerator();
		crg.setClassAGen(class1Gen);
		crg.setClassBGen(class2Gen);
		crg.setBuilderGen(crbg);
		
		GeneratorTester.testExpectedCount(crg, 2 * 2 * 2);
	}
	
	@Test
	public void testExhaustive() {
		InheritanceType[] allInheritanceTypes = InheritanceType.values(); 
		LocationType[] allLocationTypes = LocationType.values();
		ReferenceType[] allReferenceTypes = ReferenceType.values();		
		
		RelationshipSpecificationGenerator crbg = new RelationshipSpecificationGenerator();		
		crbg.setInheritance(allInheritanceTypes);
		crbg.setLocation(allLocationTypes);
		crbg.setReference(allReferenceTypes);
		
		crbg.setEnableReverseInheritance(true);
		crbg.setEnableReverseLocation(true); 
		crbg.setEnableReverseReference(true); 
		
		ClassGenerator classAGen = new ClassGenerator("A");
		ClassGenerator classBGen = new ClassGenerator("B");
				
		RelatedPairGenerator crg = new RelatedPairGenerator();
		crg.setClassAGen(classAGen);
		crg.setClassBGen(classBGen);
		crg.setBuilderGen(crbg);
		
		GeneratorTester.testExpectedCount(crg, 
				(2 * allInheritanceTypes.length - 1) * // normal + reverse - unreversible
				(2 * allLocationTypes.length - 1) * 
				(2 * allReferenceTypes.length - 1));
	}
}
