package edu.uiuc.astgen.test.fieldreferences;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.junit.Test;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.basic.MultipleFieldDeclarationGenerator;
import edu.uiuc.astgen.fieldreferences.DualClassTwoFieldReferenceGenerator;
import edu.uiuc.astgen.fieldreferences.FieldReferenceType;
import edu.uiuc.astgen.statements.DummyAssignmentBuilder;
import edu.uiuc.astgen.statements.ExpressionStatementBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilderGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;
import org.testorrery.Chain;
import org.testorrery.GeneratorTester;
import org.testorrery.IGenerator;

public class DualClassTwoFieldReferenceGeneratorTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testBasicFunctionality() {
		String classNameA = "A", classNameB = "B";
		TypeDeclaration typeDecl = ast.newTypeDeclaration();
		typeDecl.setName(ast.newSimpleName(classNameA));
		IGenerator<FieldReferenceType> fieldRefTypeGen = new Chain<FieldReferenceType>(
				FieldReferenceType.NEW_INSTANCE, 
				FieldReferenceType.QUALIFIED_THIS, 
				FieldReferenceType.THIS,
				FieldReferenceType.UNQUALIFIED,
				FieldReferenceType.SUPER);		
		
		MultipleFieldDeclarationGenerator multipleFieldDeclGen = new MultipleFieldDeclarationGenerator("f", "g", typeDecl, false);
		multipleFieldDeclGen.setFieldRefTypeGen(fieldRefTypeGen);
		multipleFieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newPrimitiveType(PrimitiveType.INT)));				
		
		DualClassTwoFieldReferenceGenerator typeGen = new DualClassTwoFieldReferenceGenerator(classNameA, classNameB, multipleFieldDeclGen);
		//new DummyAssignmentBuilder(), ,
		StatementWithExpressionBuilderGenerator statementExpGenerator = new StatementWithExpressionBuilderGenerator(new StatementWithExpressionBuilder[] {
				new ExpressionStatementBuilder()});
		typeGen.setExprInStmtGen(statementExpGenerator);		
		
		for (CompilationUnit cuDecl : typeGen) {
			System.out.println(cuDecl);
		}
		
		GeneratorTester.testExpectedCount(typeGen, 60);
	}
}
