package edu.uiuc.astgen.test.fieldreferences;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.junit.Test;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.basic.MultipleFieldDeclarationGenerator;
import edu.uiuc.astgen.fieldreferences.SingleClass2FieldsOptInitializedGenerator;
import edu.uiuc.astgen.nestedexpressions.NestedExpressionBuilder;
import edu.uiuc.astgen.nestedexpressions.NestedExpressionBuilderGenerator;
import edu.uiuc.astgen.nestedexpressions.OperandType;
import edu.uiuc.astgen.statements.DummyAssignmentBuilder;
import edu.uiuc.astgen.statements.ExpressionStatementBuilder;
import edu.uiuc.astgen.statements.IfTestBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilder;
import edu.uiuc.astgen.statements.StatementWithExpressionBuilderGenerator;
import edu.uiuc.astgen.statements.WhileTestBuilder;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;
import org.testorrery.Chain;
import org.testorrery.GeneratorTester;

public class SingleClass2FieldsOptInitializedGeneratorTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testBasicFunctionality() {
		TypeDeclaration typeA = ast.newTypeDeclaration();
		typeA.setName(ast.newSimpleName("A"));		
		MultipleFieldDeclarationGenerator multipleFieldDeclGen = new MultipleFieldDeclarationGenerator("f", "g", typeA, true);
		
		SingleClass2FieldsOptInitializedGenerator typeGen = new SingleClass2FieldsOptInitializedGenerator("A", multipleFieldDeclGen);
		
		for (TypeDeclaration typeDecl : typeGen) {
			System.out.println(typeDecl);
		}
		
		GeneratorTester.testExpectedCount(typeGen, 8);
	}
	
	@Test
	public void testBasicFunctionality_PrimitiveType() {
		TypeDeclaration typeA = ast.newTypeDeclaration();
		typeA.setName(ast.newSimpleName("A"));		
		MultipleFieldDeclarationGenerator multipleFieldDeclGen = new MultipleFieldDeclarationGenerator("f", "g", typeA, true);		
		multipleFieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newPrimitiveType(PrimitiveType.BYTE), ast.newPrimitiveType(PrimitiveType.BOOLEAN)));		

		SingleClass2FieldsOptInitializedGenerator typeGen = new SingleClass2FieldsOptInitializedGenerator("A", multipleFieldDeclGen);
		
		for (TypeDeclaration typeDecl : typeGen) {
			System.out.println(typeDecl);
		}
		
		GeneratorTester.testExpectedCount(typeGen, 16);
	}
	
	@Test
	public void testBasicFunctionality_With3FieldTypes() {
		TypeDeclaration typeA = ast.newTypeDeclaration();
		typeA.setName(ast.newSimpleName("A"));		
		
		MultipleFieldDeclarationGenerator multipleFieldDeclGen = new MultipleFieldDeclarationGenerator("f", "g", typeA, true);
		multipleFieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean")),
				ast.newSimpleType(ast.newSimpleName("Object"))));		
		
		SingleClass2FieldsOptInitializedGenerator typeGen = new SingleClass2FieldsOptInitializedGenerator("A", multipleFieldDeclGen);
		
		for (TypeDeclaration typeDecl : typeGen) {
			System.out.println(typeDecl);
		}
		
		GeneratorTester.testExpectedCount(typeGen, 24 );
	}	
}
