package edu.uiuc.astgen.test.fieldreferences;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.junit.*;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.fieldreferences.SingleClassFieldReferenceGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

import org.testorrery.GeneratorTester;

public class SingleClasssFieldReferenceGeneratorTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testBasicFunctionality() {
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator("theField");
		fieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean")),
				ast.newSimpleType(ast.newSimpleName("Object"))));
		
		SingleClassFieldReferenceGenerator fieldRefGen = new SingleClassFieldReferenceGenerator("A", fieldDeclGen);
		
		for (TypeDeclaration typeDecl : fieldRefGen) {
			System.out.println(typeDecl);
		}
		
		GeneratorTester.testExpectedCount(fieldRefGen, 5280);
	}
	
	@Test
	public void testVaryingFieldType() {
		FieldDeclarationGenerator fieldDeclGen = new FieldDeclarationGenerator("theField");
		fieldDeclGen.setTypeGen(new ASTNodeChain<Type>(
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean")),
				ast.newSimpleType(ast.newSimpleName("Object"))));
		
		SingleClassFieldReferenceGenerator fieldRefGen = new SingleClassFieldReferenceGenerator("A", fieldDeclGen);
		
		int i = 0;
		String typeDeclString;
		for (TypeDeclaration typeDecl : fieldRefGen) {
			typeDeclString = typeDecl.toString();
			System.out.println(i++);
			if (i % 50 == 0) {
				System.out.println(typeDeclString);
			}
			if (typeDeclString.contains("Object theField")) {
				Assert.assertFalse(typeDeclString.contains("++"));
				Assert.assertFalse(typeDeclString.contains("&&"));
			}
			if (typeDeclString.contains("Integer theField")) {
				Assert.assertFalse(typeDeclString.contains("&&"));
			}
			if (typeDeclString.contains("Boolean theField")) {
				Assert.assertFalse(typeDeclString.contains("++"));
			}
		}

	}
	
}
