package edu.uiuc.astgen.test.methodrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.junit.Test;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.ASTNodeLiteral;
import edu.uiuc.astgen.basic.FieldDeclarationGenerator;
import edu.uiuc.astgen.basic.MethodDeclarationGenerator;
import edu.uiuc.astgen.fieldreferences.DualClassFieldReferenceGenerator;
import edu.uiuc.astgen.methodrelationships.DualClassMethodParentGenerator;
import edu.uiuc.astgen.methodrelationships.SingleClassMethodReferenceGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

import org.testorrery.GeneratorTester;

public class DualClassMethodParentGeneratorTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testBasicFunctionality() {
		MethodDeclarationGenerator methDeclGen = new MethodDeclarationGenerator("m"); 
		FieldDeclarationGenerator fieldDecGen = new FieldDeclarationGenerator("theField");
		DualClassMethodParentGenerator methodRefGen = new DualClassMethodParentGenerator(fieldDecGen, methDeclGen, "A", "B");
		
		GeneratorTester.testExpectedCount(methodRefGen, 6 * 4 * 10);
		
	}
	
	@Test
	public void testIntReturnType() {
		MethodDeclarationGenerator methDeclGen = new MethodDeclarationGenerator("m");
		FieldDeclarationGenerator fieldDecGen = new FieldDeclarationGenerator("theField");
		methDeclGen.setReturnTypeGenerator(new ASTNodeLiteral<PrimitiveType>(ast.newPrimitiveType(PrimitiveType.INT)));
		
		DualClassMethodParentGenerator methodRefGen = new DualClassMethodParentGenerator(fieldDecGen, methDeclGen, "A", "B");
		
		GeneratorTester.testExpectedCount(methodRefGen, 6 * 1728 * 5);
	}	
	
	@Test
	public void testExhaustive() {
		SingleVariableDeclaration param = ast.newSingleVariableDeclaration();
		param.setName(ast.newSimpleName("arg0"));
		param.setType(ast.newSimpleType(ast.newSimpleName("Object")));
		
		ASTNodeChain<SingleVariableDeclaration> paramChain = new ASTNodeChain<SingleVariableDeclaration>();
		paramChain.add(new ASTNodeLiteral<SingleVariableDeclaration>(null)); // no parameter
		paramChain.add(param);		
		
		ASTNodeChain<Type> returnTypeChain = new ASTNodeChain<Type>(
				ast.newPrimitiveType(PrimitiveType.VOID), 
				ast.newSimpleType(ast.newSimpleName("Integer"))
				//ast.newSimpleType(ast.newSimpleName("Boolean")),
				//ast.newSimpleType(ast.newSimpleName("Object"))
				);
		
		MethodDeclarationGenerator methDeclGen = new MethodDeclarationGenerator("m"); 
		methDeclGen.addParameterGenerator(paramChain);
		methDeclGen.setReturnTypeGenerator(returnTypeChain);
		
		FieldDeclarationGenerator fieldDecGen = new FieldDeclarationGenerator("theField");
		DualClassMethodParentGenerator methodRefGen = new DualClassMethodParentGenerator(fieldDecGen, methDeclGen, "A", "B");
		
		GeneratorTester.testExpectedCount(methodRefGen, 130200);			

	}
	
}
