package edu.uiuc.astgen.test.methodrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.junit.Ignore;
import org.junit.Test;

import edu.uiuc.astgen.ASTNodeChain;
import edu.uiuc.astgen.ASTNodeLiteral;
import edu.uiuc.astgen.basic.MethodDeclarationGenerator;
import edu.uiuc.astgen.methodrelationships.SingleClassMethodReferenceGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

import org.testorrery.GeneratorTester;

public class SingleClassMethodReferenceGeneratorTest {

	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	
	@Test
	public void testBasic() {
		MethodDeclarationGenerator methDeclGen = new MethodDeclarationGenerator("m"); 
		
		SingleClassMethodReferenceGenerator scmrg = new SingleClassMethodReferenceGenerator(methDeclGen);
		
		GeneratorTester.testExpectedCount(scmrg, 8);
		
	}
	
	@Test
	public void testWithParams() {
		SingleVariableDeclaration param = ast.newSingleVariableDeclaration();
		param.setName(ast.newSimpleName("arg0"));
		param.setType(ast.newSimpleType(ast.newSimpleName("Object")));
		
		MethodDeclarationGenerator methDeclGen = new MethodDeclarationGenerator("m"); 
		methDeclGen.addParameterGenerator(new ASTNodeLiteral<SingleVariableDeclaration>(param));
		
		SingleClassMethodReferenceGenerator scmrg = new SingleClassMethodReferenceGenerator(methDeclGen);
		
		GeneratorTester.testExpectedCount(scmrg, 12);
		
	}
	
	@Test
	public void testIntReturnType() {
		MethodDeclarationGenerator methDeclGen = new MethodDeclarationGenerator("m"); 
		methDeclGen.setReturnTypeGenerator(new ASTNodeLiteral<PrimitiveType>(ast.newPrimitiveType(PrimitiveType.INT)));
		
		SingleClassMethodReferenceGenerator scmrg = new SingleClassMethodReferenceGenerator(methDeclGen);
		
		GeneratorTester.testExpectedCount(scmrg, 1728);
	}
	
	@Test
	public void testExhaustive() {
		SingleVariableDeclaration param = ast.newSingleVariableDeclaration();
		param.setName(ast.newSimpleName("arg0"));
		param.setType(ast.newSimpleType(ast.newSimpleName("Object")));
		
		ASTNodeChain<SingleVariableDeclaration> paramChain = new ASTNodeChain<SingleVariableDeclaration>();
		paramChain.add(new ASTNodeLiteral<SingleVariableDeclaration>(null)); // no parameter
		paramChain.add(param);		
		
		ASTNodeChain<Type> returnTypeChain = new ASTNodeChain<Type>(
				ast.newPrimitiveType(PrimitiveType.VOID), 
				ast.newSimpleType(ast.newSimpleName("Integer")),
				ast.newSimpleType(ast.newSimpleName("Boolean")),
				ast.newSimpleType(ast.newSimpleName("Object")));
		
		MethodDeclarationGenerator methDeclGen = new MethodDeclarationGenerator("m"); 
		methDeclGen.addParameterGenerator(paramChain);
		methDeclGen.setReturnTypeGenerator(returnTypeChain);
		
		SingleClassMethodReferenceGenerator scmrg = new SingleClassMethodReferenceGenerator(methDeclGen);		
		
		GeneratorTester.testExpectedCount(scmrg, 9540);
	}
}
