package edu.uiuc.astgen.test.nestedexpressions;

import org.junit.*;

import edu.uiuc.astgen.nestedexpressions.AssignmentBuilderGenerator;
import edu.uiuc.astgen.nestedexpressions.BinaryOperandLocation;
import edu.uiuc.astgen.nestedexpressions.BuilderGenerator;
import edu.uiuc.astgen.nestedexpressions.OperandType;

public class AssignmentBuilderGeneratorTest {	
	
	@Test
	public void objectExpressions() {
		AssignmentBuilderGenerator gen = new AssignmentBuilderGenerator(OperandType.OBJECT);
		gen.setLocationsAll();
		runTest(gen,
				"target=dummy",
				"dummy=target",
				"target=target");
	}

	@Test
	public void booleanExpressions() {
		AssignmentBuilderGenerator gen = new AssignmentBuilderGenerator(OperandType.BOOLEAN);
		gen.setLocations(BinaryOperandLocation.LEFT_ONLY);
		runTest(gen,
				"target=dummy",
				"target&=dummy",
				"target|=dummy",
				"target^=dummy");
	}
	
	@Test
	public void numberExpressions() {
		AssignmentBuilderGenerator gen = new AssignmentBuilderGenerator(OperandType.NUMERIC);
		gen.setLocations(BinaryOperandLocation.LEFT_ONLY);
		runTest(gen,
				"target=dummy",
				"target+=dummy",
				"target-=dummy",
				"target*=dummy",
				"target/=dummy",
				"target&=dummy",
				"target|=dummy",
				"target^=dummy",
				"target%=dummy",
				"target<<=dummy",
				"target>>=dummy",
				"target>>>=dummy");
	}
	
	static void runTest(BuilderGenerator gen, String...expectedExpressions) {
		BuilderTestUtil.runTest(gen, "target", "dummy", expectedExpressions);
	}
}
