package edu.uiuc.astgen.test.nestedexpressions;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;
import org.junit.Assert;

import edu.uiuc.astgen.nestedexpressions.BuilderGenerator;
import edu.uiuc.astgen.nestedexpressions.NestedExpressionBuilder;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;

public class BuilderTestUtil {

	static void runTest(BuilderGenerator gen, final String targetName, final String dummyName, final String...expectedExpressions) {
		AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
		
		Expression targetExpression;
		Name dummyVariable;		
		String generated;
		boolean found;
		int foundCount = 0;
		int i;
		for (NestedExpressionBuilder builder : gen) {
			targetExpression = ast.newSimpleName(targetName);
			dummyVariable = ast.newSimpleName(dummyName);
			
			generated = builder.build(targetExpression, dummyVariable).toString();
			System.out.println(generated);
			
			found = false;
			for (i = 0; i < expectedExpressions.length; i++) {
				if (generated.equals(expectedExpressions[i])) {
					expectedExpressions[i] = null;
					found = true;
					foundCount++;
					break;
				}
			}
			Assert.assertTrue("Duplicate or unexpected string \"" + generated + "\"", found);
		}
		Assert.assertEquals("Not all expected strings were generated", expectedExpressions.length, foundCount);
	}
	
	static void runTest(BuilderGenerator gen, String targetName, String dummyName, int expectedCount) {
		AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
		Expression targetExpression;
		Name dummyVariable;		
		String generated;
		int foundCount = 0;
		for (NestedExpressionBuilder builder : gen) {
			targetExpression = ast.newSimpleName(targetName);
			dummyVariable = ast.newSimpleName(dummyName);
			
			generated = builder.build(targetExpression, dummyVariable).toString();
			System.out.println(generated);
			
			foundCount++;
		}
		Assert.assertEquals("Not all expected builders were generated", expectedCount, foundCount);
	}
}
