package edu.uiuc.astgen.test.nestedexpressions;

import org.junit.*;

import edu.uiuc.astgen.nestedexpressions.InfixBuilderGenerator;
import edu.uiuc.astgen.nestedexpressions.BinaryOperandLocation;
import edu.uiuc.astgen.nestedexpressions.OperandType;

public class InfixBuilderGeneratorTest {	
	
	@Test
	public void objectExpressions() {
		InfixBuilderGenerator gen = new InfixBuilderGenerator(OperandType.OBJECT);
		gen.setLocationsAll();
		runTest(gen,
				"target == dummy",
				"target != dummy",
				"dummy == target",
				"dummy != target",
				"target == target",
				"target != target");
	}

	@Test
	public void booleanExpressions() {
		InfixBuilderGenerator gen = new InfixBuilderGenerator(OperandType.BOOLEAN);
		gen.setLocations(BinaryOperandLocation.LEFT_ONLY);
		runTest(gen, 
				"target == dummy",
				"target != dummy",
				"target & dummy",
				"target && dummy",
				"target || dummy",
				"target | dummy",
				"target ^ dummy");
	}
	
	@Test
	public void numberExpressions() {
		InfixBuilderGenerator gen = new InfixBuilderGenerator(OperandType.NUMERIC);
		gen.setLocations(BinaryOperandLocation.LEFT_ONLY);
		runTest(gen,
				"target == dummy",
				"target != dummy",
				"target & dummy",
				"target / dummy",
				"target > dummy",
				"target >= dummy",
				"target << dummy",
				"target < dummy",
				"target <= dummy",
				"target - dummy",
				"target | dummy",
				"target + dummy",
				"target % dummy",
				"target >> dummy",
				"target >>> dummy",
				"target * dummy",
				"target ^ dummy");
	}
	
	static void runTest(InfixBuilderGenerator gen, String...expectedExpressions) {
		BuilderTestUtil.runTest(gen, "target", "dummy", expectedExpressions);
	}
}
