package edu.uiuc.astgen.test.nestedexpressions;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
import org.junit.*;

import edu.uiuc.astgen.nestedexpressions.TypedFieldExpressionSpec;
import edu.uiuc.astgen.nestedexpressions.TypedFieldExpressionSpecGenerator;
import edu.uiuc.astgen.test.ASTNodeGeneratorTestUtil;
import org.testorrery.IGenerator;
import org.testorrery.Chain;

public class TypedFieldExpressionSpecGeneratorTest {
	AST ast = ASTNodeGeneratorTestUtil.getSampleAST();
	FieldDeclaration numericField;
	FieldDeclaration booleanField;
	FieldDeclaration objectField;
	
	@Before
	public void setUp() {
		VariableDeclarationFragment varDeclFrag = ast.newVariableDeclarationFragment();
		varDeclFrag.setName(ast.newSimpleName("theField"));		
		FieldDeclaration prototype = ast.newFieldDeclaration(varDeclFrag);
		
		numericField = (FieldDeclaration)ASTNode.copySubtree(ast, prototype);
		numericField.setType(ast.newPrimitiveType(PrimitiveType.INT));
	
		booleanField = (FieldDeclaration)ASTNode.copySubtree(ast, prototype);
		booleanField.setType(ast.newPrimitiveType(PrimitiveType.BOOLEAN));
		
		objectField = (FieldDeclaration)ASTNode.copySubtree(ast, prototype);
		objectField.setType(ast.newSimpleType(ast.newSimpleName("Object")));
	}
	
	
	@Test
	public void testNumeric() {		
		runTest(new TypedFieldExpressionSpecGenerator(numericField), true, false, true);
	}
	
	@Test
	public void testBoolean() {
		runTest(new TypedFieldExpressionSpecGenerator(booleanField), false, true, true);
	}
	
	@Test
	public void testObject() {
		runTest(new TypedFieldExpressionSpecGenerator(objectField), false, false, true);		
	}
	
	@Test
	public void testAllThree() {
		Chain<FieldDeclaration> fieldSeq = new Chain<FieldDeclaration>(numericField, booleanField, objectField);
		IGenerator<TypedFieldExpressionSpec> gen = TypedFieldExpressionSpecGenerator.newGenerator(fieldSeq);
		runTest(gen, true, true, true);		
		
	}
	
	private void runTest(
			IGenerator<TypedFieldExpressionSpec> gen, 
			boolean shouldFindNumeric, 
			boolean shouldFindBoolean, 
			boolean shouldFindObject) {
		boolean foundNumeric = false;
		boolean foundBoolean = false;
		boolean foundObject = false;
		for (TypedFieldExpressionSpec spec : gen) {
			System.out.println(spec);
			String builtExpr = spec.getExpressionBuilder().build(ast.newSimpleName("foo"), ast.newSimpleName("bar")).toString();
			System.out.println("Built Expression: " + builtExpr);
			if (builtExpr.contains("&&")) {
				foundBoolean = true;
				Assert.assertTrue(spec.getFieldDecl().toString().contains(booleanField.getType().toString()));
				Assert.assertTrue(spec.getInitializingExpression().toString().equals("false"));
			}
			if (builtExpr.contains("++")) {
				foundNumeric = true;
				Assert.assertTrue(spec.getFieldDecl().toString().contains(numericField.getType().toString()));
				Assert.assertTrue(spec.getInitializingExpression().toString().equals("0"));
			}
			if (builtExpr.contains("==")) {
				foundObject = true;
			}
			if (spec.getFieldDecl().toString().contains("Object")) {
				Assert.assertTrue(spec.getInitializingExpression().toString().equals("null"));
			}
		}
		Assert.assertEquals(shouldFindBoolean, foundBoolean);
		Assert.assertEquals(shouldFindNumeric, foundNumeric);
		Assert.assertEquals(shouldFindObject, foundObject);
		
	}
}
